function [values] = fourbar_co(r1,r2,r3,r4,rc,beta,theta3,sigma,theta1)

% This function analyzes a four-bar linkage when the coupler is the 
% driving link.  The input values are:

%r1      = length of vector 1 (frame)
%r2      = length of vector 2 (crank)
%r3      = length of vector 3 (coupler)
%r4      = length of vector 4 (slider offset)
%rc      = length of the coupler2
%beta    = the angle between the coupler and the coupler2
%theta3  = coupler angle (degrees)
%sigma   = +1 or -1.  Identifies assembly mode

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

% the first bushing position is in the origin (0,0)

%values (1:4)   = vector lengths
%values (5:8)   = vector angles (degrees)
%values (9:10) = x,y components of position of crank pin (point Q)
%values (11:12) = x,y components of position of follower pin (point P)
%values (13:14) = the second bushing position
%values (15:16) = the coupler2 position
%values (17)    = assembly flag.  If values(29) = 0, mechanism cannot 
%                 assembled.

%convert input data
values=zeros(17,1);
r(1)=r1;
r(2)=r2;
r(3)=r3;
r(4)=r4;
theta=zeros(4,1);
theta(1)=theta1;
theta(3)=theta3;
fact=pi/180;
t1=theta(1)*fact;
t3=theta(3)*fact;
s1=sin(t1);
c1=cos(t1);
s3=sin(t3);
c3=cos(t3);
% position calculations

A=2*r(1)*r(4)*c1-2*r(3)*r(4)*c3;
B=2*r(1)*r(4)*s1-2*r(3)*r(4)*s3;
C=r(1)*r(1)+r(3)*r(3)+r(4)*r(4)-r(2)*r(2)-2*r(1)*r(3)*(c1*c3+s1*s3);

arg=B*B-C*C+A*A;
if (arg>=0)
	values(17)=1;
% Check for the denominator equal to zero

   if abs(C-A)>=(10^(-10)*(abs(C)+abs(A)))
		t4=2*atan((-B+sigma*sqrt(arg))/(C-A));
		s4=sin(t4);
		c4=cos(t4);
		t2=atan2((r(1)*s1+r(4)*s4-r(3)*s3),(r(1)*c1+r(4)*c4-r(3)*c3));
		s2=sin(t2);
		c2=cos(t2);
	elseif abs(C-A)<(10^(-10)*(abs(C)+abs(A)))
	
% If the denominator is zero, compute theta(2) first
		A=-2*r(1)*r(2)*c1+2*r(3)*r(2)*c3;
		B=-2*r(1)*r(2)*s1+2*r(3)*r(2)*s3;
		C=r(1)*r(1)+r(3)*r(3)+r(2)*r(2)-r(4)*r(4)-2*r(1)*r(3)*(c1*c3+s1*s3);
		arg=B*B-C*C+A*A;
		if (arg>=0)
			t2=2*atan((-B-sigma*sqrt(arg))/(C-A));
			s2=sin(t2);
			c2=cos(t2);
			t4=atan2((-r(1)*s1+r(2)*s2+r(3)*s3),(-r(1)*c1+r(2)*c2+r(3)*c3));
			s4=sin(t4);
			c4=cos(t4);
		end	
	end
	theta(4)=t4/fact;
	theta(2)=t2/fact;
	
	% coordinates of P and Q
	
	values(9)=r(2)*c2;
	values(10)=r(2)*s2;
	values(11)=values(9)+r(3)*c3;
   values(12)=values(10)+r(3)*s3;
	values(15) = values(9)+rc*cos(theta3*pi/180+beta);
	values(16) = values(10)+rc*sin(theta3*pi/180+beta);
%   store the second bushing position   
   values(13)=r(1)*c1;
   values(14)=r(1)*s1;
   v(1:4) = values(9:12);

end

%store results in array values
%v
for i = 1:4
	values(i)=r(i);
   values(i+4)=theta(i);
end

