function [coords] = inflection3(Ax,Ay,Ix,Iy,Astarx,Astary,Cx,Cy,Jx,Jy,ptflag,icflag)

% MATLAB m-file for inflection circle calculations when A (or Astar), pole, 
% and inflection circle are known.  The output are JA, and Astar (or A).

% Input Variables
%   Ax=X coordinate of point
%   Ay=Y coordinate of point
%   Ix=X coordinate of instant center
%   Iy=Y coordinate of instant center
%   Astarx=X coordinate of curvature of point path of A.
%   Astary=Y coordinate of curvature of point path of A.
%	Cx = x coordinate of center of inflection circle
%	Cy = y coordinate of center of inflection circle
%	Jx = x coordinate of J
% 	Jy = y coordinate of J
%	R = radius of inflection circle
%	ptflag = point flag.  If ptflag = 0, the point (A) is known and center of
%			 the path is to be found.  If ptflag = 1, the center of the path 
%			 (Astar) is known, and the point (A) is to be found.
%	icflag = inflection circle flag.  If flag=0, CX and CY are know.  If
%			 flag = 1, JX and JY are known, and the inflection circle and JA 
% 			 can be found. 	

% Output Variables 
%   coords(1) = X coordinate of JA.
%   coords(2) = y coordinate of JA.
%	coords(3) = x coordinate of J 
%	coords(4) = y coordinate of J
%	coords(5) = gamma (degrees)
%	coords(6) = radius of inflection circle
%	coords(7) = x coordinate of center of inflection circle
%	coords(8) = y coordinate of center of inflection circle
%	coords(9) = x coordinate of point A
%	coords(10) = y coordinate of point A
%	coords(11) = x coordinate of point Astar
%	coords(12) = y coordinate of point Astar

%Program

coords=zeros(1,12);
JAx=0;
JAy=0;
	
% calculations when Cx and Cy are known	
	
if icflag ==0
	rCdIx=(Cx-Ix);
	rCdIy=(Cy-Iy);
	rCdI=sqrt(rCdIx^2+rCdIy^2);
	Jx=Ix+2*rCdIx;
	Jy=Iy+2*rCdIy;
	R = abs(rCdI);
end	

% Calculations when J is known

if icflag==1
	JdI=sqrt((Jx-Ix)^2+(Jy-Iy)^2);
	R = 0.5*abs(JdI);
	Cx=Ix+(Jx-Ix)/2;
	Cy=Iy+(Jy-Iy)/2;		
end
beta=atan2((Jy-Iy), (Jx-Ix));
if beta<0; beta=beta+2*pi; end

% Make calculations when point is known

if ptflag ==0
	rAdIx=(Ax-Ix);
	rAdIy=(Ay-Iy);
	rAdI=sqrt(rAdIx^2+rAdIy^2);
	angle = atan2(rAdIy, rAdIx);
	if angle<0; angle=angle+2*pi; end
	gamma = angle-beta;
	JAdI=2*R*cos(gamma);
	JAx=Ix+JAdI*cos(angle);
	JAy=Iy+JAdI*sin(angle);
	roots= inflection1(Ax, Ay, Ix, Iy, JAx, JAy, gamma,Jx,Jy,0);
	Astarx=roots(1);
	Astary=roots(2);
end

% Make calculations when center of curvature is known

if ptflag ==1
	rAstardIx=(Astarx-Ix);
	rAstardIy=(Astary-Iy);
	rAstardI=sqrt(rAstardIx^2+rAstardIy^2);
	rho=atan2((Astary-Iy),(Astarx-Ix));
	B=2*((cos(rho)*(Ix-Cx)+sin(rho)*(Iy-Cy)));
	C=(Ix-Cx)^2+(Iy-Cy)^2-R*R;
	s1=(-B+sqrt(B^2-4*C))/2;
	s2=(-B-sqrt(B^2-4*C))/2;
	s=s1;
	if (abs(s1)<abs(s2)); s=s2; end
	JAx=Ix+s*cos(rho);
	JAy=Iy+s*sin(rho);
	JAdIx=(JAx-Ix);
	JAdIy=(JAy-Iy);
	JAdI=sqrt(JAdIx^2+JAdIy^2);
	angle = atan2(JAdIy, JAdIx);
	if angle<0; angle=angle+2*pi; end
	gamma = angle-beta;
	roots= inflection2(Ax, Ay, Ix,Iy,Astarx,Astary,gamma,Jx,Jy,0);
	Ax=roots(1);
	Ay=roots(2);
end
coords(1) = JAx;
coords(2) = JAy;
coords(3) = Jx;
coords(4) = Jy;
coords(5) = gamma*180/pi;
coords(6) = R;
coords(7) = Cx;
coords(8) = Cy;
coords(9) = Ax;
coords(10) = Ay;
coords(11) = Astarx;
coords(12) = Astary;
