function [Values] = sliderpoint(ax1,ay1,theta1,ax2,ay2,theta2, ax3,ay3,...
					       theta3,Xn,Yn)

% This function determines the coordinates of the slider point in three
% relative to the frame given the coordinates of the slider point
% relative to the coupler system.  The coordinates of the slider line 
% are also given.  The input values are:

%ax1     = x coordinate of coupler coordinate system in position 1
%ay1     = y coordinate of coupler coordinate system in position 1
%ax2     = x coordinate of coupler coordinate system in position 2
%ay2     = y coordinate of coupler coordinate system in position 2
%ax3     = x coordinate of coupler coordinate system in position 3
%ay3     = y coordinate of coupler coordinate system in position 3
%theta1  = angle from frame x axis to coupler x axis in position 1 (degrees)
%theta2  = angle from frame x axis to coupler x axis in position 2 (degrees)
%theta3  = angle from frame x axis to coupler x axis in position 3 (degrees)
%Xn      = x coordinate of slider point relative to coupler coordinate system
%Yn      = y coordinate of slider point relative to coupler coordinate system

% The slider point coordinates are returned in the vector "values".  The 
% coordinates are stored in "Values" according to the following:

% Values(1) = slider point x in position 1 relative to frame system
% Values(2) = slider point y in position 1 relative to frame system
% Values(3) = slider point x in position 2 relative to frame system
% Values(4) = slider point y in position 2 relative to frame system
% Values(5) = slider point x in position 3 relative to frame system
% Values(6) = slider point y in position 3 relative to frame system
% Values(7) = slope of line relative to frame system (radians)
% Values(8) = distance from point 1 to point 3.

% compute sine and cosine of angles.

fact=pi/180;
theta1r=theta1*fact;
theta2r=theta2*fact;
theta3r=theta3*fact;
st1=sin(theta1r);
st2=sin(theta2r);
st3=sin(theta3r);
ct1=cos(theta1r);
ct2=cos(theta2r);
ct3=cos(theta3r);

% Transform the slider point to the frame coordinate system

xn1= Xn*ct1-Yn*st1+ax1;
yn1= Xn*st1+Yn*ct1+ay1;
xn2= Xn*ct2-Yn*st2+ax2;
yn2= Xn*st2+Yn*ct2+ay2;
xn3= Xn*ct3-Yn*st3+ax3;
yn3= Xn*st3+Yn*ct3+ay3;
	
% Compute the slope of the slider line:

angle = atan2(yn3-yn1,xn3-xn1)/fact;

% Return the computed values

Values(1)=xn1;
Values(2)=yn1;
Values(3)=xn2;
Values(4)=yn2;
Values(5)=xn3;
Values(6)=yn3;
Values(7)=angle;
Values(8)=sqrt((yn3-yn1)^2+(xn3-xn1)^2);
