% centrode.m
% MATLAB program for finding centrodes of  crank-rocker mechanism

%Variables
%   r1=frame length (cm)
%   r2=crank length (cm)
%   r3=coupler length (cm)
%   r4=rocker length (cm)
%   cr1=coupler radius (cm) (to point D1 from the crank pin)
%   beta1=coupler angle (deg) (to point D2 from the rocker pin)
%   cpad=coupler angle (rad) (to point D2 from the rocker pin)
%   Q1=frame angle (deg)
%   Q11=frame angle (rad)
%   Q2=crank angle (rad)
%   Q21=crank angle (deg)
%   Q22=crank angle (deg) (vector)
%   Q3,Q33=coupler angle (deg)
%   Q4,Q44=rocker angle (deg)
%   cang1=angle from the line connecting the coupler point one and the crank pin (rad)
%   cang2=angle from the line connecting the coupler point two and the rocker pin (rad)
%   mode=assembly mode
%   tt=number of cycles to animate
%   dt=number of animation positions per revolution
%   itotal=number of animation positions
%   cycle=number of cycles as an integer
%   bush1,bush2=x,y coordinates of the bushings
%   cir1,cir2=x,y coordinates of the circles inside the bushings
%   Bx,By=x,y coordinates of the crank pin
%   Cx,Cy=x,y coordinates of the rocker pin
%   Bx2,By2=x,y coordinates of the crank pointer
%   Cx2,Cy2=x,y coordinates of the rocker pointer
%   d2r,r2d=conversion factors between radians and degrees
%   i=counting variable
%   ans=repetition variable
%	readmode = 2 if data are to be read in interactively
%			=1 if the data are to be read in through a file.

%Graphics variables
%   xmin,xmax,ymin,ymax=axes limits
%   displ=line representing the crank angle vs. rocker angle
%   dbead=bead which follows the crank angle/rocker angle throughout animation
%   bushy1=line representing the bushing at the crank hinge
%   bushy2=line representing the bushing at the rocker hinge
%   circ1=line representing the circle inside the bushing at the crank hinge 
%   circ2=line representing the circle inside the bushing at the rocker hinge
%   joint1=line representing the joint at the crank pin 
%   joint2=line representing the joint at the rocker pin
%   crank=line representing the crank 
%   coupler=line representing the coupler
%   rocker=line representing the rocker
%   couplerptr=line representing the coupler point
%   couplerpt1=line from the crank pin to the coupler point one
%   couplerpt2=line from the rocker pin to the coupler point two

%Program  

clf;
clear all;
ttt1=1;
ddx1=1;
ddy1=1;
ans='y';
disp('   ')
disp('         Centrode Analysis Program')

%Input the linkage variables

readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (centrode.dat): ', 's');
if isempty(inputfile); inputfile = 'centrode.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
if readmode==2
	tt=input('Enter number of cycles (rev) [2]: ');
	if isempty(tt);tt = 2;end
	r1=input('Enter the frame length [3]: ');
	if isempty(r1);r1 = 3;end
	r2=input('Enter the crank length [3]: ');
	if isempty(r2);r2 = 3;end
	r3=input('Enter the coupler length [2]: ');
	if isempty(r3);r3 = 2;end
	r4=input('Enter the rocker length [3]: ');
	if isempty(r4);r4 = 3;end
	Q1=input('Enter the frame angle (deg) [0]: ');
	if isempty(Q1);Q1 = 0;end

% save the data to the file sliderio.dat
	
	temp1(1)=tt;
	temp1(2)=r1;
	temp1(3)=r2;
	temp1(4)=r3;
	temp1(5)=r4;
	temp1(6)=Q1;
	temp1t=temp1(:);  % store the transpose of temp1 for ease of reading	
	fprintf(inputfile,'%f \n',temp1t)
end

% Read input data from the file designated by inputfile.

if readmode==1	
	centrodeio = fscanf(fid,'%f');
	tt=centrodeio(1);
	r1=centrodeio(2);
	r2=centrodeio(3);
	r3=centrodeio(4);
	r4=centrodeio(5);
	Q1=centrodeio(6);
end
fclose('all');
Q11=Q1*pi/180;
	
disp('I am working ...');

% Define constants

mode=-1;
i=0;
d2r=pi/180;
r2d=180/pi;
dddx=1;
dddy=1;
qstart=0.0;
qstop=360.0;
   
% Check for the Grashof type

r=[r1 r2 r3 r4];
[rnew,Inew]=sort(r);
drock=1;
if (rnew(1)+rnew(4)<rnew(2)+rnew(3))    
	drock=0;
	if r3==rnew(1); drock=1; end		
end
if drock ==1
	qstart=1.00001*r2d*acos(((r2+r3)^2+r1^2-r4^2)/(2*r1*(r2+r3)));
	qstop=0.999999*r2d*acos((r1^2+r2^2-(r3+r4)^2)/(2*r1*r2));
end	

% Loop to analyze the linkage for tt full cycles

icent1=1;
icent2=1;
dt=(qstop-qstart)/100;
for Q21 = qstart:dt:qstop;
	i=i+1;
	Q2 = Q21*d2r;

% Call function to compute the position

	para = fourbar(r1,r2,r3,r4,Q21,1,0,mode,Q1,2); 
	Q3=para(7);
	Q33(i)=Q3;
	Q4=para(8);
	Q44(i)=Q4;
	assem=para(29);

% Store position information in matrices.

	Bx(i)= para(17);
	By(i)= para(18);
	Cx(i)= para(19);
	Cy(i)= para(20);
	a1(1)=0.0;
	a1(2)=0.0;
	a2(1)=Bx(i);
	a2(2)=By(i);
	b1(1)=r1*cos(Q11);
	b1(2)=r1*sin(Q11);
	b2(1)=Cx(i);
	b2(2)=Cy(i);
	coords=lineintersect(a1,a2,b1,b2);
	Dx4(i)=coords(1);
	Dy4(i)=coords(2);

% Check distance to instant center.  If it is large, 
% set it near the previous value.

	if i~=1; 
		ddx2=Dx4(i)-Dx4(i-1);
		ddy2=Dy4(i)-Dy4(i-1);		
		if ddx1 ~= 0; dddx=ddx2/ddx1; end
		if ddy1 ~= 0; dddy=ddy2/ddy1; end
		if dddx > 0 | dddy > 0 
			ddx1=ddx2;
			ddy1=ddy2;
		end
	end
	if i > 1 & dddx<0 & dddy<0
		if icent1==1; 
			icent1=i-1;
		end
		if icent2==1 & icent1~=i-1; 
			icent2=i-1;
		end
	end
	
% Compute the angle to the moving centrode

	Qp=r2d*atan2((Dy4(i)-By(i)), (Dx4(i)-Bx(i)));	
	Qbeta(i)=Qp-Q3;
	r5(i)=sqrt((Dy4(i)-By(i))^2+(Dx4(i)-Bx(i))^2);
	Q22(i) = Q21;
end	
itotal=i;

% Determine the three parts of the centrode.

jstop1=itotal;
jstop2=itotal;
if icent1~=1; jstop1=icent1; end
for j=1:1:jstop1
	Dx1(j)=Dx4(j);
	Dy1(j)=Dy4(j);
end

% Determine the second part of the centrode

k=0;
Dx2(1)=Dx4(itotal);
Dy2(1)=Dy4(itotal);
jstart2=icent1+1;
jstop2=icent2;
if icent2==1; jstop2 = itotal; end
if icent1==1; jstart2 = itotal; end
for j=jstart2:1:jstop2;
	k=k+1;
	Dx2(k)=Dx4(j);
	Dy2(k)=Dy4(j);
end
ktotal = k;

% Determine the third part of the centrode

m=0;
jstart3=icent2+1;
jstop3=itotal;
if icent2==1; jstop3 = itotal; end
if icent2==1; jstart3 = itotal; end
for j=jstart3:1:jstop3;
	m=m+1;
	Dx3(m)=Dx4(j);
	Dy3(m)=Dy4(j);
end
mtotal = m;

% Find the coordinates of the moving centrode for each position.
% Start with first part.
 
for k=1:1:itotal
	for j=1:1:jstop1
		angle=(Qbeta(j)+Q33(k))*d2r;
		centx1(k,j)=Bx(k)+r5(j)*cos(angle);
		centy1(k,j)=By(k)+r5(j)*sin(angle);
	end

% Do the second part.

	l=0;
	for j=jstart2:1:jstop2
		l=l+1;
		angle=(Qbeta(j)+Q33(k))*d2r;
		centx2(k,l)=Bx(k)+r5(j)*cos(angle);
		centy2(k,l)=By(k)+r5(j)*sin(angle);
	end

% Do the third part.

	m=0;
	for j=jstart3:1:itotal
		m=m+1;
		angle=(Qbeta(j)+Q33(k))*d2r;
		centx3(k,m)=Bx(k)+r5(j)*cos(angle);
		centy3(k,m)=By(k)+r5(j)*sin(angle);
	end
end

% Define the axes for the graph showing the centrodes.  First find the 
% extreme values for the points being plotted.

clf;
width=0.95;
height=0.95;
h0=axes('position', [.05 .05 width height],'box','on','xcolor','k',...
        'ycolor','k','fontname', 'times', 'fontsize', 10);
set(gca,'color', 'w');
Ex = r1*cos(Q11);
Ey = r1*sin(Q11);

max1=max(Bx);
max2=max(Cx);
max3=max(Dx4);
max4=max(centx1);
max5=max(centx2);
max6=max(centx3);
xmax=max([max1, max2, max3, max4, max5, max6, Ex]);
if xmax>3*r1; xmax=3*r1; end

max1=max(By);
max2=max(Cy);
max3=max(Dy1);
max4=max(centy1);
max5=max(centy2);
max6=max(centy3);
ymax=max([max1, max2, max3, max4, max5, max6, Ey]);
if ymax>6*r4; ymax=6*r4; end

min1=min(Bx);
min2=min(Cx);
min3=min(Dx1);
min4=min(centx1);
min5=min(centx2);
min6=min(centx3);
xmin=min([min1, min2, min3, min4, min5, min6, Ex]);
if xmin<-3*r1; xmin=-3*r1; end

min1=min(By);
min2=min(Cy);
min3=min(Dy1);
min4=min(centy1);
min5=min(centy2);
min6=min(centy3);
ymin=min([min1, min2, min3, min4, min5, min6, Ey]);
if ymin<-2*r4; ymin=-2*r4; end

%get the bushing points

bush1=bushing(((ymax-ymin)/50),0,0,20,0);
cir1=circle2(((ymax-ymin)/60),0,0,20);
bush2=bushing(((ymax-ymin)/50), Ex, Ey, 20,0);
cir2=circle2(((ymax-ymin)/60), Ex, Ey, 20);
max1=max(bush1(:, 1));
max2=max(bush2(:, 1));
xmax1=max([max1, max2, xmax]);
max1=max(bush1(:, 2));
max2=max(bush2(:, 2));
ymax1=max([max1, max2, ymax]);
min1=min(bush1(:, 1));
min2=min(bush2(:, 1));
xmin1=min([min1, min2, xmin]);
min1=min(bush1(:, 2));
min2=min(bush2(:, 2));
ymin1=min([min1, min2, ymin]);
xmin=xmin1-0.05*abs(xmax1-xmin1);
xmax=xmax1+0.05*abs(xmax1-xmin1);
ymin=ymin1-0.05*abs(ymax1-ymin1);
ymax=ymax1+0.05*abs(ymax1-ymin1);

axis equal;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis([xmin xmax ymin ymax]);
  
% Set up the vectors needed to draw the linkage

joint1=line('xdata', [], 'ydata', [], 'marker', 'o', 'erasemode',...
    'xor','markersize',6, 'color', [1,0,0]);
joint2=line('xdata', [], 'ydata', [], 'marker', 'o', 'erasemode',...
    'xor','markersize',6, 'color', [1,0,0]);
crank=line('xdata', [0 0], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
	'xor','color', [1,0,0]);
coupler=line('xdata', [], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
	'xor','color', [1,0,0]);
fcentrode1=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
	'none','color', 'k');	
fcentrode2=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
	'none','color', 'k');	
fcentrode3=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
	'none','color', 'k');	
gridpt=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
	'xor','markersize',6,'color', 'k', 'marker', 'o');	
rocker=line('xdata', Ex, 'ydata' ,Ey, 'linewidth',2,...
	'erasemode', 'xor','color', [1,0,0]);
bushy1=line('xdata', bush1(:,1), 'ydata',bush1(:,2), 'linestyle', '-',...
	'color', 'k');
bushy2=line('xdata', bush2(:,1), 'ydata',bush2(:,2), 'linestyle', '-',...
	'color', 'k');
circ1=line('xdata', cir1(:,1), 'ydata',cir1(:,2), 'linestyle', '-',...
	'color', 'k');
circ2=line('xdata', cir2(:,1), 'ydata',cir2(:,2), 'linestyle', '-', ...
	'color', 'k');
mcentrode1=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
	'xor','color', [1,0,0]);	
mcentrode2=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
	'xor','color', [1,0,0]);	
mcentrode3=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
	'xor','color', [1,0,0]);	

% Plot the of the linakges alone

ans='y';

% Draw the coupler curve first.

set(fcentrode1,'xdata', Dx1, 'ydata' ,Dy1);
set(fcentrode2,'xdata', Dx2, 'ydata' ,Dy2);
set(fcentrode3,'xdata', Dx3, 'ydata' ,Dy3);

% Animate the linkage as long as ans = 'y'
	
timesink=1;
while ans=='y'
	for kk=1:1:tt
		for i=1:1:itotal
			set(joint1,'xdata', Bx(i), 'ydata',By(i));
			set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
			set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
			set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
			set(rocker,'xdata', [Ex Cx(i)],'ydata', [Ey Cy(i)]);
			set(mcentrode1,'xdata', [centx1(i,:)],'ydata' ,[centy1(i,:)]);
			set(mcentrode2,'xdata', [centx2(i,:)],'ydata' ,[centy2(i,:)]);
			set(mcentrode3,'xdata', [centx3(i,:)],'ydata' ,[centy3(i,:)]);
			set(gridpt,'xdata',Dx4(i), 'ydata' ,Dy4(i));
			drawnow;   %flush the draw buffer

% Do meaningless calculation to slow down the linkage

			for k=1:1:timesink
				zzz=log10(k);
			end
		end
	end
% Plot linkage in fixed position.

	i = floor(itotal/2);
	set(joint1,'xdata', Bx(i), 'ydata',By(i));
	set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
	set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
	set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
	set(rocker,'xdata', [Ex Cx(i)],'ydata', [Ey Cy(i)]);
	set(gridpt,'xdata',Dx4(i), 'ydata' ,Dy4(i));
	set(mcentrode1,'xdata', [centx1(i,:)],'ydata' ,[centy1(i,:)]);
	set(mcentrode2,'xdata', [centx2(i,:)],'ydata' ,[centy2(i,:)]);
	set(mcentrode3,'xdata', [centx3(i,:)],'ydata' ,[centy3(i,:)]);
   
%flush the draw buffer

	drawnow; 
		
% Ask if animation should be repeated

	ans=input('repeat animation? y/n [y]: ','s');
	if isempty(ans);ans='y';end
	if ans=='y'
		rep=input('Change animation speed? (+/-/0) [-]:','s');
		if isempty(rep);rep='-';end
		if rep=='+'; timesink=timesink/10; end
		if rep=='-'; timesink=timesink*10; end
		if timesink<1; timesink=1; end
	end
end

