function [Values] = circlepoint (ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
                    theta3, Astarx,Astary)

% This function determines the coordinates of the circle point given the
% coordinates of the center point.  The input values are:

%ax1     = x coordinate of coupler coordinate system in position 1
%ay1     = y coordinate of coupler coordinate system in position 1
%ax2     = x coordinate of coupler coordinate system in position 2
%ay2     = y coordinate of coupler coordinate system in position 2
%ax3     = x coordinate of coupler coordinate system in position 3
%ay3     = y coordinate of coupler coordinate system in position 3
%theta1  = angle from frame x axis to coupler x axis in position 1 (degrees)
%theta2  = angle from frame x axis to coupler x axis in position 2 (degrees)
%theta3  = angle from frame x axis to coupler x axis in position 3 (degrees)
%Astarx  = x coordinate of center point relative to coupler coordinate system
%Astary  = y coordinate of center point relative to coupler coordinate system

% The circle point coordinates are returned in the vector "values".  The 
% coordinates are stored in "Values" according to the following:

%Values(1) = circle point x in position 1 relative to frame system
%Values(2) = circle point y in position 1 relative to frame system
%Values(3) = circle point x relative to coupler system
%Values(4) = circle point y relative to coupler system

% compute sine and cosine of angles.

fact=pi/180;
theta1r=theta1*fact;
theta2r=theta2*fact;
theta3r=theta3*fact;
st1=sin(theta1r);
st2=sin(theta2r);
st3=sin(theta3r);
ct1=cos(theta1r);
ct2=cos(theta2r);
ct3=cos(theta3r);

% Transform the center point to the coupler coordinate system

XC1= (Astarx-ax1)*ct1+(Astary-ay1)*st1;
YC1= -(Astarx-ax1)*st1+(Astary-ay1)*ct1;
XC2= (Astarx-ax2)*ct2+(Astary-ay2)*st2;
YC2= -(Astarx-ax2)*st2+(Astary-ay2)*ct2;
XC3= (Astarx-ax3)*ct3+(Astary-ay3)*st3;
YC3= -(Astarx-ax3)*st3+(Astary-ay3)*ct3;
	
% Compute the circle point.

A=pole([XC1;YC1],[XC2;YC2],[XC2;YC2],[XC3;YC3]);

% Locate circle point coordinates relative to frame

Values(1)= A(1)*ct1-A(2)*st1+ax1;
Values(2)= A(1)*st1+A(2)*ct1+ay1;

% Locate circle point coordinates relative to coupler

Values(3)=A(1);
Values(4)=A(2);
