% fourbar_analysis.m
% MATLAB program for analyzing four-bar mechanisms

%Variables

%   r1=frame length
%   r2=crank length
%   r3=coupler length
%   r4=rocker length
%   cr1=coupler radius (to point D1 from the crank pin)
%   beta1=coupler angle (deg) (to point D2 from the rocker pin)
%   cpad=coupler angle (rad) (to point D2 from the rocker pin)
%	fourbario = data file with input data in it.
%	readmode = io flag.  readmode = 1 for file input and 2 for interactive input.
%   Q1=frame angle (deg)
%   Q11=frame angle (rad)
%   Q2=crank angle (rad)
%   Q21=crank angle (deg)
%   Q22=crank angle (deg) (vector)
%   Q3,Q33=coupler angle (deg)
%   Q4,Q44=rocker angle (deg)
%   wdr=angular velocity of the driver link (crank or coupler) (rad/sec)
%   w22=w2 (vector)
%   w33=angular velocity of the coupler (rad/sec)
%   w44=angular velocity of the rocker (rad/sec)
%   cang1=angle from the line connecting the coupler point one and the crank pin (rad)
%   cang2=angle from the line connecting the coupler point two and the rocker pin (rad)
%   mode,assem=assembly mode
%   tt=number of cycles to animate
%   dt=number of animation positions per revolution
%   itotal=number of animation positions
%   bush1,bush2=x,y coordinates of the bushings
%   cir1,cir2=x,y coordinates of the circles inside the bushings
%   Bx,By=x,y coordinates of the crank pin
%   Cx,Cy=x,y coordinates of the rocker pin
%   Bx2,By2=x,y coordinates of the crank pointer
%   Cx2,Cy2=x,y coordinates of the rocker pointer
%   d2r,r2d=conversion factors between radians and degrees
%   i=counting variable
%   ans=repitition variable

%Graphics variables

%   xmin,xmax,ymin,ymax=axes limits
%   h1=handle on the first axes in the window (crank angle vs. rocker angle)
%   h2=handle on the second axes in the window (crank angle vs. rocker velocity)
%   h3=handle on the third axes in the window (crank angle vs. input torque)
%   h4=handle on the fourth axes in the window (linkage animation)
%   displ4=line representing the crank angle vs. rocker angle
%   vel=line representing the crank angle vs. rocker velocity
%   displ3=line representing the crank angle vs. coupler angle
%   d4bead=bead which follows the crank angle/rocker angle throughout animation
%   vbead=bead which follows the crank angle/rocker velocity throughout animation
%   d3bead=bead which follows the crank angle/coupler angle throughout animation
%   bushy1=line representing the bushing at the crank hinge
%   bushy2=line representing the bushing at the rocker hinge
%   circ1=line representing the circle inside the bushing at the crank hinge 
%   circ2=line representing the circle inside the bushing at the rocker hinge
%   joint1=line representing the joint at the crank pin 
%   joint2=line representing the joint at the rocker pin
%   crank=line representing the crank 
%   coupler=line representing the coupler
%   rocker=line representing the rocker
%   couplerptr1=from crank circle point to coupler point
%   couplerptr2=from rocker circle point to coupler point
%   couplerpoint=line representing the coupler curve
%	cpflag = 1 if both couplerptr1 and couplerptr2 lines are to be 
%            drawn.  cpflag = 2 if only couplerpt1 line is to be 
%            drawn.

%Program  

clf;
clear all;
ans='y';
disp('   ')
disp('         Four-Bar Linkage Analysis Program')

%Input the linkage variables

readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (fourbario.dat): ', 's');
if isempty(inputfile); inputfile = 'fourbario.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
if readmode==2
	tt=input('Enter number of cycles (rev) [3]: ');
	if isempty(tt);tt = 3;end
	r1=input('Enter the frame length [2]: ');
	if isempty(r1);r1 = 2;end
	Q1=input('Enter the frame angle (deg) [0]: ');
	if isempty(Q1);Q1 = 0;end
	r2=input('Enter the crank length [2.5]: ');
	if isempty(r2);r2 = 2.5;end
	r3=input('Enter the coupler length [1.0]: ');
	if isempty(r3);r3 = 1.0; end
	r4=input('Enter the rocker length [2.5]: ');
	if isempty(r4);r4 = 2.5;end
	cr1=input('Enter coupler point radius [0.5]: ');
	if isempty(cr1);cr1=0.5;end
	beta1=input('Enter angle from coupler line to coupler point (deg) [0]: ');
	if isempty(beta1);beta1=0;end
	disp('   ')
	disp('    Flag to determine how coupler is to be drawn  ');
	cpflag=input('Enter 1 for coupler line and 2 for coupler triangle [1]: ');
	if isempty(cpflag);cpflag=1;end
	disp('   ')
	disp('    Flag to determine driver link (crank or coupler)  ');
	driver=input('Enter 1 for crank driver and 2 for coupler driver [1]: ');
	if isempty(driver);driver=1;end
	disp('   ')
	disp('    Flag to determine if both linkage modes are to be computed ');
	times=input('Enter 1 for designated mode only and 2 for both modes [1]: ');
	if isempty(times);times=1;end
	mode=input('Enter the assembly mode (+1 or -1) [-1]: ');
	if isempty(mode);mode = -1;end
	wdr=input('Enter the angular velocity of driver link(rad/sec) [5]: ');
	if isempty(wdr);wdr = 5;end
	
% save the data to the file fileinput.dat
	
	tempio(1)=tt;
	tempio(2)=r1;
	tempio(3)=Q1;	
	tempio(4)=r2;
	tempio(5)=r3;
	tempio(6)=r4;
	tempio(7)=cr1;
	tempio(8)=beta1;
	tempio(9)=cpflag;
	tempio(10)=driver;
	tempio(11)=times;
	tempio(12)=mode;
	tempio(13)=wdr;
	tempiot=tempio(:);  % store the transpose of tempio for ease of reading
	fprintf(inputfile,'%f \n',tempiot)
end

% Read input data from the file fourbario.dat

if readmode==1	
	fourbario = fscanf(fid,'%f');
	tt=fourbario(1);
	r1=fourbario(2);
	Q1=fourbario(3);
	r2=fourbario(4);
	r3=fourbario(5);
	r4=fourbario(6);
	cr1=fourbario(7);
	beta1=fourbario(8);
	cpflag=fourbario(9);
	driver=fourbario(10);
	times=fourbario(11);
	mode=fourbario(12);
	wdr=fourbario(13);
end
fclose('all');
Q11=Q1*pi/180;
disp('I am working ...');
fprintf('fourbar.dat', ' Q2	   Q3     Q4    w3     w4   \n\n');

% Define constants

i=0;
d2r=pi/180;
r2d=180/pi;

% Determine the angle limits

qstart=0;
qstop=360;
if driver==1; [qstart,qstop]=fb_angle_limits_cr(r1, r2, r3,r4,Q1); end
if driver==2; [qstart,qstop]=fb_angle_limits_co(r1, r2, r3,r4,Q1); end
temp=qstart;
if qstop<qstart;
	 qstart=qstop;
	 qstop=temp;
end
npos=100;
dq = (qstop-qstart)/(npos-1);
QM1=qstart;
i2=0;

for mmm=1:1:times

% Analyze for two cycles in different modes if times=2.

	if mmm ~=1
		mode = -mode;
		dq=-dq;
	end
	
% Loop to analyze the linkage for a full cycle

	for motion=1:1:npos
		if motion>1; QM1=QM1+dq; end

% Call function to compute the position.  If driver=1, call fourbar with 
% crank as driver.  If driver=2, call fourbar with coupler as driver.

		if driver==1
			para = fourbar_cr(r1,r2,r3,r4,QM1,wdr,0,mode,Q1,2);
		end
		if driver==2
			para = fourbar_co(r1,r2,r3,r4,QM1,wdr,0,mode,Q1,2);
		end	

% Store the linkage information in para into named vectors
	 
		assem=para(29);
		j=0;
		if assem==1
			i=i+1;
			Q22(i)=para(6);
			Q33(i)=para(7);
			Q44(i)=para(8);
			if Q33(i)<0; Q33(i)=Q33(i)+360; end
			if Q44(i)<0; Q44(i)=Q44(i)+360; end
			w22(i)=para(10); 
			w33(i)=para(11);
			w44(i)=para(12);
			cang1=(Q33(i)+beta1)*d2r;

% Store position information in matrices.

			Bx(i)= para(17);
			By(i)= para(18);
			Cx(i)= para(19);
			Cy(i)= para(20);
			Dx1(i)=Bx(i)+cr1*cos(cang1);
			Dy1(i)=By(i)+cr1*sin(cang1);
			
% Determine coordinates for coupler curve for first mode
			
			if mmm==1
				Dx2(i)=Dx1(i);
				Dy2(i)=Dy1(i);
			end
			
% Determine coordinates for coupler curve for second mode
			
			if mmm==2
				i2=i2+1;
				Dx3(i2)=Dx1(i);
				Dy3(i2)=Dy1(i);
			end
					
% Print results in a table

			fprintf('fourbar.dat','%8.2f %8.2f %8.2f %8.2f %8.2f \n',...
			Q22(i),Q33(i),Q44(i),w33(i),w44(i)) 
		end
	end
end	

% Define the axes for the graph showing the linkage

clf;
width=0.95;
height=0.95;
h0=axes('position', [.05 .05 width height],'box', 'on', 'xcolor', 'k','ycolor',...
       'k');
set(gcf,'color', 'w')
	   
% Locate the coordinates of the bushing and revolute joints.  First find
% the shortest link

rmin=min([r1, r2, r3, r4]);

bush1=bushing(rmin/10,0,0,40,0);
cir1=circle2(rmin/10,0,0,40);
bush2=bushing(rmin/10,(r1*cos(Q11)),(r1*sin(Q11)),40,0);
cir2=circle2(rmin/10,(r1*cos(Q11)),(r1*sin(Q11)),40);

% Find the limits for the plots

xmina=min([min(Dx1) min(Bx) min(Cx) min(bush1(:,1)) ...
      min(bush2(:,1)) r1*cos(Q11)]);
xmaxa=max([max(Dx1) max(Bx) max(Cx) max(bush1(:,1)) ...
      max(bush2(:,1)) r1*cos(Q11)]);
ymina=min([min(Dy1) min(By) min(Cy) min(bush1(:,2)) ...
      min(bush2(:,2)) r1*sin(Q11)]);
ymaxa=max([max(Dy1) max(By) max(Cy) max(bush1(:,2)) ...
      max(bush2(:,2)) r1*sin(Q11)]);
rangex=xmaxa-xmina;
rangey=ymaxa-ymina;
xmin=xmina-0.05*rangex;
xmax=xmaxa+0.05*rangex;
ymin=ymina-0.05*rangey;
ymax=ymaxa+0.05*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);
  
% Set up the vectors needed to draw the linkage

joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       8, 'erasemode', 'xor', 'color', 'k');
joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       8, 'erasemode', 'xor', 'color', 'k');
crank=line('xdata', [0 0], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
coupler=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
couplerptr1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');	   
couplerptr2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');	   
couplerpoint1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
couplerpoint2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
gridpt=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','markersize',6,'color', 'r', 'marker', 'o');	
rocker=line('xdata', [r1*cos(Q1) 0], 'ydata' ,[r1*sin(Q1) 0], 'linewidth',2,...
       'erasemode', 'xor','color', 'k');
bushy1=line('xdata', bush1(:,1), 'ydata',bush1(:,2), 'linestyle', '-',...
        'color', 'r');
bushy2=line('xdata', bush2(:,1), 'ydata',bush2(:,2), 'linestyle', '-', ...
        'color', 'r');
circ1=line('xdata', cir1(:,1), 'ydata',cir1(:,2), 'linestyle', '-',...
        'color', 'r');
circ2=line('xdata', cir2(:,1), 'ydata',cir2(:,2), 'linestyle', '-', ...
        'color', 'r');

% Plot the of the linkage alone

itotal=i;
ans='y';

% Draw the coupler curves first.

set(couplerpoint1,'xdata', Dx2, 'ydata' ,Dy2);
if times==2
	set(couplerpoint2,'xdata', Dx3, 'ydata' ,Dy3);
end

% Animate the linkage as long as ans = 'y'
	
timesink=1;
while ans=='y'
	for m=1:1:tt
		for i=1:1:itotal
			set(joint1,'xdata', Bx(i), 'ydata',By(i));
			set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
			set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
			set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
			set(rocker,'xdata', [(r1*cos(Q11)) Cx(i)],'ydata', ...
				[(r1*sin(Q11)) Cy(i)]);
			set(couplerptr1,'xdata', [Bx(i) Dx1(i)],...
				'ydata' ,[By(i) Dy1(i)]);
			if cpflag ==2
				set(couplerptr2,'xdata', [Cx(i) Dx1(i)],...
					'ydata' ,[Cy(i) Dy1(i)]);
			end
			set(gridpt,'xdata',Dx1(i), 'ydata' ,Dy1(i));
			drawnow;   %flush the draw buffer

% Do meaningless calculation to slow down the linkage

			for k=1:1:timesink
				zzz=log10(k);
			end
		end
	end

% Plot linkage in fixed position.

	i = floor(itotal/4);
	set(joint1,'xdata', Bx(i), 'ydata',By(i));
	set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
	set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
	set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
	set(rocker,'xdata', [(r1*cos(Q11)) Cx(i)],'ydata', ...
		[(r1*sin(Q11)) Cy(i)]);
	set(gridpt,'xdata',Dx1(i), 'ydata' ,Dy1(i));
	set(couplerptr1,'xdata', [Bx(i) Dx1(i)], ...
		'ydata' ,[By(i) Dy1(i)]);
	if cpflag ==2
		set(couplerptr2,'xdata', [Cx(i) Dx1(i)],...
			'ydata' ,[Cy(i) Dy1(i)]);
	end
  
%flush the draw buffer

	drawnow; 
		
% Ask if animation should be repeated

	ans=input('Repeat animation? y/n [y]: ','s');
	if isempty(ans);ans='y';end
	if ans=='y'
		rep=input('Change animation speed? (+/-/0) [-]:','s');
		if isempty(rep);rep='-';end
		if rep=='+'; timesink=timesink/10; end
		if rep=='-'; timesink=timesink*10; end
		if timesink<1; timesink=1; end
	end
end

% Pause the graphics before showing four plots per graph

pause;
clf;

% Define the axes for the first graph (input/output angle graph)

width=0.38;
height=0.35;
h1=axes('position',[0.6 .55 width height],'box','on','xcolor','k','ycolor',...
      'k');   %axis location
set(gcf,'color', 'w')
xlabel('Input Angle','color', 'k')
ylabel('Rocker Angle','color', 'k')
Displ4=line('xdata', [], 'ydata', [], 'erasemode', 'none',...
      'color', 'r', 'linestyle', '-');
d4bead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, 'erasemode',...
      'xor');

% Define the axes for the second graph (rocker velocity graph)

h2=axes('position', [0.6 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k');
set(gcf,'color', 'w')
xlabel('Input Angle','color', 'k')
ylabel('Rocker Velocity','color', 'k')
Vel=line('xdata', [], 'ydata', [], 'erasemode', 'none', ...
      'color', 'r', 'linestyle', '-');
vbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, ...
      'erasemode','xor');

% Define the axes for the third graph (rocker angle)

h3=axes('position', [0.1 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k');
set(gcf,'color', 'w')
xlabel('Input Angle','color', 'k')
ylabel('Coupler Angle','color', 'k')
Displ3=line('xdata', [], 'ydata', [], 'erasemode','none', ...
      'color', 'r', 'linestyle', '-');
d3bead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6,...
      'erasemode','xor');

% Define the axes for the fourth graph (linkage position)

h4=axes('position', [0.1 .55 width height],'box', 'on', 'xcolor', 'k','ycolor',...
       'k');
set(gcf,'color', 'w')
axis equal;
axis([xmin xmax ymin ymax]);
  
% Set up the vectors needed to draw the linkage

joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       8, 'erasemode', 'xor', 'color', 'k');
joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       8, 'erasemode', 'xor', 'color', 'k');
crank=line('xdata', [0 0], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
coupler=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
couplerptr1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');	   
couplerptr2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');	   
gridpt=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','markersize',6,'color', 'r', 'marker', 'o');	
couplerpoint1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
couplerpoint2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
rocker=line('xdata', [r1*cos(Q1) 0], 'ydata' ,[r1*sin(Q1) 0], 'linewidth',2,...
       'erasemode', 'xor','color', 'k');
bushy1=line('xdata', bush1(:,1), 'ydata',bush1(:,2), 'linestyle', '-',...
       'markersize', 8,'color', 'r');
bushy2=line('xdata', bush2(:,1), 'ydata',bush2(:,2), 'linestyle', '-', ...
        'markersize', 8, 'color', 'r');
circ1=line('xdata', cir1(:,1), 'ydata',cir1(:,2), 'linestyle', '-',...
        'markersize', 8,'color', 'r');
circ2=line('xdata', cir2(:,1), 'ydata',cir2(:,2), 'linestyle', '-', ...
        'markersize', 8, 'color', 'r');

% Plot the results.

ans='y';
	
% Draw the fixed curves first.

if driver==1;inputv=Q22; end
if driver==2;inputv=Q33; end

% Draw four plots on one graph.  Start by setting the axis limits for the graphs

qstart=min(inputv);
qstop=max(inputv);
axes(h3);
axis([qstart qstop min(Q33) max(Q33)]);  %set axis limits for the coupler angle plot
axes(h1);
axis([qstart qstop min(Q44) max(Q44)]);  %set axis limits for the rocker angle plot
axes(h2);
axis([qstart qstop min(w44) max(w44)]);  %set axis limits for the velocity plot

set(couplerpoint1,'xdata', Dx2, 'ydata' ,Dy2);
if times==2
	set(couplerpoint2,'xdata', Dx3, 'ydata' ,Dy3);
end
set(Displ3,'xdata', inputv,'ydata', Q33);
set(Displ4,'xdata', inputv,'ydata', Q44);
set(Vel,'xdata', inputv,'ydata', w44);

% Animate the results as long as ans = 'y'

while ans=='y'
	for m=1:1:tt
		for i=1:1:itotal
			set(joint1,'xdata', Bx(i), 'ydata',By(i));
			set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
			set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
			set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
			set(rocker,'xdata', [(r1*cos(Q11)) Cx(i)],'ydata', ...
				[(r1*sin(Q11)) Cy(i)]);
			set(gridpt,'xdata',Dx1(i), 'ydata' ,Dy1(i));
			set(couplerptr1,'xdata', [Bx(i) Dx1(i)],...
				'ydata' ,[By(i) Dy1(i)]);
			if cpflag ==2
				set(couplerptr2,'xdata', [Cx(i) Dx1(i)],...
				'ydata' ,[Cy(i) Dy1(i)]);
			end
			set(d3bead, 'xdata', inputv(i), 'ydata', Q33(i));
			set(d4bead, 'xdata', inputv(i), 'ydata', Q44(i));
			set(vbead, 'xdata', inputv(i), 'ydata', w44(i));
			drawnow;   %flush the draw buffer
		
% Do meaningless calculation to slow down the linkage

			for k=1:1:timesink
				zzz=log10(k);
			end
		end
	end

% Plot figures in final position.

	i = floor(itotal/3);

% Plot position first

	set(joint1,'xdata', Bx(i), 'ydata',By(i));
	set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
	set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
	set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
	set(rocker,'xdata', [(r1*cos(Q11)) Cx(i)],'ydata', ...
         [(r1*sin(Q11)) Cy(i)]);
	set(gridpt,'xdata',Dx1(i), 'ydata' ,Dy1(i));
	set(couplerptr1,'xdata', [Bx(i) Dx1(i)], ...
         'ydata' ,[By(i) Dy1(i)]);
	if cpflag ==2
		set(couplerptr2,'xdata', [Cx(i) Dx1(i)],...
				'ydata' ,[Cy(i) Dy1(i)]);
	end
   
% Plot Torque information

	axes(h3);
	set(d3bead, 'xdata', inputv(i), 'ydata', Q33(i));
 
% Plot angle information

	axes(h1);
	set(d4bead, 'xdata', inputv(i), 'ydata', Q44(i));

% Plot velocity information

	axes(h2);
	set(vbead, 'xdata', inputv(i), 'ydata', w44(i));

%flush the draw buffer

	drawnow; 

% Ask if the animation should be repeated

	ans=input('Repeat animation? y/n [y]: ', 's');
	if isempty(ans);
		ans='y';
	end
	if ans=='y'
		rep=input('Change animation speed? (+/-/0) [-]:','s');
		if isempty(rep);rep='-';end
		if rep=='+'; timesink=timesink/10; end
		if rep=='-'; timesink=timesink*10; end
		if timesink<1; timesink=1; end
	end
end

