% hr_crankrocker.m

% MATLAB program to produce an atlas of coupler curves for crank
% rocker mechanisms.

%Variables
%   r1=frame length (cm)
%   r2=crank length (cm)
%   r3=coupler length (cm)
%   r4=rocker length (cm)
%   rg=grid row to animate
%   r33g=horizontal grid spacing
%   xg,yg=x,y coordinates of the grid points
%   rr,ppheta=polar coordinates of the grid points
%   cr1=coupler radius (cm) (to point D1 from the crank pin)
%   cr2=coupler radius (cm) (to point D1 from the crank pin)
%   beta1=coupler angle (deg) (to point D2 from the rocker pin)
%   beta2=coupler angle (deg) (to point D2 from the rocker pin)
%   cpad=coupler angle (rad) (to point D2 from the rocker pin)
%   Q1=frame angle (deg)
%   Q11=frame angle (rad)
%   Q2=crank angle (rad)
%   Q21=crank angle (deg)
%   Q22=crank angle (deg) (vector)
%   Q3,Q33=coupler angle (deg)
%   Q4,Q44=rocker angle (deg)
%   w2=angular velocity of the crank (rad/sec)
%   w22=w2 (vector)
%   w33=angular velocity of the coupler (rad/sec)
%   w44=angular velocity of the rocker (rad/sec)
%   cang1=angle from the line connecting the coupler point one and the crank pin (rad)
%   cang2=angle from the line connecting the coupler point two and the rocker pin (rad)
%   Tout=torque on output link (rocker) (N-cm)
%   Tin=torque on crank pin (N-cm)
%   mode,assem=assembly mode
%   tt=number of cycles to animate
%   dt=number of animation positions per revolution
%   itotal=number of animation positions
%   cycle=number of cycles as an integer
%   bush1,bush2=x,y coordinates of the bushings
%   cir1,cir2=x,y coordinates of the circles inside the bushings
%   Bx,By=x,y coordinates of the crank pin
%   Cx,Cy=x,y coordinates of the rocker pin
%   Bx2,By2=x,y coordinates of the crank pointer
%   Cx2,Cy2=x,y coordinates of the rocker pointer
%   mx,my=x,y coordinates of the grid points
%   d2r,r2d=conversion factors between radians and degrees
%   i=counting variable
%   ans=repitition variable
%	vbx=crank pin velocity in the x direction
%	vby=crank pin velocity in the y direction
%	vcx=rocker pin velocity in the x direction
%	vcy=rocker pin velocity in the y direction
%	nhgp=# of grid points in the x direction
%	nvgp=# of grid points in the y direction
%	gp=starting index of grid points to animate
%	hh=counter
%	timesink=delay variable for animation speed
%	vm=velocity magnitude of selected grid point
%	nhgpoc=# of gridpoints along r3
%	tl=length along gridpoints horizontal
%	cxrc=distance to center of gridpoints
%	spacx=spacing between gridpoints
%	cxmin,cxmax,cymin,cymax=window limits based on grid points
%	xmin1,xmax1,ymin1=window limits based on grid points
%	xmin2,xmax2,ymin2=window limits based on linkage
%	xmin,xmax,ymin,ymax=window limits
%	a1=frame/crank joint x,y coordinates
%	b1=crank pin x,y coordinates
%	c1=rocker pin x,y coordinates
%	d1=frame/rocker joint x,y coordinates
%	xg=x coordinates of grid points
%	ygy=y coordinates of grid points
%	yg=y coordinates of selected row of grid points
%	xp,yp=x,y coordinates of the mouse selected point
%	rr,ppheta=polar coordinates of the selected row of grid points
%	Qs=starting crank angle
%	mx,my=x,y coordinates of the selected grid points throughout animation
%	rep=string variable dictating a change in the spee of animation

%Graphics variables
%   xmin,xmax,ymin,ymax=axes limits
%   h1=handle on the first axes in the window (crank angle vs. rocker angle)
%   h2=handle on the second axes in the window (crank angle vs. rocker velocity)
%   h3=handle on the third axes in the window (crank angle vs. input torque)
%   h4=handle on the fourth axes in the window (linkage animation)
%   displ=line representing the crank angle vs. rocker angle
%   vel=line representing the crank angle vs. rocker velocity
%   torque=line representing the crank angle vs. input torque
%   dbead=bead which follows the crank angle/rocker angle throughout animation
%   vbead=bead which follows the crank angle/rocker velocity throughout animation
%   tbead=bead which follows the crank angle/input torque throughout animation
%   bushy1=line representing the bushing at the crank hinge
%   bushy2=line representing the bushing at the rocker hinge
%   circ1=line representing the circle inside the bushing at the crank hinge 
%   circ2=line representing the circle inside the bushing at the rocker hinge
%   joint1=line representing the joint at the crank pin 
%   joint2=line representing the joint at the rocker pin
%   crank=line representing the crank 
%   coupler=line representing the coupler
%   rocker=line representing the rocker
%   couplerptr=line representing the coupler point
%   couplerpt1=line from the crank pin to the coupler point one
%   couplerpt2=line from the rocker pin to the coupler point two
%   gpvel=line representing gridpoint velocity
%   gridpt=line representing gridpoints
%   gridline=line representing grid point paths
%   gridpoint=line representing gridpoints throughout animation
%   gline=line representing selected grid points
%   gpoint=line representing selected grid point

%Program

clf;
clear all;
r2=1;
ans='y';
disp('   ')
disp('         Crank-Rocker Coupler Curve Program')

%Input the linkage variables

readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (hrcrankrockerio.dat): ', 's');
if isempty(inputfile); inputfile = 'hrcrankrockerio.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
if readmode==2
	tt=input('Enter number of cycles (rev) [3]: ');
	if isempty(tt);tt = 3;end
	y=[1; 2; 3; 5];
	while y(1)+y(4)>y(2)+y(3)
		r1=input('Enter the frame length [3.304]: ');
		if isempty(r1);r1 = 3.304;end
		r3=input('Enter the coupler length [2.913]: ');
		if isempty(r3);r3 = 2.913;end
		r4=input('Enter the rocker length [1.565]: ');
		if isempty(r4);r4 = 1.565;end

% Check for Grashoff type
		x=[r1; r2; r3; r4];
		y=sort(x);
		if(y(1)+y(4)>y(2)+y(3))
			fprintf('Linkage is non-Grashoff - Please re-enter link lengths.');
		end 
	end
	w2=input('Enter the angular velocity of crank(rad/sec) [5]: ');
	if isempty(w2);w2 = 5;end
	tl0=input('Enter total length of coupler grid [4*r3]: ');
	if isempty(tl0);tl0 = 4*r3;end
	tv0=input('Enter total height of coupler grid [2*r3]: ');
	if isempty(tv0);tv0 = 2*r3;end
	nhgp0=input('Enter number of coupler points in horizontal direction [20]: ');
	if isempty(nhgp0);nhgp0 = 20;end
	nvgp0=input('Enter number of coupler points in vertical direction [10]: ');
	if isempty(nvgp0);nvgp0 = 10;end

% save the data to the file fileinput.dat
	
	tempio(1)=tt;
	tempio(2)=r1;
	tempio(3)=r3;
	tempio(4)=r4;	
	tempio(5)=w2;
	tempio(6)=tl0;
	tempio(7)=tv0;
	tempio(8)=nhgp0;
	tempio(9)=nvgp0;
	tempiot=tempio(:);  % store the transpose of tempio for ease of reading
	fprintf(inputfile,'%f \n',tempiot)
end

% Read input data from the file hrcrankrockerio.dat

if readmode==1	
	hrcrankrockerio = fscanf(fid,'%f');
	tt=hrcrankrockerio(1);
	r1=hrcrankrockerio(2);
	r3=hrcrankrockerio(3);
	r4=hrcrankrockerio(4);
	w2=hrcrankrockerio(5);
	tl0=hrcrankrockerio(6);
	tv0=hrcrankrockerio(7);
	nhgp0=hrcrankrockerio(8);
	nvgp0=hrcrankrockerio(9);
end
fclose('all');
ans2='y';
while ans2=='y'
	
%Set up the Animation Grid
		
	fprintf('Animation grid\n')
	
	nvgp=nvgp0;
	nhgp=nhgp0;
	nvgpoc=nvgp+1;
	tl=tl0;
	tv=tv0;
	cxrc=(1/2)*r3;
	spacx=(1/(nhgp-1))*tl;
	spacy=(1/(nvgp-1))*tv;
	cxmin=cxrc-(tl/2);
	cxmax=cxrc+(tl/2);
	cymin=-(nvgpoc/2)*spacy;
	cymax=(nvgpoc/2)*spacy;
		
% Locate the first position of the linkage.  In this position, the crank 
% and frame are aligned so that r1, r2, r3 and r4 form a triangle.  Angle
% AA is between r3 and r4 and BB is between r4 and the r1.

	AA=acos(((r3.^2)+(r4.^2)-((r1+r2).^2))/(2*r4*r3));
	BB=asin((r3/(r1+r2))*sin(AA));
	CC=AA+BB;
	ax=r2*cos(CC);
	ay=r2*sin(CC);
		
% Set the first position so that the coupler is horizontal.  The left end 
% is at b1 and the right end is at c1.  The pivot for r4 is at d1.  The 
% pivot at the base of r2 is at a1 (0,0).

	b1=[ax ay];
	c1=[(r3+ax) ay];
	d1x=-r1*cos(CC);
	d1y=-r1*sin(CC);
	d1=[d1x d1y];
	a1=[0 0];
		
% Locate the coordinates of the bushing and revolute joints.

	bush1=bushing(r2/10,0,0,20,0);
	cir1=circle2(r2/10,0,0,20);
	bush2=bushing(r2/10,(-r1*cos(CC)),(-r1*sin(CC)),20,0);
	cir2=circle2(r2/10,(-r1*cos(CC)),(-r1*sin(CC)),20);
		
% Find the limits for the plots

	xmin1=cxmin+ax;
	xmin2=d1x*.95;
	xmin=min([xmin1 xmin2]);
	xmax1=cxmax+ax;
	xmax2=d1x*1.05;
	xmax=max([xmax1 xmax2]);
	ymin1=cymin+ay;
	ymin2=d1y*.95;
	ymin=min([ymin1 ymin2]);
	ymax=cymax+ay;
	xmina=min([xmin d1(1) b1(1) c1(1) min(bush1(:,1))  min(bush2(:,1))]);
	xmaxa=max([xmax d1(1) b1(1) c1(1) max(bush1(:,1)) max(bush2(:,1))]);
	ymina=min([ymin d1(2) b1(2) c1(2) min(bush1(:,2))  min(bush2(:,2))]);
	ymaxa=max([ymax d1(2) b1(2) c1(2) max(bush1(:,2)) max(bush2(:,2))]);
		
% Define the axes for the first plot.

	width=1.0;
	height=1.0;
	h7=axes('position',[0 0 width height],'box','on',...
	        'fontsize', 10, 'fontname', 'times');
	set(gcf,'color', 'w')
	rangex=xmaxa-xmina;
	rangey=ymaxa-ymina;
	xmin=xmina-0.05*rangex;
	xmax=xmaxa+0.05*rangex;
	ymin=ymina-0.05*rangey;
	ymax=ymaxa+0.05*rangey;
	values=axisadjust(xmin, xmax, ymin, ymax, width, height);
	xmin=values(1);
	xmax=values(2);
	ymin=values(3);
	ymax=values(4);
	axis equal
	axis([xmin xmax ymin ymax]);

% Compute the grid coordinates

	gxstart=cxmin-spacx;
	for i=1:nhgp
		xg(i)=b1(1)+gxstart+(i*spacx);
	end
	for i=1:nvgp
		ygy(i)=b1(2)+cymin+(i*spacy);
	end
		
% Analyze the linkage for each grid point
		
	for k=1:(nhgp*nvgp)
		gridpt(k)=line('xdata',[],'ydata',[],'marker','.',...
		          'markersize',8,'erasemode','none','color','k');
	end
	joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
		6, 'erasemode', 'xor', 'color', 'k');
	joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
		6, 'erasemode', 'xor', 'color', 'k');
	crank=line('xdata', [0 0], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
		'xor','color', 'k');
	rocker=line('xdata', [0 0], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
		'xor','color', 'k');
	coupler=line('xdata', [], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
		'xor','color', 'k');
	coupline1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
		'xor','color', 'k');
	coupline2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
		'xor','color','k');
	couplerptr=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
		'xor','color', 'k','linestyle',':');	     
	couplerpt=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
		'none','color', 'k');	   
	busho1=line('xdata',[],'ydata',[],'linewidth',1,'linestyle','-',...
	    'erasemode','none','color','r');
	circo1=line('xdata',[],'ydata',[],'linewidth',1,'linestyle','-',...
	    'erasemode','none','color','r');
	busho2=line('xdata',[],'ydata',[],'linewidth',1,'linestyle','-',...
	    'erasemode','none','color','r');
	circo2=line('xdata',[],'ydata',[],'linewidth',1,'linestyle','-',...
	    'erasemode','none','color','r');
	k=0;
		
% Draw each of the grid points on the coupler.

	for i=1:nhgp
		for j=1:nvgp
			k=k+1;
			set(gridpt(k),'xdata',xg(i),'ydata',ygy(j));
		end
	end
		
% Draw the linkage

	set(crank,'xdata',[0 b1(1)],'ydata',[0 b1(2)]);
	set(coupler,'xdata',[b1(1) c1(1)],'ydata',[b1(2) c1(2)]);
	set(rocker,'xdata',[c1(1) d1(1)],'ydata',[c1(2) d1(2)]);
	set(joint1,'xdata', b1(1), 'ydata',b1(2));
	set(joint2,'xdata', c1(1), 'ydata',c1(2));
	set(busho1,'xdata',bush1(:,1),'ydata',bush1(:,2));
	set(circo1,'xdata',cir1(:,1),'ydata',cir1(:,2));
	set(busho2,'xdata',(bush2(:,1)),'ydata',(bush2(:,2)));
	set(circo2,'xdata',(cir2(:,1)),'ydata',(cir2(:,2)));
	pause

% Select a grid row to animate to show coupler curves

	fprintf('Use mouse to indicate row to animate ');
	[xp,yp]=ginput(1);
	disp('   ')

% Identify row which is closest to y coordinate selected.
		
	for i=1:nvgp
		if ygy(i)>yp;break;end
	end
	if i==1
		yg1=ygy(i);
		ngg=1;
	else
		if (yp-ygy(i-1))<(ygy(i)-yp)
		yg1=ygy(i-1);
		ngg=i-1;
	else
		yg1=ygy(i);
		ngg=i;
		end
	end
	yg=yg1*ones(size(xg));
	xga=xg;
	yga=yg;
	xg=xga-ax;
	yg=yga-ay;
	
% Locate each coupler point by rr and ppheta

	rr=sqrt((xg.^2)+(yg.^2));
	ppheta=atan2(yg,xg);
	clf;
	disp('I am working ...');
		
% Locate the first position of the crank

	Q1=(180/pi)*(pi+CC);
	Q11=pi+CC;
	gp=1; 	   
     
% Define constants
		
	i=0;
	d2r=pi/180;
	r2d=180/pi;
	mode=-1;
	dt=5;
	nhgpa=nhgp;
	nvgpa=nvgp;
	nvgp=1;
	Qs=Q1;
	if Qs<0
		Qs=Qs+180;
	elseif Qs>180
		Qs=Qs-180;
	end

% Loop to analyze the linkage for one cycle

	for Q21 = Qs:dt:360+Qs;
		i=i+1;
		Q2 = Q21*d2r;

% Call fourbar function to compute the position
		  
		para = fourbar(r1,r2,r3,r4,Q21,w2,0,mode,Q1,2); 
		Q3=para(7);
		Q33(i)=Q3;
		Q4=para(8);
		Q44(i)=Q4;
		w33(i)=para(11);
        w44(i)=para(12);
        assem=para(29);
			
% Store position information in matrices.
			
        Bx(i)= para(17);
        By(i)= para(18);
        Cx(i)= para(19);
        Cy(i)= para(20);
		vbx(i)=para(21);
		vby(i)=para(22);
		vcx(i)=para(23);
		vcy(i)=para(24);
		for j=1:nhgp
			mx(j,i)=Bx(i)+rr(j)*cos((para(7)*pi/180)+ppheta(j));
			my(j,i)=By(i)+rr(j)*sin((para(7)*pi/180)+ppheta(j));
		end
		Q22(i) = Q21;
	end
	clf;
	itotal=i;
	ans='y';
	hh=0;		
	while hh<2

% Draw linkage full size

		width=0.95;
		height=0.95;
		h6=axes('position',[.05 .05 width height],'box','on',...
		        'fontname', 'times', 'fontsize', 10);  
		joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
			6, 'erasemode', 'xor', 'color', 'k');
		joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
			6, 'erasemode', 'xor', 'color', 'k');
		crank=line('xdata', [], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
			'xor','color', 'k');
		rocker=line('xdata', [], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
			'xor','color', 'k');
		coupler=line('xdata', [], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
			'xor','color', 'k');	   
		coupline1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
			'xor','color', 'k');
		coupline2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
			'xor','color', 'k');
		busho1=line('xdata',[],'ydata',[],'linewidth',1,'linestyle','-',...
		     'erasemode','none','color','r');
		circo1=line('xdata',[],'ydata',[],'linewidth',1,'linestyle','-',...
		     'erasemode','none','color','r');
		busho2=line('xdata',[],'ydata',[],'linewidth',1,'linestyle','-',...
		     'erasemode','none','color','r');
		circo2=line('xdata',[],'ydata',[],'linewidth',1,'linestyle','-',...
		     'erasemode','none','color','r');

		for k=gp:(gp+nhgp-1)
			gridpoint(k)=line('xdata',[],'ydata',[],'marker','o','markersize',6,...
			  'erasemode','xor','color','k');
			for j=1:itotal
				gridline(k,j) =line('xdata',[],'ydata',[],'linewidth',1,...
				      'linestyle','-','erasemode','none','color','r');
			end
		end
		xmina=min([min(min(mx(gp:(gp+nhgp-1),:))) min(Bx) min(Cx)... 
		     d1(1) min(bush1(:,1)) min(bush2(:,1)) -tl/2]);
		xmaxa=max([max(max(mx(gp:(gp+nhgp-1),:))) max(Bx) max(Cx)... 
		     d1(1) max(bush1(:,1)) max(bush2(:,1)) tl/2]);
		ymina=min([min(min(my(gp:(gp+nhgp-1),:))) min(By) min(Cy)...
		     d1(2) min(bush1(:,2)) min(bush2(:,2))]);
		ymaxa=max([max(max(my(gp:(gp+nhgp-1),:))) max(By) max(Cy)...
		     d1(2) max(bush1(:,2)) max(bush2(:,2))]);
		rangex=xmaxa-xmina;
		rangey=ymaxa-ymina;
		xmin=xmina-0.05*rangex;
		xmax=xmaxa+0.05*rangex;
		ymin=ymina-0.05*rangey;
		ymax=ymaxa+0.05*rangey;
		values=axisadjust(xmin, xmax, ymin, ymax, width, height);
		xmin=values(1);
		xmax=values(2);
		ymin=values(3);
		ymax=values(4);
		axis equal
		axis([xmin xmax ymin ymax]);
		hh=hh+1;

% Write out the link lengths

		r1t=['r1 = ',num2str(r1)];
		r2t=['r2 = ',num2str(r2)];
		r3t=['r3 = ',num2str(r3)];
		r4t=['r4 = ',num2str(r4)];
		yyy1=ymax-0.03*(ymax-ymin);
		text(xmin+(xmax-xmin)*.2,yyy1,r1t,'fontsize', 10, 'fontname', 'times')
		text(xmin+(xmax-xmin)*.35,yyy1,r2t,'fontsize', 10, 'fontname', 'times')
		text(xmin+(xmax-xmin)*.5,yyy1,r3t,'fontsize', 10, 'fontname', 'times')
		text(xmin+(xmax-xmin)*.65,yyy1,r4t,'fontsize', 10, 'fontname', 'times')
		
% Write the coupler point location if only one curve is displayed

		if hh==2
			r6=sqrt((Bx(1)-mx(gp,1)).^2+(By(1)-my(gp,1)).^2);
			beta=(180/pi)*atan2((my(gp,1)-By(1)),(mx(gp,1)-Bx(1)));
			r6t=['BE = ',num2str(r6)];
			betat=['Beta = ',num2str(beta)];
			yyy2=yyy1-0.1*(ymax-ymin);
			text(xmin+(xmax-xmin)*.1,yyy2,r6t,'fontsize', 10, 'fontname', 'times')
			text(xmin+(xmax-xmin)*.35,yyy2,betat,'fontsize', 10, 'fontname', 'times')
		end
	
% Animate the linkage

		timesink=1;
		while ans=='y'
			spac=[.02 .3 .6 .8];
			xx=xmin+spac*(xmax-xmin);
			yy=ymin;
			
% Draw the linkage and coupler curves

			for i=1:itotal
				if i>1	
				
% Draw the row of coupler points for hh=0 and draw just one point if hh=1.
	    
					for k=gp:(gp+nhgp-1)
						midx(i)=mx(k,i-1)+(mx(k,i)-mx(k,i-1))/2;
						midy(i)=my(k,i-1)+(my(k,i)-my(k,i-1))/2;
						set(gridline(k,i),'xdata',[mx(k,i-1) midx(i)],...
						         'ydata',[my(k,i-1) midy(i)]);
					end
				end
				set(busho1,'xdata',bush1(:,1),'ydata',bush1(:,2));
				set(busho2,'xdata',bush2(:,1),'ydata',bush2(:,2));
				set(circo1,'xdata',cir1(:,1),'ydata',cir1(:,2));
				set(circo2,'xdata',cir2(:,1),'ydata',cir2(:,2));
				set(joint1,'xdata', Bx(i), 'ydata',By(i));
				set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
				set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
				set(rocker,'xdata',[(r1*cos(Q11)) Cx(i)],'ydata', [(r1*sin(Q11)) Cy(i)]);
				set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
				for k=gp:(gp+nhgp-1)
					set(gridpoint(k),'xdata', mx(k,i), 'ydata',my(k,i));
				end

% Draw coupler lines if hh=2

				if hh==2
					set(coupline1, 'xdata', [Bx(i) mx(k,i)], 'ydata',[By(i) my(k,i)]);
					set(coupline2, 'xdata', [Cx(i) mx(k,i)], 'ydata',[Cy(i) my(k,i)]);
				end
				drawnow; %flush the draw buffer
				
% Do meaningless calculation to slow down the linkage

				for k=1:1:timesink
					zzz=log10(k);
				end
			end   
			disp('   ')
			ans=input('repeat animation? y/n [y]: ','s');
			if isempty(ans);ans='y';end
			if ans=='y'
				rep=input('Change animation speed? (+/-/0) [-]:','s');
				if isempty(rep);rep='-';end
				if rep=='+'; timesink=timesink/5; end
				if rep=='-'; timesink=timesink*5; end
				if timesink<1; timesink=1; end
			end
		end
		ans='y';
			
% Select one point to animate alone.

		if hh<2
			fprintf('Use mouse to input point to animate');
			[xp,yp]=ginput(1);
			disp('   ')
		
% Identify the point selected by locating the x coordinate of 
% the grid points nearest to selected point

			for i=1:nhgp
				if mx(i)>xp;break;end
			end
			if i==1
				mx1=mx(1);
				gp=1;
			else
				if (xp-mx(i-1))<(mx(i)-xp) 
					mx1=mx(i-1);
					gp=i-1;
				else
					mx1=mx(i);
					gp=i;
				end
			end
			nhgp=1;
			clf
		end
	end 

	clf
	r6=sqrt((Bx(1)-mx(gp,1)).^2+(By(1)-my(gp,1)).^2);
	beta=(180/pi)*atan2((my(gp,1)-By(1)),(mx(gp,1)-Bx(1)));
	for i=1:itotal
		para=rbody1(r6,beta,Bx(i),By(i),Cx(i),Cy(i),vbx(i),vby(i),...
		     vcx(i),vcy(i),0,0,0,0,2);
		vmx=para(3);
		vmy=para(4);
		vm(i)=sqrt((vmx.^2)+(vmy.^2));
	end 

% Define the axes for the first graph (input/output angle graph)
		
	width=0.38;
	height=0.35;
	h1=axes('position',[0.6 .55 width height],'box','on','xcolor','k','ycolor',...
      'k', 'fontsize', 10, 'fontname', 'times');   %axis location
	set(gcf,'color', 'w')
	xlabel('Crank Angle','color', 'k')
	ylabel('Rocker Angle','color', 'k')
	Displ=line('xdata', [], 'ydata', [], 'erasemode', 'none',...
		'color', 'r', 'linestyle', '-');
	dbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, 'erase',...
		'xor');

% Define the axes for the second graph (rocker velocity graph)
		
	h2=axes('position', [0.6 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k', 'fontsize', 10, 'fontname', 'times');
	set(gcf,'color', 'w')
	xlabel('Crank Angle','color', 'k')
	ylabel('Rocker Velocity','color', 'k')
	Vel=line('xdata', [], 'ydata', [], 'erasemode', 'none', ...
		'color','r', 'linestyle', '-');
	vbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, ...
		'erase','xor');
 
% Define the axes for the third graph (crank angle graph)
		
	h3=axes('position', [0.1 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k', 'fontsize', 10, 'fontname', 'times');
	set(gcf,'color', 'w')
	xlabel('Crank Angle','color', 'k')
	ylabel('Grid Point Velocity','color', 'k')
	gpvel=line('xdata', [], 'ydata', [], 'erasemode','none', ...
		'color', 'r', 'linestyle', '-');
	tbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, 'erase','xor');
 
% Define the axes for the fourth graph (linkage position)
		
	h4=axes('position', [0.1 .55 width height],'box', 'on', 'xcolor', 'k','ycolor',...
       'k', 'fontsize', 10, 'fontname', 'times');
	set(gcf,'color', 'w')
	axis equal;
	axis([xmin xmax ymin ymax]);
   
% Set up the vectors needed to draw the linkage
		
	joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
		6, 'erasemode', 'xor', 'color', 'k');
	joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
		6, 'erasemode', 'xor', 'color', 'k');
	crank=line('xdata', [0 0], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
		'xor','color', 'k');
	coupler=line('xdata', [], 'ydata' ,[], 'linewidth',2, 'erasemode', ...
		'xor','color', 'k');	   	      
	coupline1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
		'xor','color','k');
	coupline2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
		'xor','color', 'k');
	rocker=line('xdata', [r1*cos(Q1) 0], 'ydata' ,[r1*sin(Q1) 0], 'linewidth',2,...
		'erasemode', 'xor','color', 'k');
	for k=gp:(gp+nhgp-1)
		for j=1:itotal
			gridline(k,j) =line('xdata',[],'ydata',[],'linewidth',1,'linestyle',...
			     '-','erasemode','none','color','r');
		end
	end
	gridpoint=line('xdata',[],'ydata',[],'marker','o','markersize',6,...
	        'erasemode','xor','color','k');
	bushy1=line('xdata', bush1(:,1), 'ydata',bush1(:,2), 'linestyle', '-',...
	        'markersize', 6,...
		'color', 'r');
	bushy2=line('xdata', bush2(:,1), 'ydata',bush2(:,2), 'linestyle', '-', ...
		'markersize', 6, 'color', 'r');
	circ1=line('xdata', cir1(:,1), 'ydata',cir1(:,2), 'linestyle', '-',...
	    'markersize', 6,'color', 'r');
	circ2=line('xdata', cir2(:,1), 'ydata',cir2(:,2), 'linestyle', '-', ...
		'markersize', 6, 'color', 'r'); 
	xmina=min([min(min(mx(k,:))) min(Bx) min(Cx)... 
		    min(bush1(:,1)) min(bush2(:,1))]);
	xmaxa=max([max(max(mx(k,:))) max(Bx) max(Cx)... 
		     max(bush1(:,1)) max(bush2(:,1))]);
	ymina=min([min(min(my(k,:))) min(By) min(Cy)...
		     min(bush1(:,2)) min(bush2(:,2))]);
	ymaxa=max([max(max(my(k,:))) max(By) max(Cy)...
		     max(bush1(:,2)) max(bush2(:,2))]);
	rangex=xmaxa-xmina;
	rangey=ymaxa-ymina;
	xmin=xmina-0.05*rangex;
	xmax=xmaxa+0.05*rangex;
	ymin=ymina-0.05*rangey;
	ymax=ymaxa+0.05*rangey;
	values=axisadjust(xmin, xmax, ymin, ymax, width, height);
	xmin=values(1);
	xmax=values(2);
	ymin=values(3);
	ymax=values(4);
	axis equal
	axis([xmin xmax ymin ymax]);
				   
% Set the axis limits for the graphs
		
	axes(h3);
	axis([Qs (Qs+360) min(vm) max(vm)]);  %set axis limits for the torque plot
	axes(h1);
	axis([Qs (Qs+360) min(Q44) max(Q44)]);  %set axis limits for the rocker angle plot
	axes(h2);
	axis([Qs (Qs+360) min(w44) max(w44)]);  %set axis limits for the velocity plot
 
% Plot the results.
		
	itotal=i;
	ans='y';
	
% Draw the fixed curves first.
		
	set(gpvel,'xdata', Q22,'ydata', vm);
	set(Displ,'xdata', Q22,'ydata', Q44);
	set(Vel,'xdata', Q22,'ydata', w44);
	for i=1:itotal
		if i>1
			set(gridline(k,i),'xdata',[mx(k,i-1) midx(i)], 'ydata',[my(k,i-1) midy(i)]); 
		end
	end
 
% Animate the results as long as ans = 'y'

	while ans=='y'
        for mt=1:1:tt
			for i=1:itotal
				set(joint1,'xdata', Bx(i), 'ydata',By(i));
				set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
				set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
				set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
				set(coupline1, 'xdata', [Bx(i) mx(k,i)], 'ydata',[By(i) my(k,i)]);
				set(coupline2, 'xdata', [Cx(i) mx(k,i)], 'ydata',[Cy(i) my(k,i)]);			
				set(rocker,'xdata', [(r1*cos(Q11)) Cx(i)],'ydata', ...
					[(r1*sin(Q11)) Cy(i)]);
				set(gridpoint,'xdata', mx(k,i), 'ydata',my(k,i));
				set(tbead, 'xdata', Q22(i), 'ydata', vm(i));
				set(dbead, 'xdata', Q22(i), 'ydata', Q44(i));
				set(vbead, 'xdata', Q22(i), 'ydata', w44(i));
				drawnow; %flush the draw buffer
						
% Do meaningless calculation to slow down the linkage

				for ktime=1:1:timesink
					zzz=log10(ktime);
				end 
			end
		end

% Plot figures in final position.
			
		i = 6;

% Plot position first
			
		set(joint1,'xdata', Bx(i), 'ydata',By(i));
		set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
		set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
		set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
		set(coupline1, 'xdata', [Bx(i) mx(k,i)], 'ydata',[By(i) my(k,i)]);
		set(coupline2, 'xdata', [Cx(i) mx(k,i)], 'ydata',[Cy(i) my(k,i)]);
		set(rocker,'xdata', [(r1*cos(Q11)) Cx(i)],'ydata', ...
			[(r1*sin(Q11)) Cy(i)]);
		set(gridpoint,'xdata', mx(k,i), 'ydata',my(k,i));

% Plot grid-point velocity information

		set(tbead, 'xdata', Q22(i), 'ydata', vm(i));
 
% Plot angle information
			
		set(dbead, 'xdata', Q22(i), 'ydata', Q44(i));
 
% Plot velocity information

		set(vbead, 'xdata', Q22(i), 'ydata', w44(i));
 
%flush the draw buffer

		drawnow; 
 
 % Ask if the animation should be repeated
 
		ans=input('Repeat animation? y/n [y]: ', 's');
		if isempty(ans);ans='y';end
		if ans=='y'
			rep=input('Change animation speed? (+/-/0) [-]:','s');
			if isempty(rep);rep='-';end
			if rep=='+'; timesink=timesink/10; end
			if rep=='-'; timesink=timesink*10; end
			if timesink<1; timesink=1; end
		end
	end
	ans2=input('Repeat animation for a new set of grid points? y/n [y]: ','s');
	if isempty(ans2);ans2='y';end
	if ans2=='y'; clf; end
end
