function [coords] = inflection1(Ax, Ay, Ix, Iy, JAx, JAy, gamma,JX,JY,flag)

% MATLAB m-file for inflection circle calculations when point, pole,
% and JA coordinates are known and Astar is to be found.  In addition, 
% either the gamma angle or J are known.  If gamma is known, J and JA
% are to be found.  If J is known, gamma and JA are to be found.

% Input Variables
%   Ax=X coordinate of point
%   Ay=Y coordinate of point
%   Ix=X coordinate of instant center
%   Iy=Y coordinate of instant center
%   JAx=X coordinate of JA on inflection circle
%   JAy=Y coordinate of JA on inflection circle
%	gamma = angle, measured CCW about I, from the diameter of the inflection
%           circle (ray IJ) to the ray IA
%	JX = x coordinate of J
% 	JY = y coordinate of J
%	R = radius of inflection circle
%	flag =	calculation flag.  If flag=0, gamma is not know.  If flag = 1, gamma
%			is known, and the inflection circle and J can be found.  If flag = 2, J
% 			is known, and the inflection circle and gamma are to be found.		

% Output Variables 
%   coords(1) = X coordinate of Astar.
%   coords(2) = y coordinate of Astar.
%	coords(3) = x coordinate of J 
%	coords(4) = y coordinate of J
%	coords(5) = gamma(degrees)
%	coords(6) = radius of inflection circle
%	coords(7) = x coordinate of center of inflection circle.
%	coords(8) = y coordinate of center of inflection circle.

%Program
  
coords=zeros(1,8);
rAdJAx=(Ax-JAx);
rAdJAy=(Ay-JAy);
rAdIx=(Ax-Ix);
rAdIy=(Ay-Iy);
rAdI=sqrt(rAdIx^2+rAdIy^2);
rAdJA=sqrt(rAdJAx^2+rAdJAy^2);

% Compute positive direction when A is JA.

dirx=sqrt(2);
diry=dirx;
if rAdI~=0;
	dirx=rAdIx/rAdI;
	diry=rAdIy/rAdI;
end

% Compute Astar and positive direction when A is arbitrary.

rAdAstar=10^10*min([JAx, JAy, Ix, Iy, Ax, Ay]);
if rAdJA ~= 0;
	rAdAstar=rAdI^2/rAdJA;
	dirx=rAdJAx/rAdJA;
	diry=rAdJAy/rAdJA;
end

% Compute the coordinates of Astar.

Astarx = Ax-rAdAstar*dirx;
Astary = Ay-rAdAstar*diry;
coords(1)=Astarx;
coords(2)=Astary;
if flag==1
	angle=atan2((JAy-Iy), (JAx-Ix));
	beta = angle-gamma*pi/180;
	if angle ==pi/2
		disp('   ')
		disp('  ****** Error - A and I are coincident ******')
	end
	if angle ~=pi/2
		JAdI=sqrt((JAx-Ix)^2+(JAy-Iy)^2);
		JdI=JAdI/cos(gamma*pi/180);
		Jx=Ix+JdI*cos(beta);
		Jy=Iy+JdI*sin(beta);
		R=abs(JdI/2);
		Cx=Ix+JdI*(0.5)*cos(beta);
		Cy=Iy+JdI*(0.5)*sin(beta);
	end
	coords(3)=Jx;
	coords(4)=Jy;
	coords(5)=gamma;
	coords(6)=R;
	coords(7)=Cx;
	coords(8)=Cy;
end
if flag==2
	angle=atan2((JAy-Iy), (JAx-Ix));
	beta =atan2((JY-Iy), (JX-Ix));
	gamma=angle-beta;
	if angle ==pi/2
		disp('   ')
		disp('  ****** Error - A and I are coincident ******')
	end
	if angle ~=pi/2
		JdI=sqrt((JX-Ix)^2+(JY-Iy)^2);
		JAdI=JdI/cos(gamma);
		R=abs(JdI/2);
		Jx=Ix+JdI*cos(beta);
		Jy=Iy+JdI*sin(beta);
		Cx=Ix+JdI*(0.5)*cos(beta);
		Cy=Iy+JdI*(0.5)*sin(beta);
	end
	coords(3)=Jx;
	coords(4)=Jy;
	coords(5)=gamma*180/pi;
	coords(6)=R;
	coords(7)=Cx;
	coords(8)=Cy;
end
	
	
	
	

