function [coords] = inflection2(Ax, Ay, Ix,Iy,Astarx,Astary,gamma,JX,JY,flag)

% MATLAB m-file for inflection circle calculations when point, pole, and
% Astar coordinates are known.  In addition, either the gamma angle or J 
% are known. If gamma is known, J and JA are to be found.  If J is known, 
% gamma and JA are to be found.

% Input Variables
%   Ax=X coordinate of point
%   Ay=Y coordinate of point
%   Ix=X coordinate of instant center
%   Iy=Y coordinate of instant center
%   Astarx=X coordinate of curvature of point path of A.
%   Astary=Y coordinate of curvature of point path of A.
%	gamma = angle, measured CCW about I, from the diameter of the inflection
%           circle (ray IJ) to the ray IA
%	JX = x coordinate of J
% 	JY = y coordinate of J
%	R = radius of inflection circle
%	flag =	calculation flag.  If flag=0, gamma is not know.  If flag = 1, gamma
%			is known, and the inflection circle and J can be found.  If flag = 2, J
% 			is known, and the inflection circle and gamma are to be found.		

% Output Variables 
%   coords(1) = X coordinate of JA.
%   coords(2) = y coordinate of JA.
%	coords(3) = x coordinate of J 
%	coords(4) = y coordinate of J
%	coords(5) = gamma(degrees)
%	coords(6) = radius of inflection circle
%	coords(7) = x coordinate of center of inflection circle.
%	coords(8) = y coordinate of center of inflection circle.

%Program

coords=zeros(1,8);
rAdAstarx=(Ax-Astarx);
rAdAstary=(Ay-Astary);
rAdIx=(Ax-Ix);
rAdIy=(Ay-Iy);
rAdI=sqrt(rAdIx^2+rAdIy^2);
rAdAstar=sqrt(rAdAstarx^2+rAdAstary^2);

% Check for A=Astar

if rAdAstar ==0;
	disp('   ')
	disp('  ****** Error - A and Astar are coincident ******')
end

% Compute rAdJA and positive direction when A is arbitrary.

rAdJA=10^10*min([Astarx, Astary, Ix, Iy, Ax, Ay]);
if rAdAstar ~= 0;
	rAdJA=rAdI^2/rAdAstar;
	dirx=rAdAstarx/abs(rAdAstar);
	diry=rAdAstary/abs(rAdAstar);
end

% Compute the coordinates of JA.

JAx = Ax-rAdJA*dirx;
JAy = Ay-rAdJA*diry;
coords(1)=JAx;
coords(2)=JAy;

if flag==1
	angle=atan2((JAy-Iy), (JAx-Ix));
	beta = angle-gamma*pi/180;
	if angle ==pi/2
		disp('   ')
		disp('  ****** Error - A and I are coincident ******')
	end
	if angle ~=pi/2
		JAdI=sqrt((JAx-Ix)^2+(JAy-Iy)^2);
		JdI=JAdI/cos(gamma*pi/180);
		Jx=Ix+JdI*cos(beta);
		Jy=Iy+JdI*sin(beta);
		R=abs(JdI/2);
		Cx=Ix+JdI*(0.5)*cos(beta);
		Cy=Iy+JdI*(0.5)*sin(beta);
	end
	coords(3)=Jx;
	coords(4)=Jy;
	coords(5)=gamma;
	coords(6)=R;
	coords(7)=Cx;
	coords(8)=Cy;
end
if flag==2
	angle=atan2((JAy-Iy), (JAx-Ix));
	beta =atan2((JY-Iy), (JX-Ix));
	gamma=angle-beta;
	if angle ==pi/2
		disp('   ')
		disp('  ****** Error - A and I are coincident ******')
	end
	if angle ~=pi/2
		JdI=sqrt((JX-Ix)^2+(JY-Iy)^2);
		JAdI=JdI/cos(gamma);
		R=abs(JdI/2);
		Jx=Ix+JdI*cos(beta);
		Jy=Iy+JdI*sin(beta);
		Cx=Ix+JdI*(0.5)*cos(beta);
		Cy=Iy+JdI*(0.5)*sin(beta);
	end
	coords(3)=Jx;
	coords(4)=Jy;
	coords(5)=gamma*180/pi;
	coords(6)=R;
	coords(7)=Cx;
	coords(8)=Cy;
end
	
	
	
	
