function [values] = isldcrkc(r1,r2,r4,theta2,td2,tdd2,theta1,flag)

% This function analyzes a slider crank inversion when the crank
% is the driving link.  The input values are:

%r1      = length of vector 1 (frame)
%r2      = length of vector 2 (crank)
%r4      = length of vector 4 (slider offset)
%theta2  = crank angle (degrees)
%td2     = crank angular velocity (rad/sec)
%tdd2    = crank angular acceleration (rad/sec^2)
%theta1  = angle from frame x axis to line through frame pivots (degrees).
%flag    = analysis flag.  If flag = 1, only a position analysis is conducted. 
%          If flag = 2, both a position and velocity analysis is conducted. 
%		   If flag = 3, a position, velocity, and acceleration analysis
%		               is conducted.

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

%values (1:4)   = vector lengths
%values (5:8)   = vector angles (degrees)
%values (9:12)  = derivatives of vector lengths
%values (13:16) = derivatives of vector angles (rad/sec)
%values (17:20) = second derivatives of vector lengths
%values (21:24) = second derivatives of vector angles (rad/sec^2)
%values (25:26) = x,y components of position of slider pin (point P)
%values (27:28) = x,y components of position of offset point (point Q)
%values (29:30) = x,y components of velocity of slider pin (point P)
%values (31:32) = x,y components of velocity of offset point (point Q)
%values (33:34) = x,y components of acceleration of slider pin (point P)
%values (35:36) = x,y components of acceleration of offset point (point Q)
%values (37)    = assembly flag.  If values(37) = 0, mechanism cannot 
%                 assembled.

%convert input data

values=zeros(37,1);
r(2)=r2;
r(1)=r1;
r(4)=r4;
theta(1)=theta1;
theta(2)=theta2;
fact=pi/180;
t1=theta(1)*fact;
t2=theta(2)*fact;
td=zeros(4,1);
tdd=zeros(4,1);
rd=zeros(4,1);
rdd=zeros(4,1);
td(2)=td2;
tdd(2)=tdd2;
s2=sin(t2);
c2=cos(t2);
s1=sin(t1);
c1=cos(t1);

% position calculations

r(3)=sqrt(r(2)*r(2)+r(1)*r(1)-r(4)*r(4)-2*r(1)*r(2)*(c1*c2+s1*s2));
A=r(2)*c2-r(1)*c1;
arg1=r(4)*r(4)+r(3)*r(3)-A*A;
if (arg1>=0)
	values(37)=1; %assembly flag
	
% find sign for square root 

	t31=(2*atan((r(4)+sqrt(arg1))/(A+r(3))));
	t32=(2*atan((r(4)-sqrt(arg1))/(A+r(3))));
	er1=r(2)*s2-r(1)*s1-(r(3)*sin(t31)-r(4)*cos(t31));
	er2=r(2)*s2-r(1)*s1-(r(3)*sin(t32)-r(4)*cos(t32));
	theta(3)=t31/fact;
	t3=t31;
	beta=1;
	if(abs(er1)>abs(er2))
		beta=-1;
		theta(3)=t32/fact;
		t3=t32;
	end	
	s3=sin(t3);
	c3=cos(t3);
	t4=t3-pi/2;
	theta(4)=t4/fact;
	c4=cos(t4);
	s4=sin(t4);
	
	% coordinates of P and Q
	
	values(25)=r(2)*c2;
	values(26)=r(2)*s2;
	values(27)=r(1)*c1+r(4)*c4;
	values(28)=r(1)*s1+r(4)*s4;

	% velocity calculation
	
	if flag>1
		AM=[c3, -r(3)*s3-r(4)*s4; s3, r(3)*c3+r(4)*c4];
		BM=[-r(2)*td(2)*s2;r(2)*td(2)*c2];
		CM=AM\BM;
		rd(3)=CM(1);
		td(3)=CM(2);
	
		% velocities of P and Q
	
		values(29)=-r(2)*td(2)*s2;
		values(30)=r(2)*td(2)*c2;
		values(31)=-r(4)*td(3)*s4;
		values(32)=r(4)*td(3)*c4;
	end
	
	%acceleration calculation
	
	if flag>2
		BM=[-r(2)*tdd(2)*s2-r(2)*td(2)^2*c2+r(3)*td(3)^2*c3+...
		    2*rd(3)*td(3)*s3+r(4)*td(3)*td(3)*c4;...
    		r(2)*tdd(2)*c2-r(2)*td(2)*td(2)*s2+r(3)*td(3)*td(3)*s3-...
		    2*rd(3)*td(3)*c3+r(4)*td(3)*td(3)*s4];
		CM=AM\BM;
		rdd(3)=CM(1);
		tdd(3)=CM(2);
		
		% accelerations of P and Q
	
		values(33)=-r(2)*tdd(2)*s2-r(2)*td(2)*td(2)*c2;
		values(34)=r(2)*tdd(2)*c2-r(2)*td(2)*td(2)*s2;
		values(35)=-r(4)*tdd(3)*s3-r(4)*td(3)*td(3)*c4;
		values(36)=+r(4)*tdd(3)*c3-r(4)*td(3)*td(3)*s4;
	end
end

%store results in array values

for i = 1:4
	values(i)=r(i);
	values(i+4)=theta(i);
	values(i+8)=rd(i);
	values(i+12)=td(i);
	values(i+16)=rdd(i);
	values(i+20)=tdd(i);
end
