% off_cam.m 
%
% This program determines the cam profile for an oscillating flat-
% faced follower.  

% Variables for calculation 
%   rb=base circle radius
%   r1=distance from cam to follower pivot
%	 r3=length of follower face
%	 d = follower offset
%   xc=x coordinate of cam point
%   yc=y coordinate of cam point
%   rised = maximum follower displacement in degrees
%   rise = maximum follower displacement in radians

%   Graphics Variables

%   h1=handle on the first axes in the window (cam animation)
%   h2=handle on the second axes in the window (displacement diagram)
%   h3=handle on the fourth axes in the window (pressure angle)
%   Displ=line representing the follower displacement
%   dbead=bead which follows the displacement throughout the animation
%   Pangle=line representing the pressure angle
%   pabead=bead which follows pressure angle throughout the animation
%   stem=line for follower
%   cam=line for cam
%   pin1=line for the cam axis
%   bush1=line representing the cam bushing
%   pin2=line for the follower axis
%   bush2=line representing the follower bushing
%   xmin, xmax= x limits for the axes
%   ymin, ymax= y limits for the axes

% program 

clear all;
clf;
ansi='y';
ans='y';
disp('   ')
disp('      Cam Synthesis for Oscillating Flat-Faced Follower')
disp('  ')
fact=pi/180;

% Input values for cam variables 

readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (off_camio.dat): ', 's');
if isempty(inputfile); inputfile = 'off_camio.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
if readmode==2
	while ansi=='y'
		rb=input('Enter base circle radius [2]:  ');
		if isempty(rb); rb=2; end
		r1=input('Enter distance between fixed pivots [6]:  ');
		if isempty(r1); r1=6; end
		if r1<rb
			disp ('r1 must be larger than rb  ')
			disp (' ')
			r1=input('Enter distance between fixed pivots [5*rb]:  ');
			if isempty(r1); r1=5*rb; end
		end
		r3=input('Length of follower face [1.5*r1]:  ');
		if isempty(r3); r3=1.5*r1; end
		d=input('Enter follower offset [0.5]:  ');
		if isempty(d); d=0.5; end
		rised=input('Enter follower rise (deg) [15]:  ');
		if isempty(rised); rised=15; end
		direction=input('Enter cam rotation direction (CW(-), CCW(+)) [-]: ','s'); 
		if isempty(direction); direction='-';end
		ainc=input('Enter angle increment for design (deg) [10]:  '); 
		if isempty(ainc); ainc=10; end
		ncycle=input('Enter number of animation cycles [4]:  ');
		if isempty(ncycle); ncycle=4; end

		ansi=input('Reinter input variables? y/n [n]: ', 's');
		if isempty(ansi);
			ansi='n';
		end
	end
	
% save the data to the file fileinput.dat
	
	tempio(1)=rb;
	tempio(2)=r1;
	tempio(3)=r3;	
	tempio(4)=d;
	tempio(5)=rised;
	tempio(6)=direction;
	tempio(7)=ainc;
	tempio(8)=ncycle;
	tempio(9)=ansi;	
	tempiot=tempio(:);  % store the transpose of tempio for ease of reading
	fprintf(inputfile,'%f \n',tempiot)
end

% Read input data from the file off_camio.dat

if readmode==1	
	off_camio = fscanf(fid,'%f');
	rb=off_camio(1);
	r1=off_camio(2);
	r3=off_camio(3);
	d=off_camio(4);
	rised=off_camio(5);
	direction=off_camio(6);
	ainc=off_camio(7);
	ncycle=off_camio(8);
	ansi=off_camio(8);
end
fclose('all');
rise=rised*fact;

% Print heading for input information

disp('   ')
fprintf('                         Input Variables ') 

disp('   ')
fprintf('  Base Rad.  Pivot Dist.  Follower offset  Follower Length    rise') 

% Print the input values.

disp('   ')
fprintf('%9.3f %11.3f %15.3f %15.3f %13.3f \n',rb, r1, d, r3, rised)
disp('   ')

% Compute phi0.

ae=r1/(1+d/rb);
be=sqrt(ae^2-rb^2);
phi0=atan2(rb, be);

% Compute the angular displacement of the flat-faced follower.  The variable
% dtheta will change the resolution of the cam.  Make it smaller for
% more accurate results.
		
dtheta=ainc;
if direction=='+'
	dtheta=-ainc;
end
temp=-dtheta;
n=abs(360/dtheta);

% Compute the follower displacement and follower center coordinates 
% relative to cam.

for j=1:1:n
	temp=temp+dtheta;
	theta(j)=temp;
	camangle(j)=abs(theta(j));
	[coords]=o_follower2(camangle(j),rise);
	f(j)=coords(1);
	fd(j)=f(j)/fact;
	tt=theta(j)*fact;
	if tt<0; tt=tt+2*pi; end
	mu=3*pi/2+tt-f(j)-phi0;
	beta(j)=tt-f(j)-phi0;
	cm=cos(mu);
	sm=sin(mu);
	ct=cos(tt);
	st=sin(tt);
	XC(j)=r1*ct;
	YC(j)=r1*st;
	XD(j)=XC(j)+d*cos(mu);
	YD(j)=YC(j)+d*sin(mu);
end

% Plot the displacement profile, velocity and acceleration for 
% the follower.  Start by normalizing f, fp, and fpp

temp=-1;
for j=1:1:360
	temp=temp+1;
	thetal(j)=temp;
	[coords]=follower(thetal(j),rise);
	fl(j)=coords(1);	
	flp(j)=coords(2);
	flpp(j)=coords(3);
end

fmax=max(fl);
fmin=min(fl);
fpmax=max(flp);
fpmin=min(flp);
fppmax=max(flpp);
fppmin=min(flpp);
fn=max([abs(fmax), abs(fmin)]);
fpn=max([abs(fpmax), abs(fpmin)]);
fppn=max([abs(fppmax), abs(fppmin)]);
fnorm=fl/fn;
fpnorm=flp/fpn;
fppnorm=flpp/fppn;

% Set up the axes for displacement drawing.

clf;
width=0.9;
height=0.8;
h6=axes('position', [.1 .1 width height],'box', 'on', 'xcolor',...
       'k','ycolor','k');
set(gcf,'color', 'w')
xlabel('Cam Angle','color', 'k')
ylabel('Follower Displacement, Velocity, and Acceleration','color', 'k')
title('\it{Follower Displacement Diagram}','color', 'r','fontsize', 12)

xmin=0;
xmax=360;
ymina=min([min(fnorm), min(fpnorm), min(fppnorm)]);
ymaxa=max([max(fnorm), max(fpnorm), max(fppnorm)]);
yrange=ymaxa-ymina;
ymin=ymina-0.1*yrange;
ymax=ymaxa+0.1*yrange;
axis([xmin xmax ymin ymax]);

% Set up vectors to be drawn in the follower displacement plot

position=line('xdata', [], 'ydata' ,[], 'linewidth',3,...
      'erasemode', 'none','color', 'r');
velocity=line('xdata', [], 'ydata' ,[], 'linewidth',2,...
      'erasemode', 'none','color', 'b');
acceleration=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'none','color', 'k');

% draw the position, velocity, and acceleration curves

set(position,'xdata',thetal, 'ydata',fnorm);
set(velocity,'xdata',thetal,'ydata', fpnorm);
set(acceleration,'xdata',thetal, 'ydata',fppnorm);
text(10, 1.0,['Max. Pos. is: ', num2str(fn)],'color', 'r');
text(10, 0.9,['Max. Vel. is: ', num2str(fpn) ' *\omega'],'color', 'b');
text(10, 0.8,['Max. Acc. is: ', num2str(fppn) ' *\omega^2'],'color', 'k');
pause

% Compute the XG.

B=[XD(1)-XD(n);YD(1)-YD(n)];
A=[-cos(beta(1)) cos(beta(n));-sin(beta(1)) sin(beta(n))];
t=A\B;
s(1)=t(1);
XG(1)=XD(1)+s(1)*cos(beta(1));
YG(1)=YD(1)+s(1)*sin(beta(1));
gamma(1)=(beta(1)+beta(n))/2+3*pi/2;
for i=2:1:n
	B=[XD(i)-XD(i-1);YD(i)-YD(i-1)];
	A=[-cos(beta(i)) cos(beta(i-1));-sin(beta(i)) sin(beta(i-1))];
	t=A\B;
	s(i)=t(1);
	XG(i)=XD(i)+s(i)*cos(beta(i));
	YG(i)=YD(i)+s(i)*sin(beta(i));
	gamma(i)=(beta(i)+beta(i-1))/2+3*pi/2;
	if gamma(i)>2*pi; gamma(i)=gamma(i)-2*pi; end
end

% Compute the XM.

B=[XG(n)-XG(1); YG(n)-YG(1)];
A=[-cos(gamma(n)) cos(gamma(1));-sin(gamma(n)) sin(gamma(1))];
t=A\B;
v(n)=t(1);
XM(n)=XG(n)+v(n)*cos(gamma(n));
YM(n)=YG(n)+v(n)*sin(gamma(n));
for i=1:1:n-1
	B=[XD(i)-XD(i+1); YD(i)-YD(i+1)];
	A=[-cos(gamma(i)) cos(gamma(i+1));-sin(gamma(i)) sin(gamma(i+1))];
	t=A\B;
	v(i)=t(1);
	XM(i)=XG(i)+v(i)*cos(gamma(i));
	YM(i)=YG(i)+v(i)*sin(gamma(i));
end

% Compute the cam coordinates.

for i=1:1:n
	B=[XM(i)-XG(i); YM(i)-YG(i)];
	kappa(i)=beta(i)+pi/2;
	A=[-cos(kappa(i)) cos(beta(i));-sin(kappa(i)) sin(beta(i))];
	t=A\B;
	z(i)=t(2);
	xc(i)=XG(i)+z(i)*cos(beta(i));
	yc(i)=YG(i)+z(i)*sin(beta(i));
	rcc(i)=sqrt(xc(i)^2+yc(i)^2);
	angc(i)=atan2(yc(i), xc(i));
	if angc(i)<0; angc(i)=angc(i)+2*pi; end
end
xc(n+1)=xc(1);
yc(n+1)=yc(1);

% Compute the pressure angle

for i=1:1:n
	psi=atan2((yc(i)-YC(i)),(xc(i)-XC(i)))-pi/2;
	lamda=atan2((yc(i)-YM(i)),(xc(i)-XM(i)));
	pa(i)=acos((cos(psi)*cos(lamda)+sin(psi)*sin(lamda)))/fact;
end

% Compute the radius of curvature

for j=1:1:n
	jp1=j+1;
	jm1=j-1;
	if j==1
		a1=[xc(n), yc(n)];
		b1=[xc(n+1), yc(n+1)];
		a2=b1;
		b2=[xc(jp1), yc(jp1)];
		sign=(xc(n+1)-xc(n))*(yc(jp1)-yc(n+1))-(xc(jp1)-xc(n+1))*(yc(n+1)-yc(n));		
	end
	if j>1
		a1=[xc(jm1), yc(jm1)];
		b1=[xc(j), yc(j)];
		a2=b1;
		b2=[xc(jp1), yc(jp1)];
		sign=(xc(j)-xc(jm1))*(yc(jp1)-yc(j))-(xc(jp1)-xc(j))*(yc(j)-yc(jm1));
	end
	coords=pole(a1,a2,b1,b2);
	xcent(j)=coords(1);
	ycent(j)=coords(2);
	rho(j)=10^10;

% If flag =0, compute the true value for rho

	if coords(3)==0
		rho(j)=sqrt((xc(j)-xcent(j))^2+(yc(j)-ycent(j))^2);
	end	

% Compute the sign for rho

	if sign ~=0;
		sign=sign/abs(sign);
	end
	if sign==0; sign=1; end
	rho(j)=rho(j)*sign;
end

% If rho is large, truncate it for drawing purposes

rhom=2*sqrt((max(xc))^2+(max(yc))^2);
for j=1:1:n
	rhod(j)=rho(j);
	if abs(rho(j))>rhom
		rhod(j)=rhom*rho(j)/abs(rho(j));
	end
end

% Set up coordinates of bushings and pins.

rpivot=rb/10;
ninc=20;  
npoints=ninc+1;
centerx=0;
centery=0;
coord1=circle2(rpivot,centerx,centery,ninc);

% find coordinates of cam pin

for i=1:1:npoints
	xcoordcp(i)=coord1(i,1);
	ycoordcp(i)=coord1(i,2);
	xcoordfp(i)=xcoordcp(i)+r1;
	ycoordfp(i)=ycoordcp(i);	
end

% Find coordinates of cam and follower bushings

binc=ninc/2;
coord2=bushing(rpivot,centerx,centery,binc,0);
bpoints=ninc/2+16;
for i=1:1:bpoints
	xbush1(i)=coord2(i,1);
	ybush1(i)=coord2(i,2);
	xbush2(i)=xbush1(i)+r1;
	ybush2(i)=ybush1(i);
end

% Find coordinates of follower stem

xstem(1)=r1;
ystem(1)=0;
xstem(2)=xstem(1)-d*cos(pi/2-phi0);
ystem(2)=ystem(1)-d*sin(pi/2-phi0);
xstem(3)=xstem(2)+r3*cos(pi-phi0);
ystem(3)=ystem(2)+r3*sin(pi-phi0);

% Set up the axes for first drawing.

clf;
width=0.95;
height=0.95;
h1=axes('position', [.05 .05 width height],'box', 'on', 'xcolor', 'k','ycolor',...
       'k');
set(gcf,'color', 'w')

cx=[ xstem,  xbush2, xc];
cy=[ ystem,  ybush2, yc];

xmina=min(cx);
xmaxa=max(cx);
ymina=min(cy);
ymaxa=max(cy);
rangex=xmaxa-xmina;
rangey=ymaxa-ymina;
xmin=xmina-0.05*rangex;
xmax=xmaxa+0.05*rangex;
ymin=ymina-0.05*rangey;
ymax=ymaxa+0.05*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);

% Set up vectors to be drawn in the first plot

cam=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'none','color', 'k');
stem=line('xdata', [], 'ydata' ,[], 'linewidth',3,...
      'erasemode', 'none','color', 'k');
		
bush1=line('xdata',[],'ydata',[],'erasemode','none','color','r');
pin1=line('xdata',[],'ydata',[],'erasemode','none','color','r');
bush2=line('xdata',[],'ydata',[],'erasemode','none','color','r');
pin2=line('xdata',[],'ydata',[],'erasemode','none','color','r');

		
% draw the pins and bushings.

set(pin1,'xdata', xcoordcp, 'ydata',ycoordcp);
set(bush1,'xdata', xbush1,'ydata', ybush1);
set(pin2,'xdata', xcoordfp, 'ydata',ycoordfp);
set(bush2,'xdata', xbush2,'ydata', ybush2);

% Draw the cam.

set (cam,'xdata', xc, 'ydata', yc);

% Draw follower stem

set (stem, 'xdata', xstem, 'ydata',ystem);

% Print a file of the cam coordinates and pressure angle

disp('   ')
fprintf('cam.dat','                Cam coordinates') 
fprintf('cam.dat','       cam angle      xc         yc') 
for i=1:1:n
	fprintf('cam.dat','%12.0f %12.3f %10.3f \n',theta(i),xc(i),yc(i))
end
disp(' *** Computing the cam profile ***')

% **********************************************************

% The cam system is designed, now animate the results.

% **********************************************************

pause;
clf;

% Define the axes for the follower displacement

width=0.38;
height=0.35;
h2=axes('position',[0.6 .55 width height],'box','on','xcolor','k','ycolor',...
      'k');   %axis location
set(gcf,'color', 'w')
xlabel('Cam Angle','color', 'k')
ylabel('Follower Displacement, deg','color', 'k')
Displ=line('xdata', [], 'ydata', [], 'erasemode', 'none',...
      'color', 'r', 'linestyle', '-');
dbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, 'erase',...
      'xor');

% Define the axes for the pressure angle

h3=axes('position', [0.1 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k');
set(gcf,'color', 'w')
xlabel('Cam Angle','color', 'k')
ylabel('Pressure Angle, deg','color', 'k')
Pangle=line('xdata', [], 'ydata', [], 'erasemode', 'none', ...
      'color', 'r', 'linestyle', '-');
pabead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, ...
      'erasemode','xor');

% Define the axes for the radius of curvature

h4=axes('position', [0.6 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k');
set(gcf,'color', 'w')
xlabel('Cam Angle','color', 'k')
ylabel('Radius of Curvature ','color', 'k')
radius=line('xdata', [], 'ydata', [], 'erasemode', 'none', ...
      'color', 'r', 'linestyle', '-');
rabead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, ...
      'erasemode','xor');
  
% Set up the vectors needed to draw the linkage

h1=axes('position', [0.1 .55 width height],'box', 'on','xcolor', 'k',...
      'ycolor', 'k');
set(gcf,'color', 'w')

cam=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'xor','color', 'k');
stem=line('xdata', [], 'ydata' ,[], 'linewidth',3,...
      'erasemode', 'xor','color', 'k');

bush1=line('xdata',[],'ydata',[], 'erasemode','none','color', 'r');
pin1=line('xdata',[],'ydata',[], 'erasemode','none','color', 'r');
bush2=line('xdata',[],'ydata',[], 'erasemode','none','color', 'r');
pin2=line('xdata',[],'ydata',[], 'erasemode','none','color', 'r');
 
% Loop to analyze the cam machanism for a full cycle

nposn=30;
rotinc=floor(n/nposn);  %rotation increment to get nrot positions per rotation
k=0;
jj=0;
for j=1:rotinc:n
	jj=jj+1;
	Q21=-theta(j);
	Q2 = Q21*fact;
	ct=cos(Q2);
	st=sin(Q2);

% Rotate the follower by f(j)

	muu=pi-phi0-f(j);
	xstem(1)=r1;
	ystem(1)=0.0;
	xstem(2)=-d*cos(muu-pi/2)+xstem(1);
	ystem(2)=-d*sin(muu-pi/2)+ystem(1);
	xstem(3)=xstem(2)+r3*cos(muu);
	ystem(3)=ystem(2)+r3*sin(muu);

% find coordinates of follower

	for i=1:1:3
		k=k+1;
		xstema(jj,i)=xstem(i);
		ystema(jj,i)=ystem(i);
		tempx(k)=xstema(jj,i);
		tempy(k)=ystema(jj,i);
	end

% rotate the cam coordinates by theta(j)

	ii=0;
	for i=1:rotinc:n
		ii=ii+1;
		k=k+1;	
		xca(jj,ii)=xc(i)*ct-yc(i)*st;
		yca(jj,ii)=xc(i)*st+yc(i)*ct;	
		tempx(k)=xca(jj,ii);
		tempy(k)=yca(jj,ii);
	end
	nn=ii;
	xca(jj,ii+1)=xc(1)*ct-yc(1)*st;
	yca(jj,ii+1)=xc(1)*st+yc(1)*ct;	
end

% Set the axis limits for the graphs.  Start with the cam mechanism
% position graph
		 
axes(h1);
xmina=min(tempx)-0.3*abs(min(tempx));
xmaxa=max(tempx)+0.1*abs(max(tempx));
ymina=min(tempy)-0.2*abs(min(tempy));
ymaxa=max(tempy)+0.2*abs(max(tempy));
rangex=xmaxa-xmina;
rangey=ymaxa-ymina;
xmin=xmina-0.05*rangex;
xmax=xmaxa+0.05*rangex;
ymin=ymina-0.05*rangey;
ymax=ymaxa+0.05*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);

axes(h3);
ymin=min(pa)-0.1*abs(max(pa)-min(pa));
ymax=max(pa)+0.1*abs(max(pa)-min(pa));
axis([0 360 ymin ymax]);  %set axis limits for the pressure angle

axes(h2);
ymin=min(fd)-0.1*abs(max(fd)-min(fd));
ymax=max(fd)+0.1*abs(max(fd)-min(fd));
axis([0 360 ymin ymax]);  %set axis limits for the displacement plot

axes(h4);
ymin=min(rhod)-0.1*abs(max(rhod)-min(rhod));
ymax=max(rhod)+0.1*abs(max(rhod)-min(rhod));
axis([0 360 ymin ymax]);  %set axis limits for the radius of curvature plot

% Plot the results.

itotal=n;
ans='y';
	
% Draw the fixed curves first.

set(pin1,'xdata', xcoordcp, 'ydata',ycoordcp);
set(bush1,'xdata', xbush1,'ydata', ybush1);
set(pin2,'xdata', xcoordfp, 'ydata',ycoordfp);
set(bush2,'xdata', xbush2,'ydata', ybush2);
set(Displ,'xdata', camangle,'ydata', fd);
set(Pangle,'xdata', camangle,'ydata', pa);
set(radius,'xdata', camangle,'ydata', rho);

% Animate the results as long as ans = 'y'

timesink=1;
while ans=='y'
	for kk=1:1:ncycle
		ii=0;
		for i=1:rotinc:n
			ii=ii+1;
			for j=1:1:nn+1
				xcam(j)=xca(ii,j);
				ycam(j)=yca(ii,j);
			end
			for j=1:1:3
				xstemn(j)=xstema(ii,j);
				ystemn(j)=ystema(ii,j);
			end

			set(cam,'xdata',xcam,'ydata', ycam);
			set(stem,'xdata', xstemn,'ydata', ystemn);

			set(pabead, 'xdata', camangle(i), 'ydata', pa(i));

			set(dbead, 'xdata', camangle(i), 'ydata', fd(i));
			
			set(rabead, 'xdata', camangle(i), 'ydata', rho(i));

			drawnow;   %flush the draw buffer

% Do meaningless calculation to slow down the graphics

			for k=1:1:timesink
				zz=log10(k);
			end
		end
	end
	
% Plot figures in final position.  Plot position first.

	i = nposn/2;
		
	for j=1:1:nn+1
		xcam(j)=xca(i,j);
		ycam(j)=yca(i,j);
	end
	for j=1:1:3
		xstemn(j)=xstema(i,j);
		ystemn(j)=ystema(i,j);
	end
		
	set(cam,'xdata',xcam,'ydata', ycam);
	set(stem,'xdata', xstemn,'ydata', ystemn);
	set(pabead, 'xdata', camangle(i), 'ydata', pa(i));
	set(dbead, 'xdata', camangle(i), 'ydata', fd(i));
	set(rabead, 'xdata', camangle(i), 'ydata', rho(i));

%flush the draw buffer

	drawnow; 

% Ask if the animation should be repeated

	ans=input('Repeat animation? y/n [y]: ', 's');
	if isempty(ans);
		ans='y';
	end

% Ask if animation should be slowed down or speeded up
	
	if ans=='y'	
		anst=input('Change for animation speed? +/-/0 [-]: ', 's');
		if isempty(anst);
			anst='-';
		end
		if anst=='-'; timesink=timesink*4; end
		if anst=='+'; timesink=timesink/4; end
		if timesink<1; timesink=1; end	
	end
end
