function [values] = prrps(r1,rd1,rdd1,r3,theta2,sigma,beta,flag)

% This function analyzes an prrp mechanism when the slider at link 2
% is the driving link.  The input values are:

%r1      = length of vector 1
%rd1     = derivative of length of vector 1
%rdd1    = second derivative of length of vector 1
%r3      = length of coupler
%theta2  = angle from frame x axis to vector r2 (degrees).
%sigma   = +1 or -1.  Identifies offset configuration
%beta	   = angle between slider line 2 and slider line 1 (degrees)
%flag    = analysis flag.  If flag = 1, only a position analysis is conducted. 
%          If flag = 2, both a position and velocity analysis are conducted. 
%		     If flag = 3, position, velocity, and acceleration analyses
%		             are conducted.

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

%values (1:4)   = vector lengths
%values (5:8)   = vector angles (degrees)
%values (9:12)  = derivatives of vector lengths
%values (13:16) = derivatives of vector angles (rad/sec)
%values (17:20) = second derivatives of vector lengths
%values (21:24) = second derivatives of vector angles (rad/sec^2)
%values (25:26) = x,y components of position of point P
%values (27:28) = x,y components of position of point Q
%values (29:30) = x,y components of velocity of point P
%values (31:32) = x,y components of velocity of point Q
%values (33:34) = x,y components of acceleration of point P
%values (35:36) = x,y components of acceleration of point Q
%values (37)    = assembly flag.  If values(37) = 0, mechanism cannot 
%                 assembled.

%convert input data

values=zeros(37,1);
r(1)=r1;
r(3)=r3;
theta1=theta2+beta;
theta(2)=theta2;
theta(1)=theta1;
fact=pi/180;
t1=theta(1)*fact;
t2=theta(2)*fact;
td=zeros(4,1);
tdd=zeros(4,1);
rd=zeros(4,1);
rdd=zeros(4,1);
rd(1)=rd1;
rdd(1)=rdd1;
s1=sin(t1);
c1=cos(t1);
s2=sin(t2);
c2=cos(t2);

% position calculations

B=-r(1)*(c1*c2+s1*s2);
C=r(1)^2-r(3)^2;
arg=B^2-C;
if (arg>=0)
	values(37)=1; %assembly flag
	r(2)=-B+sigma*sqrt(arg);
	t3=atan2((r(1)*s1-r(2)*s2),(r(1)*c1-r(2)*c2));
	theta(3)=t3/fact;
	s3=sin(t3);
	c3=cos(t3);

	% coordinates of P and Q
	
	values(25)=r(2)*c2;
	values(26)=r(2)*s2;
	values(27)=r(1)*c1;
	values(28)=r(1)*s1;
	
%velocity calculation
	
	if flag>1
		AM=[c2 -r(3)*s3; s2 r(3)*c3];
		BM=[rd(1)*c1; rd(1)*s1];
		CM=AM\BM;
		rd(2)=CM(1);
		td(3)=CM(2);
	
		% velocities of P and Q
	
		values(29)=rd(2)*c2;
		values(30)=rd(2)*s2;
		values(31)=rd(1)*c1;
		values(32)=rd(1)*s1;
	end
	
%acceleration calculation
	
	if flag>2
		BM=[rdd(1)*c1+r(3)*td(3)*td(3)*c3; rdd(1)*s1+r(3)*td(3)*td(3)*s3];
		CM=AM\BM;
		rdd(2)=CM(1);
		tdd(3)=CM(2);
		
		% accelerations of P and Q
	
		values(33)=rdd(2)*c2;
		values(34)=rdd(2)*s2;
		values(35)=rdd(1)*c1;
		values(36)=rdd(1)*s1;
	end
end

%store results in array values

for i = 1:3
	values(i)=r(i);
	values(i+4)=theta(i);
	values(i+8)=rd(i);
	values(i+12)=td(i);
	values(i+16)=rdd(i);
	values(i+20)=tdd(i);
end
