%MATLAB program for design of four-bar linkage for rigid-body guidance.
%Some of the important variables used are:

%ax1     = x coordinate of coupler coordinate system in position 1
%ay1     = y coordinate of coupler coordinate system in position 1
%ax2     = x coordinate of coupler coordinate system in position 2
%ay2     = y coordinate of coupler coordinate system in position 2
%ax3     = x coordinate of coupler coordinate system in position 3
%ay3     = y coordinate of coupler coordinate system in position 3
%theta1  = angle from frame x axis to coupler x axis in position 1 (degrees)
%theta2  = angle from frame x axis to coupler x axis in position 2 (degrees)
%theta3  = angle from frame x axis to coupler x axis in position 3 (degrees)
%acx(i)  = x coordinate of circle point relative to fixed system for position i
%acy(i)  = y coordinate of circle point relative to fixed system for position i
%XC1     = x coordinate of first circle point relative to coupler coordinate system
%YC1     = y coordinate of first circle point relative to coupler coordinate system
%XC2     = x coordinate of second circle point relative to coupler coordinate system
%YC2     = y coordinate of second circle point relative to coupler coordinate system
%Astar   = coordinates of center point corresponding to [XC1, YC1]
%Bstar   = coordinates of center point corresponding to [XC2, YC2]
%p12     = coordinates of pole between positions 1 and 2
%p13     = coordinates of pole between positions 1 and 3
%p23     = coordinates of pole between positions 2 and 3
%p23prime= coordinates of image pole corresponding to p23
%pflagij = pole ij flag.  If pflagij = 1, the pole ij is at infinity; if 
%          pflagij = 0, pole ij is finite.

% This program uses the following m.files.  These must be located in the same directory
% or the path to the routines must be clearly defined:

%	centerpoint.m - Routine for finding center point given circle point
%	circlepoint.m - Routine for finding circle point given center point
%	assemblymode.m - Routine to find assembly mode
%	fourbar.m - Routine to analyze four-bar linkage
%	circle.m - Routine to determine the coordinates of a circle
%	ipole.m - Routine to find image pole given displacement poles
%	pole.m - Routine to find displacement poles
%	bushing.m - Routine to find coordinates of a bushing

clf;
clear all;
drawfact=20;
ans='y';
disp('   ')
disp('         Four Bar Linkage Design for Rigid-Body Guidance')
height=0.8;
width=0.8;
h=axes('position',[.05 .05 width height],'box','on','xcolor','k','ycolor','k');
set(gcf,'color', 'w')
ansp='y';
readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (rgb_4bario.dat): ', 's');
if isempty(inputfile); inputfile = 'rgb_4bario.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
if readmode==2

	ax1=input('Enter value of ax1 [0]: ');
	if isempty(ax1)
		ax1 = 0; 
	end

	ay1=input('Enter value of ay1 [0]: ');
	if isempty(ay1)
		ay1 = 0;
	end

	theta1=input('Enter value of theta1 (deg) [45]: ');
	if isempty(theta1)
		theta1 = 45;
	end

	ax2=input('Enter value of ax2 [3]: ');
	if isempty(ax2)
		ax2 = 3;
	end

	ay2=input('Enter value of ay2 [0]: ');
	if isempty(ay2)
		ay2 = 0;
	end

	theta2=input('Enter value of theta2 (deg) [135]: ');
	if isempty(theta2)
		theta2 = 135;
	end

	ax3=input('Enter value of ax3 [4]: ');
	if isempty(ax3)
		ax3 = 4;
	end

	ay3=input('Enter value of ax3 [2]: ');
	if isempty(ay3)
		ay3 = 2;
	end

	theta3=input('Enter value of theta3 (deg) [0]: ');
	if isempty(theta3)
		theta3 = 0;
	end
	
% save the data to the file rgb_4bario.dat
	
	temp1(1)=ax1;
	temp1(2)=ay1;
	temp1(3)=theta1;
	temp1(4)=ax2;
	temp1(5)=ay2;
	temp1(6)=theta2;
	temp1(7)=ax3; 
	temp1(8)=ay3; 
	temp1(9)=theta3; 
	temp1t=temp1(:);  % store the transpose of temp1 for ease of reading	
	fprintf(inputfile,'%f \n',temp1t)
end
	
% Read input data from the file rgb_4bario.dat

if readmode==1	
	rgb_4bario = fscanf(fid,'%f');
	ax1=rgb_4bario(1);
	ay1=rgb_4bario(2);
	theta1=rgb_4bario(3);
	ax2=rgb_4bario(4);
	ay2=rgb_4bario(5);
	theta2=rgb_4bario(6);
	ax3=rgb_4bario(7);
	ay3=rgb_4bario(8);
	theta3=rgb_4bario(9);
end

% Define the plots

position1=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode', 'xor',...
        'color', 'r');
position2=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode', 'xor',...
        'color', 'r');
position3=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode', 'xor',...
        'color', 'r');

a1=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
a2=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
a3=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');

pole12=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 8,...
        'erasemode', 'xor','color', 'k');
pole13=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 8,...
        'erasemode', 'xor','color', 'k');
pole23=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 8,...
        'erasemode', 'xor','color', 'k');
ipole23=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 8,...
        'erasemode', 'xor','color', 'k');
scircle=line('xdata', [], 'ydata',[], 'linewidth',1, 'erasemode', 'xor',...
        'color', 'k');

joint1=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 8,...
        'color', 'k');
joint2=line('xdata', [], 'ydata', [], 'color', 'r');
joint3=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 8,...
        'color', 'k');
joint4=line('xdata', [], 'ydata', [], 'color', 'r');
crank1=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');
crank2=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');
coupler1=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
coupler2=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');		
hinge1=line('xdata', [], 'ydata',[],'erasemode', 'none', 'color', 'r');
hinge2=line('xdata', [], 'ydata',[],'erasemode', 'none', 'color', 'r');

% Identify vectors giving the three components of a and b on the coupler.

fact=pi/180;
theta1r=theta1*fact;
theta2r=theta2*fact;
theta3r=theta3*fact;
ax=[ax1 ax2 ax3];
ay=[ay1 ay2 ay3];
bx=[ax1+cos(theta1r) ax2+cos(theta2r) ax3+cos(theta3r)];
by=[ay1+sin(theta1r) ay2+sin(theta2r) ay3+sin(theta3r)];
drawlim1=max([max(ax); max(ay); max(bx); max(by)]);
drawlim2=min([max(ax); min(ay); min(bx); min(by)]);
drawlim=drawfact*max([abs(drawlim1); abs(drawlim2)]);

% Identify poles using pole.m function.

p12=pole([ax(1);ay(1)],[ax(2);ay(2)],[bx(1);by(1)],[bx(2);by(2)]);
p13=pole([ax(1);ay(1)],[ax(3);ay(3)],[bx(1);by(1)],[bx(3);by(3)]);
p23=pole([ax(2);ay(2)],[ax(3);ay(3)],[bx(2);by(2)],[bx(3);by(3)]);
pflag12=p12(3);
pflag13=p13(3);
pflag23=p23(3);
pflagtot=pflag12+pflag13+pflag23;

% Identify the image pole p'23

p23prime=ipole(p12,p13,p23);

% Find the circle of sliders relative to the coupler if the poles are finite.

center(1)=drawfact;
center(2)=drawfact;
center(3)=1;
if pflagtot==0;
	center=pole([p12(1);p12(2)],[p13(1);p13(2)],[p13(1);p13(2)],...
           [p23prime(1);p23prime(2)]);
end
x0=center(1);
y0=center(2);
dflag=center(3);
rc=sqrt((p12(1)-x0)^2+(p12(2)-y0)^2);

% Draw the circle relative to the frame when the poles are all finite.

if pflagtot==0;
	dalpha=2*pi/30;
	for j=1:1:31
		sx(j)=ax1;
		sy(j)=ay1;
		alpha=(j-1)*dalpha;
		if dflag==0
			sx(j)=x0+rc*cos(alpha);
			sy(j)=y0+rc*sin(alpha);
		end
	end
end

% Draw straight line relative to the frame a pole lies at infinity
	
if pflagtot>0
	stline=cosline(p12,p13,p23,p23prime);
	for j=1:1:31
		sx(j)=stline(j,1);
		sy(j)=stline(j,2);
	end
end

% If a pole is far from the positions, do not draw it

p12x=p12(1);
if abs(p12x) > drawlim; p12x=ax; end
p12y=p12(2);
if abs(p12y) > drawlim; p12y=ay; end
p13x=p13(1);
if abs(p13x) > drawlim; p13x=ax; end
p13y=p13(2);
if abs(p13y) > drawlim; p13y=ay; end
p23x=p23(1);
if abs(p23x) > drawlim; p23x=ax; end
p23y=p23(2);
if abs(p23y) > drawlim; p23y=ay; end

% Find axis limits

cx=[ax, bx, p12x, p13x, p23x, sx];
cy=[ay, by, p12y, p13y, p23y, sy];
minx=min(cx);
maxx=max(cx);
miny=min(cy);
maxy=max(cy);
rangex=maxx-minx;
rangey=maxy-miny;
xmin=minx-0.2*rangex;
xmax=maxx+0.2*rangex;
ymin=miny-0.2*rangey;
ymax=maxy+0.2*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);

% Plot the positions

set(position1, 'xdata', [ax(1) bx(1)], 'ydata', [ay(1) by(1)]);
set(position2, 'xdata', [ax(2) bx(2)], 'ydata', [ay(2) by(2)]);
set(position3, 'xdata', [ax(3) bx(3)], 'ydata', [ay(3) by(3)]);
set(a1, 'xdata',ax(1), 'ydata', ay(1));
set(a2, 'xdata',ax(2), 'ydata', ay(2));
set(a3, 'xdata',ax(3), 'ydata', ay(3));
text(ax(1)+0.1,ay(1),'A1');
text(ax(2)+0.1,ay(2),'A2');
text(ax(3)+0.1,ay(3),'A3');

% Plot the poles

set(pole12, 'xdata',p12(1), 'ydata', p12(2));
set(pole13, 'xdata',p13(1), 'ydata', p13(2));
set(pole23, 'xdata',p23(1), 'ydata', p23(2));
set(ipole23, 'xdata',p23prime(1), 'ydata', p23prime(2));

text(p12(1)+0.1,p12(2),'P12');
text(p13(1)+0.1,p13(2),'P13');
text(p23(1)+0.1,p23(2),'P23');
text(p23prime(1)+0.1,p23prime(2),'P''23');

% Plot the slider circle

set(scircle, 'xdata',sx, 'ydata', sy);

% Print input values


disp('   ')
fprintf('         Input Posiiton Information') 
disp('   ')
fprintf('   Position    ax(i)     ay(i)    theta(i)') 
disp('   ')
ii=1;jj=2;kk=3;
fprintf('%7.0f %12.3f %9.3f %10.3f\n', ii, ax1,  ay1,  theta1)
fprintf('%7.0f %12.3f %9.3f %10.3f\n', jj, ax2,  ay2,  theta2)
fprintf('%7.0f %12.3f %9.3f %10.3f\n', kk, ax3,  ay3,  theta3)

scale=sqrt((max(ax)-min(ax))^2+(max(ay)-min(ay))^2);
rpivot=0.02*scale;
delta=2*rpivot;	

%=============================================================

% Part of program where pivot points are selected.

%=============================================================

disp('   ')
ans='y';
nslider1=0;
nslider2=0;
while ans=='y'

% Ask user for mode of input.	
	
	disp('   ')
	disp('Possible modes of inputting data: ')
	disp('  1 - mouse ')
	disp('  2 - keyboard ')
	typei=input('Designate mode of input [1] ');
	if isempty(typei)
		typei = 1;
	end
	
% Input first point when itimes =1 and second point when 
% itimes = 2.
	
	for itimes=1:1:2
		ans1='y';
		
% while loop to input circle, center, and slider points.

		while ans1=='y'

% Ask user if circle, center, or slider point is desired.	
	
			disp('   ')
			disp('Possible types of points to be input: ')
			disp('  1 - circle point ')
			disp('  2 - center point ')
			typep=input('Designate type of point to be input [1]: ');
			if isempty(typep)
				typep = 1;
			end
			
% Input the circle point.	
			if typep==1

% Loop for CIRCLE point input.  Start by checking if the input
% is to be through the mouse or keyboard.

				if typei == 1
					disp('   ')
					if itimes ==1
						disp('  Use mouse to locate first circle point')
					end
					if itimes ==2
						disp('  Use mouse to locate second circle point')
					end
					[fx1,fy1]=ginput(1);
				end

% Input circle point data using the keyboard	

				if typei == 2
					disp('   ')
					if itimes ==1
						fx1=input(' Input x component of first circle point ');
						if isempty(fx1); fx1 = ax(1);end
						fy1=input(' Input y component of first circle point ');
						if isempty(fy1); fy1 = ay(1);end
					end
					if itimes ==2
						fx1=input(' Input x component of second circle point ');
						if isempty(fx1); fx1 = bx(1);end
						fy1=input(' Input y component of second circle point ');
						if isempty(fy1); fy1 = by(1);end
					end
				end
%
% The point selected is in the frame coordinate system.  Transform
% the point to the coupler coordinate system. 
%
				st1=sin(theta1r);
				ct1=cos(theta1r);
				if itimes ==1
					XC1= (fx1-ax1)*ct1+(fy1-ay1)*st1;
					YC1= -(fx1-ax1)*st1+(fy1-ay1)*ct1;
	
% Compute first center point.

					Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2,...
					       ax3,ay3,theta3,XC1,YC1);
					Astar(1) = Values(1);
					Astar(2) = Values(2);
					for ii=1:1:3
						acx(ii)=Values(2*ii+1);
						acy(ii)=Values(2*ii+2);
					end
				end
				if itimes ==2
					XC2= (fx1-ax1)*ct1+(fy1-ay1)*st1;
					YC2= -(fx1-ax1)*st1+(fy1-ay1)*ct1;
%	
% Compute second center point.
%
					Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
					     theta3,XC2,YC2);
					Bstar(1) = Values(1);
					Bstar(2) = Values(2);
					for ii=1:1:3
						bcx(ii)=Values(2*ii+1);
						bcy(ii)=Values(2*ii+2);
					end
				end
			end
			if typep==2

% Loop for inputting CENTER points.  Start by checking if the input is 
% to be through the mouse or keyboard.

				if typei == 1
					disp('   ')
					if itimes ==1
						disp('  Use mouse to locate first center point')
						[Astarx,Astary]=ginput(1);
						Astar(1)=Astarx;
						Astar(2)=Astary;
						Values = circlepoint (ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
                              theta3, Astarx,Astary);
						acx(1)=Values(1);
						acy(1)=Values(2);	
					end

% Input second center point.

					if itimes ==2
						disp('  Use mouse to locate second center point')
						[Bstarx,Bstary]=ginput(1);
						Bstar(1)=Bstarx;
						Bstar(2)=Bstary;
						Values = circlepoint (ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
                              theta3, Bstarx,Bstary);
						bcx(1)=Values(1);
						bcy(1)=Values(2);
					end
				end	

% Input first center point using the keyboard	

				if typei == 2
					disp('   ')
					if itimes ==1
						Astarx=input(' Input x component of first center point ');
						if isempty (Astarx); 
							XC1=0.5;
							YC1=0;
							Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2,...
					       	       ax3,ay3,theta3,XC1,YC1);
							Astarx = Values(1);
						end
						Astar(1)=Astarx;
						Astary=input(' Input y component of first center point ');
						if isempty(Astary);
							XC1=0.5;
							YC1=0;
							Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2,...
							        ax3,ay3,theta3,XC1,YC1);
							Astary = Values(2);
						end
						Astar(2)=Astary;
						Values = circlepoint (ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
                                  theta3, Astarx,Astary);
						acx(1)=Values(1);
						acy(1)=Values(2);
					end

% Input second center point using keyboard

					if itimes ==2
						Bstarx=input(' Input x component of second center point ');
						if isempty(Bstarx); 
							XC1=1;
							YC1=0;
							Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
					                   theta3,XC1,YC1);
							Bstarx = Values(1);
						end
						Bstar(1)=Bstarx;
						Bstary=input(' Input y component of second center point ');
						if isempty(Bstary);
							XC1=1;
							YC1=0;
							Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
					                   theta3,XC1,YC1);
							Bstary = Values(2);
						end
						Bstar(2)=Bstary;
						Values = circlepoint (ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
                                      theta3, Bstarx,Bstary);
						bcx(1)=Values(1);
						bcy(1)=Values(2);
					end
				end
			end		

% Draw circle point and center point for first point.

			if itimes ==1

% Find coordinates of first bushing and pin joint.
				
				ninc=20;  
				npoints=ninc+1;
				centerptx=Astar(1);
				centerpty=Astar(2);
				coord=circle2(rpivot,centerptx,centerpty,ninc);

% find coordinates of first pin

				for i=1:1:npoints
					xcoord1(i)=coord(i,1);
					ycoord1(i)=coord(i,2);
				end

% find coordinates of first bushing

				binc=ninc/2;
				coord1=bushing(rpivot,centerptx,centerpty,binc,0);
				bpoints=ninc/2+16;
				for i=1:1:bpoints
					xbush1(i)=coord1(i,1);
					ybush1(i)=coord1(i,2);
				end

% draw the first bushing

				set(joint1,'xdata', acx(1), 'ydata',acy(1));
				set(joint2,'xdata', xcoord1, 'ydata',ycoord1);
				set(hinge1,'xdata', xbush1,'ydata', ybush1);
 				set(crank1,'xdata', [acx(1) Astar(1)], 'ydata',[acy(1) Astar(2)]);
			end

% Draw circle point and center point for second point.

			if itimes ==2

% Find coordinates of second bushing and pin joint.
				
				ninc=20;  
				npoints=ninc+1;
				centerptx=Bstar(1);
				centerpty=Bstar(2);
				coord=circle2(rpivot,centerptx,centerpty,ninc);

% find coordinates of second pin

				for i=1:1:npoints
					xcoord2(i)=coord(i,1);
					ycoord2(i)=coord(i,2);
				end

% find coordinates of second bushing

				binc=ninc/2;
				coord2=bushing(rpivot,centerptx,centerpty,binc,0);
				bpoints=ninc/2+16;
				for i=1:1:bpoints
					xbush2(i)=coord2(i,1);
					ybush2(i)=coord2(i,2);
				end

% draw the second bushing

				set(joint3,'xdata', bcx(1), 'ydata',bcy(1));
				set(joint4,'xdata', xcoord2, 'ydata',ycoord2);
				set(hinge2,'xdata', xbush2,'ydata', ybush2);
 				set(crank2,'xdata', [bcx(1) Bstar(1)], 'ydata',[bcy(1) Bstar(2)]);
				set(coupler1,'xdata',[acx(1),bcx(1)], 'ydata',[acy(1), bcy(1)]);
			end
			ans1=input('Reinter pivot point? y/n [n]: ', 's');
			if isempty(ans1);
				ans1='n';
			end	
		end
	end
%
% Check to see if the user wants to reinput the circle points
%
	ans=input('Reinter pivot points? y/n [n]: ', 's');
	if isempty(ans);
		ans='n';
	end
end

% ***************************************************************

% The linkage is defined. Now proceed with the analysis

% ***************************************************************

temp=atan2((ay(1)-acy(1)), (ax(1)-acx(1)));
phi=atan2((bcy(1)-acy(1)), (bcx(1)-acx(1)));
beta1=temp-phi;
length1=sqrt((ay(1)-acy(1))^2 + (ax(1)-acx(1))^2);
temp=atan2((by(1)-acy(1)), (bx(1)-acx(1)));
beta2=temp-phi;
length2=sqrt((by(1)-acy(1))^2 + (bx(1)-acx(1))^2);
rc2=sqrt((ay(1)-bcy(1))^2+ (ax(1)-bcx(1))^2);
rc1=sqrt((ay(1)-acy(1))^2+ (ax(1)-acx(1))^2);
r2=sqrt((acx(1)-Astar(1))^2+ (acy(1)-Astar(2))^2);
r4=sqrt((bcx(1)-Bstar(1))^2+ (bcy(1)-Bstar(2))^2);
r1=sqrt((Bstar(1)-Astar(1))^2+ (Bstar(2)-Astar(2))^2);
r3=sqrt((bcx(1)-acx(1))^2+ (bcy(1)-acy(1))^2);
%
% Print out the link Lengths
%
disp('   ')
fprintf('             Link Lengths') 
disp('   ')
fprintf('     r1       r2       r3       r4 ') 
disp('   ')
fprintf('%8.2f %8.2f %8.2f %8.2f\n',r1,r2,r3,r4)
%
% Print out the center and circle points
%
disp('   ')
fprintf('        Center Point Coordinates ') 
disp('   ')
fprintf('   Astarx    Astary    Bstarx    Bstary ') 
disp('   ')
fprintf('%8.2f %8.2f %8.2f %8.2f\n',Astar(1),Astar(2),Bstar(1),Bstar(2))
disp('   ')
fprintf(' Circle Point Coordinates in Position 1') 
disp('   ')
fprintf('     Ax       Ay       Bx       By ') 
disp('   ')
fprintf('%8.2f %8.2f %8.2f %8.2f\n',acx(1),acy(1),bcx(1),bcy(1))
%
% Determine the linkage mode by checking the first position.
%
Q1=atan2((Bstar(2)-Astar(2)), (Bstar(1)-Astar(1)));
Q2=atan2((acy(1)-Astar(2)), (acx(1)-Astar(1)));
phi=atan2((bcy(1)-Bstar(2)), (bcx(1)-Bstar(1)));
mode=assemblymode(r1,r2,r3,r4,Q1,Q2,phi);
%
% Analyze the linkage.  Start with a While loop that give the user
% the option of changing modes
%
ansm='y';
while ansm=='y'
	thetas=0;
	thetaf=1080;
	A0=Astar;
	theta1=Q1*180/pi;
	flag=1;
	i=0;
	Qd2=0;
	Qdd2=0;
	theta3=0;Bx(1)=0;By(1)=0;Cx(1)=0;Cy(1)=0;
	dt=(thetaf-thetas)/300;
	for Q21 = thetas:dt:thetaf
		points=fourbar(r1,r2,r3,r4,Q21,Qd2,Qdd2,mode,theta1,flag); 
		if points(29)==1
			i=i+1;
			Bx(i)=points(17)+Astar(1);
			By(i)=points(18)+Astar(2);
			Cx(i)=points(19)+Astar(1);
			Cy(i)=points(20)+Astar(2);
			theta3=points(7)*pi/180;

% Locate original a and b on coupler

			Dx(i) = Bx(i)+length1*cos(theta3+beta1);
			Dy(i) = By(i)+length1*sin(theta3+beta1);
			Ex(i) = Bx(i)+length2*cos(theta3+beta2);
			Ey(i) = By(i)+length2*sin(theta3+beta2);
		end
	end
	
% Animate the results

	pause;
	clf;
	width = 0.8;
	height=0.8;
	h2=axes('position', [.1 .1 width height],'box','on','xcolor','k','ycolor','k');

% Define axes

	temp= [Bx, Cx,Dx,Ey, Astar(1), Bstar(1),ax1,ax2,ax3, xbush1, xbush2];
	xmins=min(temp);
	xmaxs=max(temp);
	temp=[By, Cy, Dy,Ey,  Astar(2), Bstar(2),ay1,ay2,ay3,ybush1,ybush2];
	ymins=min(temp);
	ymaxs=max(temp);
	rangex=xmaxs-xmins;
	rangey=ymaxs-ymins;

% Define the plot limits

	xmin=xmins-0.05*rangex;
	ymin=ymins-0.05*rangey;
	xmax=xmaxs+0.05*rangex;
	ymax=ymaxs+0.05*rangey;
	values=axisadjust(xmin, xmax, ymin, ymax, width, height);
	xmin=values(1);
	xmax=values(2);
	ymin=values(3);
	ymax=values(4);
	axis equal
	axis([xmin xmax ymin ymax]);

	position1=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
	 'xor','color', 'r');
	position2=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
 	'xor','color', 'r');
	position3=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
	 'xor','color', 'r');

	a1=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
	a2=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
	a3=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');

	joint1=line('xdata', [], 'ydata', [], 'color', 'k');
	joint2=line('xdata', [], 'ydata', [], 'color', 'k');
	hinge1=line('xdata', [], 'ydata',[],'erasemode', 'none', 'color', 'r');
	hinge2=line('xdata', [], 'ydata',[],'erasemode', 'none', 'color', 'r');
	crank1=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');
	crank2=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');
	coupler1=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode',...
       'xor','color', 'k');
	coupler2=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode',...
       'xor','color', 'k');
	coupler3=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode',...
       'xor','color', 'b');
	
	set(position1, 'xdata', [ax(1) bx(1)], 'ydata', [ay(1) by(1)]);
	set(position2, 'xdata', [ax(2) bx(2)], 'ydata', [ay(2) by(2)]);
	set(position3, 'xdata', [ax(3) bx(3)], 'ydata', [ay(3) by(3)]);
	set(joint1,'xdata', xcoord1, 'ydata',ycoord1);
	set(hinge1,'xdata', xbush1,'ydata', ybush1);
	set(joint2,'xdata', xcoord2, 'ydata',ycoord2);
	set(hinge2,'xdata', xbush2,'ydata', ybush2);
	set(a1, 'xdata',ax(1), 'ydata', ay(1));
	set(a2, 'xdata',ax(2), 'ydata', ay(2));
	set(a3, 'xdata',ax(3), 'ydata', ay(3));
	text(ax(1)+0.1,ay(1),'A1');
	text(ax(2)+0.1,ay(2),'A2');
	text(ax(3)+0.1,ay(3),'A3');

	itotal=i;
	ans='y';
	timesink=10;
	while ans=='y'
		for i=1:1:itotal
			set(crank1,'xdata',[Astar(1) Bx(i)],'ydata', [Astar(2) By(i)]);
			set(coupler1,'xdata', [Cx(i), Bx(i)],'ydata', [Cy(i), By(i)]);
			set(coupler2,'xdata', [Bx(i), Dx(i)],'ydata', [By(i), Dy(i)]);
			set(coupler3,'xdata', [ Dx(i), Ex(i)],'ydata', [Dy(i), Ey(i)]);
			set(crank2,'xdata', [Bstar(1) Cx(i)],'ydata', [Bstar(2) Cy(i)]);
			drawnow;
		
% Do meaningless calculation to slow down the graphics

			for k=1:1:timesink
				zz=log10(k);
			end
		end
	
% Ask if animation should be repeated

		ans=input('Repeat animation? y/n [y]: ', 's');
		if isempty(ans);
			ans='y';
		end

% Ask if animation should be slowed down or speeded up
	
		if ans=='y'	
			anst=input('Change for animation speed? +/-/0 [-]: ', 's');
			if isempty(anst);
				anst='-';
			end
			if anst=='-'; timesink=timesink*10; end
			if anst=='+'; timesink=timesink/10; end
			if timesink<1; timesink=1; end	
		end
	end
	
% Ask if we should change assembly mode and reanalyze

	ansm=input('Change assembly mode and repeat analysis? y/n [y]: ', 's');
	if isempty(ansm);
		ansm='y';
	end
	temp=mode;
	if ansm=='y'; mode=-temp; end
end

