% MATLAB file for design of an elliptic trammel linkage for rigid-body 
% guidance.

%Some of the important variables used are:

%ax1     = x coordinate of coupler coordinate system in position 1
%ay1     = y coordinate of coupler coordinate system in position 1
%ax2     = x coordinate of coupler coordinate system in position 2
%ay2     = y coordinate of coupler coordinate system in position 2
%ax3     = x coordinate of coupler coordinate system in position 3
%ay3     = y coordinate of coupler coordinate system in position 3
%theta1  = angle from frame x axis to coupler x axis in position 1 (degrees)
%theta2  = angle from frame x axis to coupler x axis in position 2 (degrees)
%theta3  = angle from frame x axis to coupler x axis in position 3 (degrees)
%acx(i)  = x coordinate of circle point relative to fixed system for position i
%acy(i)  = y coordinate of circle point relative to fixed system for position i
%p12     = coordinates of pole between positions 1 and 2
%p13     = coordinates of pole between positions 1 and 3
%p23     = coordinates of pole between positions 2 and 3
%p23prime= coordinates of image pole corresponding to p23

% This program uses the following m.files.  These must be located in the same 
% directory or the path to the routines must be clearly defined:

%	prrpc.m - Routine to analyze slider crank linkage when the crank is driving.
%	circle.m - Routine to determine the coordinates of a circle
%	rectangle.m - Routine to determine the coordinates of a rectangle
%	frameline.m - Routine to determine the coordinates of a frame line
%	sliderpoint.m - Routine for finding slider point given a point 
%                   near the circle of sliders.
%	ipole.m - Routine to find image pole given displacement poles
%	pole.m - Routine to find displacement poles

clf;
clear all;
drawfact=20;
ans='y';
ansp='y';
disp('   ')
disp('         Elliptic Trammel Design for Rigid-Body Guidance')
height=0.8;
width=0.8;
h=axes('position',[.05 .05 width height],'box','on','xcolor','k','ycolor','k');
set(gcf,'color', 'w')
ansp='y';
readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (rbg_eltrammelio.dat): ', 's');
if isempty(inputfile); inputfile = 'rbg_eltrammelio.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
while ansp=='y'
	if readmode==2
		ax1=input('Enter value of ax1 [0]: ');
		if isempty(ax1)
			ax1 = 0; 
		end

		ay1=input('Enter value of ay1 [0]: ');
		if isempty(ay1)
			ay1 = 0;
		end

		theta1=input('Enter value of theta1 (deg) [45]: ');
		if isempty(theta1)
			theta1 = 45;
		end

		ax2=input('Enter value of ax2 [3]: ');
		if isempty(ax2)
			ax2 = 3;
		end

		ay2=input('Enter value of ay2 [0]: ');
		if isempty(ay2)
			ay2 = 0;
		end

		theta2=input('Enter value of theta2 (deg) [135]: ');
		if isempty(theta2)
			theta2 = 135;
		end

		ax3=input('Enter value of ax3 [2]: ');
		if isempty(ax3)
			ax3 = 2;
		end

		ay3=input('Enter value of ax3 [2]: ');
		if isempty(ay3)
			ay3 = 2;
		end

		theta3=input('Enter value of theta3 (deg) [0]: ');
		if isempty(theta3)
			theta3 = 0;
		end

% save the data to the file rgb_eltrammelio.dat
	
		temp1(1)=ax1;
		temp1(2)=ay1;
		temp1(3)=theta1;
		temp1(4)=ax2;
		temp1(5)=ay2;
		temp1(6)=theta2;
		temp1(7)=ax3; 
		temp1(8)=ay3; 
		temp1(9)=theta3; 
		temp1t=temp1(:);  % store the transpose of temp1 for ease of reading	
		fprintf(inputfile,'%f \n',temp1t)
	end
	
% Read input data from the file rgb_eltrammelio.dat

	if readmode==1	
		rgb_eltrammelio = fscanf(fid,'%f');
		ax1=rgb_eltrammelio(1);
		ay1=rgb_eltrammelio(2);
		theta1=rgb_eltrammelio(3);
		ax2=rgb_eltrammelio(4);
		ay2=rgb_eltrammelio(5);
		theta2=rgb_eltrammelio(6);
		ax3=rgb_eltrammelio(7);
		ay3=rgb_eltrammelio(8);
		theta3=rgb_eltrammelio(9);
	end

% Check for parallel positions.

	ansp='n';
	d1x=cos(theta1*pi/180);
	d1y=sin(theta1*pi/180);
	d2x=cos(theta2*pi/180);
	d2y=sin(theta2*pi/180);
	d3x=cos(theta3*pi/180);
	d3y=sin(theta3*pi/180);
	cross1=abs(d1x*d2y-d1y*d2x);
	cross2=abs(d1x*d3y-d1y*d3x);
	cross3=abs(d2x*d3y-d2y*d3x);
	if abs(cross1)<10^(-8)|abs(cross2)<10^(-8)|abs(cross3)<10^(-8)
		disp('   ')
		disp(' ************* ')
		ansp=input(' Two or more positions approximately parallel, reinput positions? (y/n) [n]: ', 's');
		if isempty(ansp)
			ansp = 'n';
		end
	end
end

% Define the plots

position1=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
          'xor','color', 'r');
position2=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
          'xor','color', 'r');
position3=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
          'xor','color', 'r');

sliderline1=line('xdata', [], 'ydata' ,[], 'linewidth',1,'erasemode',...
            'none','color', 'r');
sliderline2=line('xdata', [], 'ydata' ,[], 'linewidth',1,'erasemode',...
            'none','color', 'r');

a1=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
a2=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
a3=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');

pole12=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 10,...
        'erasemode', 'xor','color', 'k');
pole13=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 10,...
        'erasemode', 'xor','color', 'k');
pole23=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 10,...
        'erasemode', 'xor','color', 'k');
ipole23=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 10,...
        'erasemode', 'xor','color', 'k');
		
scircle=line('xdata', [], 'ydata',[], 'linewidth',1, 'erasemode',...
        'xor','color', 'k');

joint1=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 8,...
        'color', 'k');
joint2=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 8,...
        'color', 'k');

coupler1=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');
coupler2=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
coupler3=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'b');

block1=line('xdata', [], 'ydata',[],'erasemode', 'xor', ...
        'linewidth' ,1, 'color', 'k');
block2=line('xdata', [], 'ydata',[],'erasemode', 'xor', ...
        'linewidth' ,1, 'color', 'k');

% Identify vectors giving the three components of a and b on the coupler.

fact=pi/180;
theta1r=theta1*fact;
theta2r=theta2*fact;
theta3r=theta3*fact;
ax=[ax1 ax2 ax3];
ay=[ay1 ay2 ay3];
bx=[ax1+cos(theta1r) ax2+cos(theta2r) ax3+cos(theta3r)];
by=[ay1+sin(theta1r) ay2+sin(theta2r) ay3+sin(theta3r)];
drawlim1=max([max(ax); max(ay); max(bx); max(by)]);
drawlim2=min([max(ax); min(ay); min(bx); min(by)]);
drawlim=drawfact*max([abs(drawlim1); abs(drawlim2)]);

% Identify poles using pole.m function.

p12=pole([ax(1);ay(1)],[ax(2);ay(2)],[bx(1);by(1)],[bx(2);by(2)]);	
p13=pole([ax(1);ay(1)],[ax(3);ay(3)],[bx(1);by(1)],[bx(3);by(3)]);
p23=pole([ax(2);ay(2)],[ax(3);ay(3)],[bx(2);by(2)],[bx(3);by(3)]);
pflag12=p12(3);
pflag13=p13(3);
pflag23=p23(3);
pflagtot=pflag12+pflag13+pflag23;

% Identify the image pole p'23

p23prime=ipole(p12,p13,p23);

% Find the circle of sliders relative to the coupler if the poles are finite.

if pflagtot==0;
	center=pole([p12(1);p12(2)],[p13(1);p13(2)],[p13(1);p13(2)],...
           [p23prime(1);p23prime(2)]);
	x0=center(1);
	y0=center(2);
	dflag=center(3);
	rc=sqrt((p12(1)-x0)^2+(p12(2)-y0)^2);

% Draw the circle relative to the frame when the poles are all finite.

	dalpha=2*pi/30;
	for j=1:1:31
		sx(j)=ax1;
		sy(j)=ay1;
		alpha=(j-1)*dalpha;
		if dflag==0
			sx(j)=x0+rc*cos(alpha);
			sy(j)=y0+rc*sin(alpha);
		end
	end
end

% Draw straight line relative to the frame a pole lies at infinity
	
if pflagtot>0
	rc=10^10;
	dflag=1;
	stline=cosline(p12,p13,p23,p23prime);
	for j=1:1:31
		sx(j)=stline(j,1);
		sy(j)=stline(j,2);
	end
	cirangle=pi/2+atan2((sy(31)-sy(1)), (sx(31)-sx(1)));
	x0=sx(15)+rc*cos(cirangle);
	y0=sy(15)+rc*sin(cirangle);
end

% Draw straight line relative to the frame if pole lies at infinity
	
if pflagtot>0
	stline=cosline(p12,p13,p23,p23prime);
	for j=1:1:31
		sx(j)=stline(j,1);
		sy(j)=stline(j,2);
	end
end

% If a pole is far from the positions, do not draw it

p12x=p12(1);
if abs(p12x) > drawlim; p12x=ax; end
p12y=p12(2);
if abs(p12y) > drawlim; p12y=ay; end
p13x=p13(1);
if abs(p13x) > drawlim; p13x=ax; end
p13y=p13(2);
if abs(p13y) > drawlim; p13y=ay; end
p23x=p23(1);
if abs(p23x) > drawlim; p23x=ax; end
p23y=p23(2);
if abs(p23y) > drawlim; p23y=ay; end

% Find axis limits

cx=[ax, bx, p12x, p13x, p23x, sx];
cy=[ay, by, p12y, p13y, p23y, sy];
rangex=max(cx)-min(cx);
rangey=max(cy)-min(cy);
xmin=min(cx)-0.1*rangex;
xmax=max(cx)+0.1*rangex;
ymin=min(cy)-0.1*rangey;
ymax=max(cy)+0.1*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);

% Plot the positions

set(position1, 'xdata', [ax(1) bx(1)], 'ydata', [ay(1) by(1)]);
set(position2, 'xdata', [ax(2) bx(2)], 'ydata', [ay(2) by(2)]);
set(position3, 'xdata', [ax(3) bx(3)], 'ydata', [ay(3) by(3)]);
set(a1, 'xdata',ax(1), 'ydata', ay(1));
set(a2, 'xdata',ax(2), 'ydata', ay(2));
set(a3, 'xdata',ax(3), 'ydata', ay(3));
text(ax(1)+0.1,ay(1),'A1');
text(ax(2)+0.1,ay(2),'A2');
text(ax(3)+0.1,ay(3),'A3');

% Plot the poles

set(pole12, 'xdata',p12(1), 'ydata', p12(2));
set(pole13, 'xdata',p13(1), 'ydata', p13(2));
set(pole23, 'xdata',p23(1), 'ydata', p23(2));
set(ipole23, 'xdata',p23prime(1), 'ydata', p23prime(2));

text(p12(1)+0.1,p12(2),'P12');
text(p13(1)+0.1,p13(2),'P13');
text(p23(1)+0.1,p23(2),'P23');
text(p23prime(1)+0.1,p23prime(2),'P''23');

% Plot the slider circle

set(scircle, 'xdata',sx, 'ydata', sy);

% Print input values

disp('   ')
fprintf('         Input Position Information') 
disp('   ')
fprintf('   Position    ax(i)     ay(i)    theta(i)') 
disp('   ')
ii=1;jj=2;kk=3;
fprintf('%7.0f %12.3f %9.3f %10.3f\n', ii, ax1,  ay1,  theta1)
fprintf('%7.0f %12.3f %9.3f %10.3f\n', jj, ax2,  ay2,  theta2)
fprintf('%7.0f %12.3f %9.3f %10.3f\n', kk, ax3,  ay3,  theta3)

scale=sqrt((max(ax)-min(ax))^2+(max(ay)-min(ay))^2);
rpivot=0.02*scale;
delta=2*rpivot;	

%=============================================================

% Part of program where joints are selected.

%=============================================================

disp('   ')
ans='y';
while ans=='y'

% Ask user for mode of input.	
	
	disp('   ')
	disp('Possible modes of inputting data: ')
	disp('  1 - mouse ')
	disp('  2 - keyboard ')
	typei=input('Designate mode of input [1] ');
	if isempty(typei)
		typei = 1;
	end
	
% **********************************************************************

% Slider point input part. 

% **********************************************************************

	for islider=1:1:2
		ans2='y';
		while ans2=='y'

% While loop for SLIDER point input.  Start by checking if the input
% is to be through the mouse or keyboard.

			if typei == 1
				disp('   ')
				if islider==1
					disp('  Use mouse to locate first slider point')
				end
				if islider==2
					disp('  Use mouse to locate second slider point')
				end
				[fx1,fy1]=ginput(1);
			end

% Input slider point data using the keyboard	

			if typei == 2
				if islider==1
					fx1=input(' Input x component of first slider point ');
					if isempty(fx1)
						fx1 = 1.5; 
					end	
				end
				if islider==2
					fx1=input(' Input x component of second slider point ');
					if isempty(fx1) 
						fx1 = 0.8; 
					end	
				end
				if islider==1
					fy1=input(' Input y component of first slider point ');
					if isempty(fy1)
						fy1 = 2.4; 
					end	
				end
				if islider==2
					fy1=input(' Input y component of second slider point ');
					if isempty(fy1)
						fy1 = 2.2; 
					end	
				end
			end

% Rectify the point to the slider point circle.  First find the line from 
% the point chosen to the center of the circle.  Then find the point on the
% circle that is on the line.  The center of the circle is at x0, y0, and 
% the radius of the circle is rc.  

			angle=atan2(fy1-y0, fx1-x0);
			xn=x0+rc*cos(angle);
			yn=y0+rc*sin(angle);

% The point selected is in the frame coordinate system.  Transform
% the point to the coupler coordinate system. 

			st1=sin(theta1r);
			ct1=cos(theta1r);
				
% Compute the three positions of the slider point and slider line coordinates

			Xn1= (xn-ax1)*ct1+(yn-ay1)*st1;
			Yn1= -(xn-ax1)*st1+(yn-ay1)*ct1;
					
% Determine coordinates of the slider point for three positions
					
			Values=sliderpoint(ax1,ay1,theta1,ax2,ay2,theta2, ax3,ay3,...
				 theta3,Xn1,Yn1);
			for ii=1:1:3
				asx(ii)=Values(2*ii-1);
				asy(ii)=Values(2*ii);
			end
		
% Determine the coordinates of the slider line
		
			lineangle=Values(7)*fact;
			length=Values(8);
			if length < 3*delta; length=3*delta; end
			ndash=20;
			csang=cos(lineangle);
			ssang=sin(lineangle);
			xe = asx(1)-0.5*length*csang;
			ye = asy(1)-0.5*length*ssang;
			flag=1;
			lineangled=Values(7);
			coord=frameline(2*length,xe,ye,ndash,lineangled,flag);
			npoints=3*ndash;

% Define values for first slider

			if islider==1
				length1=length;
				npoints1=npoints;
				lineangle1d=lineangled;
				lineangle1=lineangle;
				csang1=csang;
				ssang1=ssang;
					
% Offset slider line by half the height of the slider block.
					
				for i=1:1:npoints1
					xsl1(i)=coord(i,1)+delta*ssang;
					ysl1(i)=coord(i,2)-delta*csang;
				end					
				set(sliderline1, 'xdata', xsl1, 'ydata', ysl1);

% draw the first slider block
					
				xpin1=asx(1);
				ypin1=asy(1);
				coords = rect(4*delta,2*delta,xpin1,ypin1,lineangle1d,0);
				for j=1:1:5
					xblock1(j)=coords(j,1);
					yblock1(j)=coords(j,2);
				end
				set(block1,'xdata', xblock1, 'ydata',yblock1);
				set(joint1,'xdata', xpin1, 'ydata',ypin1);	
			end

% Define values for second slider

			if islider==2
				length2=length;
				npoints2=npoints;
				lineangle2d=lineangled;
				lineangle2=lineangle;
				csang2=csang;
				ssang2=ssang;
	
% Offset slider line by half the height of the slider block.
					
				for i=1:1:npoints2
					xsl2(i)=coord(i,1)+delta*ssang;
					ysl2(i)=coord(i,2)-delta*csang;
				end					
				set(sliderline2, 'xdata', xsl2, 'ydata', ysl2);

% draw the second slider block
					
				xpin2=asx(1);
				ypin2=asy(1);
				coords = rect(4*delta,2*delta,xpin2,ypin2,lineangle2d,0);
				for j=1:1:5
					xblock2(j)=coords(j,1);
					yblock2(j)=coords(j,2);
				end
				set(block2,'xdata', xblock2, 'ydata',yblock2);
				set(joint2,'xdata', xpin2, 'ydata',ypin2);	
			end
				
% Check to see if the user wants to reinput the slider point
		
			if islider==1;
				ans2=input('reenter first slider point? y/n [n]: ', 's');
			end
			if islider==2;
				ans2=input('reenter second slider point? y/n [n]: ', 's');
			end
			if isempty(ans2);
				ans2='n';
			end	
		end
	end

% Check to see if the user wants to reinput the pivot and slider points
		
	ans=input('reenter pivot/slider points? y/n [n]: ', 's');
	if isempty(ans);
		ans='n';
	end
end

% Draw the coupler

set(coupler1,'xdata', [xpin1, xpin2], 'ydata',[ypin1, ypin2]);
	
% **********************************************************************

% Elliptic Trammel linkage designed.  Proceed with analysis

% **********************************************************************

% Define angles and distances to the original coupler line.

temp=atan2((ay(1)-ypin1), (ax(1)-xpin1));
phi=atan2((ypin2-ypin1), (xpin2-xpin1));
beta1=temp-phi;
temp=atan2((by(1)-ypin1), (bx(1)-xpin1));
beta2=temp-phi;
rc2=sqrt((by(1)-ypin1)^2+ (bx(1)-xpin1)^2);
rc1=sqrt((ay(1)-ypin1)^2+ (ax(1)-xpin1)^2);

% Define link lengths and offset distance (r4)

r3=sqrt((ypin2-ypin1)^2+(xpin2-xpin1)^2);

% Print out the link Lengths

disp('   ')
disp('   ')
fprintf('   Link Lengths and Slide Parameters') 
disp('   ')
fprintf('     r3      slide1 ang(deg)   slide2 ang(deg)') 
disp('   ')
fprintf('%8.2f %14.2f %16.2f \n',r3,lineangle1d,lineangle2d)
disp('   ')
disp('   ')
fprintf(' Slider Point Coordinates in Position 1') 
disp('   ')
fprintf('       Bx1      By1      Bx2      By2') 
disp('   ')
fprintf('%10.2f %8.2f %8.2f %8.2f\n',xpin1, ypin1, xpin2, ypin2)
disp('   ')

% determine the parameters needed to analyse the 
% mechanism.  First locate the coordinates where the two slider 
% lines intersect.  Define each line parametrically in terms of 
% s1 and s2.

A=[csang1 -csang2; ssang1 -ssang2];
B=[xpin2-xpin1; ypin2-ypin1];
s=A\B;
xint=xpin1+s(1)*csang1;
yint=ypin1+s(1)*ssang1;
mode=1;

% Determine the angle between the two lines.

beta=lineangle1d-lineangle2d;

% Analyze the linkage for r1 between r1min and r1max

thetas=0;
thetaf=360;
flag=1;
i=0;
Qd3=0;
Qdd3=0;
dt=(thetaf-thetas)/300;
for Q31 = thetas:dt:thetaf
	points=prrpc(r3,lineangle2d,Q31,Qd3,Qdd3,beta,flag); 
	i=i+1;
	Bx(i)=points(25)+xint;
	By(i)=points(26)+yint;
	Cx(i)=points(27)+xint;
	Cy(i)=points(28)+yint;

% Locate original a and b on coupler.  The angle Q31 is from
% slider 2 to slider 2.  We need the angle from slider 1 to slider 2 
% so subtract 180 degrees from Q31 to get theta3.

	theta3=(Q31-180)*pi/180;
	Dx(i) = Cx(i)+rc1*cos(theta3+beta1);
	Dy(i) = Cy(i)+rc1*sin(theta3+beta1);
	Ex(i) = Cx(i)+rc2*cos(theta3+beta2);
	Ey(i) = Cy(i)+rc2*sin(theta3+beta2);
	
% Locate the slider block coordinates

	xpin=Cx(i);
	ypin=Cy(i);
	coords = rect(4*delta,2*delta,xpin,ypin,lineangle1d,0);
	for j=1:1:5
		xblocka(i,j)=coords(j,1);
		yblocka(i,j)=coords(j,2);
	end
	xpin=Bx(i);
	ypin=By(i);
	coords = rect(4*delta,2*delta,xpin,ypin,lineangle2d,0);
	for j=1:1:5
		xblockb(i,j)=coords(j,1);
		yblockb(i,j)=coords(j,2);
	end
end

itotal=i;	

% Find the max and min values for Cx and Bx for the sliderlines.

[max2x, k2x]=max(Bx);
[min2x, j2x]=min(Bx);
[max1x, k1x]=max(Cx);
[min1x, j1x]=min(Cx);
lang1=atan2(Cy(k1x)-Cy(j1x), Cx(k1x)-Cx(j1x));
lang2=atan2(By(k2x)-By(j2x), Bx(k2x)-Bx(j2x));
cosang1=cos(lang1);
cosang2=cos(lang2);
sisang1=sin(lang1);
sisang2=sin(lang2);
langd1=lang1*180/pi;
langd2=lang2*180/pi;
	
% Determine the extended coordinates of the slider lines at C and B.			

slength1=sqrt((Cx(k1x)-Cx(j1x))^2+(Cy(k1x)-Cy(j1x))^2)+6*delta;
slength2=sqrt((Bx(k2x)-Bx(j2x))^2+(By(k2x)-By(j2x))^2)+6*delta;
ndash=30;
xe1 = Cx(j1x)-3*delta*cosang1;
ye1 = Cy(j1x)-3*delta*sisang1;
xe2 = Bx(j2x)-3*delta*cosang2;
ye2 = By(j2x)-3*delta*sisang2;
flag=1;
coord2=frameline(slength2,xe2,ye2,ndash,langd2,flag);					
coord1=frameline(slength1,xe1,ye1,ndash,langd1,flag);					
npoints1=3*ndash;
					
% Offset slider line by half the height of the slider blocks.
					
for i=1:1:npoints1
	xsl2(i)=coord2(i,1)+delta*sisang2;
	ysl2(i)=coord2(i,2)-delta*cosang2;
	xsl1(i)=coord1(i,1)+delta*sisang1;
	ysl1(i)=coord1(i,2)-delta*cosang1;
end					

% Animate the results

pause;
clf;
h2=axes('position', [.05 .05 width height],'box', 'on', 'xcolor', 'k',...
		'ycolor', 'k');

% Define axes

temp1= [Bx, Cx, Dx, Ex,ax,bx,ax1,ax2,ax3,xsl1,xsl2];
xmins=min(temp1);
xmaxs=max(temp1);
temp2=[By, Cy, Dy, Ey,ay,by,ay1,ay2,ay3,ysl1,ysl2];
ymins=min(temp2);
ymaxs=max(temp2);
rangex=xmaxs-xmins;
rangey=ymaxs-ymins;

% Define the plot limits

xmin=xmins-0.1*rangex;
ymin=ymins-0.1*rangey;
xmax=xmaxs+0.1*rangex;
ymax=ymaxs+0.1*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);

% Identify quantities to be plotted in new plot.

position1=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
           'xor','color', 'r');
position2=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
           'xor','color', 'r');
position3=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
           'xor','color', 'r');

sliderline1=line('xdata', [], 'ydata' ,[], 'linewidth',1,'erasemode',...
          'none','color', 'r');
sliderline2=line('xdata', [], 'ydata' ,[], 'linewidth',1,'erasemode',...
          'none','color', 'r');

a1=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
     10, 'erasemode', 'xor', 'color', 'r');
a2=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
     10, 'erasemode', 'xor', 'color', 'r');
a3=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
     10, 'erasemode', 'xor', 'color', 'r');

joint1=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 8,...
      'color', 'k');
joint2=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 8,...
      'color', 'k');

coupler1=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
      'xor', 'color', 'k');
coupler2=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
      'xor','color', 'k');
coupler3=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
      'xor','color', 'b');

block1=line('xdata', [], 'ydata',[],'erasemode', 'xor', ...
       'linewidth' ,1, 'color', 'k');
block2=line('xdata', [], 'ydata',[],'erasemode', 'xor', ...
       'linewidth' ,1, 'color', 'k');

% Plot fixed quantities
		 
set(position1, 'xdata', [ax(1) bx(1)], 'ydata', [ay(1) by(1)]);
set(position2, 'xdata', [ax(2) bx(2)], 'ydata', [ay(2) by(2)]);
set(position3, 'xdata', [ax(3) bx(3)], 'ydata', [ay(3) by(3)]);

set(sliderline1, 'xdata', xsl1, 'ydata', ysl1);
set(sliderline2, 'xdata', xsl2, 'ydata', ysl2);

set(a1, 'xdata',ax(1), 'ydata', ay(1));
set(a2, 'xdata',ax(2), 'ydata', ay(2));
set(a3, 'xdata',ax(3), 'ydata', ay(3));

text(ax(1)+0.1,ay(1),'A1');
text(ax(2)+0.1,ay(2),'A2');
text(ax(3)+0.1,ay(3),'A3');

ans='y';
timesink=10;
while ans=='y'

% repeat the animation 3 times.

	for kk=1:1:3
		for i=1:1:itotal
			set(coupler1,'xdata', [Cx(i), Bx(i)],'ydata', [Cy(i), By(i)]);
			set(coupler2,'xdata', [Cx(i), Dx(i)],'ydata', [Cy(i), Dy(i)]);
			set(coupler3,'xdata', [Dx(i), Ex(i)],'ydata', [Dy(i), Ey(i)]);
			for j=1:1:5
				xblock(j)=xblocka(i,j);
				yblock(j)=yblocka(i,j);
			end
			set(block1,'xdata', xblock, 'ydata',yblock);
			for j=1:1:5
				xblock(j)=xblockb(i,j);
				yblock(j)=yblockb(i,j);
			end
			set(block2,'xdata', xblock, 'ydata',yblock);
			drawnow;
		
% Do meaningless calculation to slow down the graphics

			for k=1:1:timesink
				zz=log10(k);
			end
		end
	end

% Ask if animation should be repeated

	ans=input('Repeat animation? y/n [y]: ', 's');
	if isempty(ans);
		ans='y';
	end

% Ask if animation should be slowed down or speeded up
	
	if ans=='y'	
		anst=input('Change for animation speed? +/-/0 [-]: ', 's');
		if isempty(anst);
			anst='-';
		end
		if anst=='-'; timesink=timesink*4; end
		if anst=='+'; timesink=timesink/4; end
		if timesink<1; timesink=1; end	
	end
end
