
% MATLAB program for design of a slider crank linkage for rigid-body guidance.
% The slider is the driver, and the crank is selected first.  No two positions
% can be parallel.

%Some of the important variables used are:

%ax1     = x coordinate of coupler coordinate system in position 1
%ay1     = y coordinate of coupler coordinate system in position 1
%ax2     = x coordinate of coupler coordinate system in position 2
%ay2     = y coordinate of coupler coordinate system in position 2
%ax3     = x coordinate of coupler coordinate system in position 3
%ay3     = y coordinate of coupler coordinate system in position 3
%theta1  = angle from frame x axis to coupler x axis in position 1 (degrees)
%theta2  = angle from frame x axis to coupler x axis in position 2 (degrees)
%theta3  = angle from frame x axis to coupler x axis in position 3 (degrees)
%acx(i)  = x coordinate of circle point relative to fixed system for position i
%acy(i)  = y coordinate of circle point relative to fixed system for position i
%XC1     = x coordinate of first circle point relative to coupler coordinate system
%YC1     = y coordinate of first circle point relative to coupler coordinate system
%XC2     = x coordinate of second circle point relative to coupler coordinate system
%YC2     = y coordinate of second circle point relative to coupler coordinate system
%Astar   = coordinates of center point corresponding to [XC1, YC1]
%Bstar   = coordinates of center point corresponding to [XC2, YC2]
%p12     = coordinates of pole between positions 1 and 2
%p13     = coordinates of pole between positions 1 and 3
%p23     = coordinates of pole between positions 2 and 3
%p23prime= coordinates of image pole corresponding to p23
%pflagij = pole ij flag.  If pflagij = 1, the pole ij is at infinity; if 
%          pflagij = 0, pole ij is finite.

clf;
clear all;
drawfact=20;
ans='y';
ansp='y';
disp('   ')
disp('         Slider-Crank Design for Rigid-Body Guidance')
height=0.8;
width=0.8;
h=axes('position',[.05 .05 width height],'box','on','xcolor','k','ycolor','k');
set(gcf,'color', 'w')
ansp='y';
readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (rgb_sldcrkio.dat): ', 's');
if isempty(inputfile); inputfile = 'rgb_sldcrkio.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
while ansp=='y'
	if readmode==2
		ax1=input('Enter value of ax1 [0]: ');
		if isempty(ax1)
			ax1 = 0; 
		end

		ay1=input('Enter value of ay1 [0]: ');
		if isempty(ay1)
			ay1 = 0;
		end

		theta1=input('Enter value of theta1 (deg) [45]: ');
		if isempty(theta1)
			theta1 = 45;
		end

		ax2=input('Enter value of ax2 [3]: ');
		if isempty(ax2)
			ax2 = 3;
		end

		ay2=input('Enter value of ay2 [0]: ');
		if isempty(ay2)
			ay2 = 0;
		end

		theta2=input('Enter value of theta2 (deg) [135]: ');
		if isempty(theta2)
			theta2 = 135;
		end

		ax3=input('Enter value of ax3 [2]: ');
		if isempty(ax3)
			ax3 = 2;
		end

		ay3=input('Enter value of ax3 [2]: ');
		if isempty(ay3)
			ay3 = 2;
		end

		theta3=input('Enter value of theta3 (deg) [0]: ');
		if isempty(theta3)
			theta3 = 0;
		end

% save the data to the file rgb_sldcrkio.dat
	
		temp1(1)=ax1;
		temp1(2)=ay1;
		temp1(3)=theta1;
		temp1(4)=ax2;
		temp1(5)=ay2;
		temp1(6)=theta2;
		temp1(7)=ax3; 
		temp1(8)=ay3; 
		temp1(9)=theta3; 
		temp1t=temp1(:);  % store the transpose of temp1 for ease of reading	
		fprintf(inputfile,'%f \n',temp1t)
	end
	
% Read input data from the file rgb_sldcrkio.dat

	if readmode==1	
		rgb_sldcrkioio = fscanf(fid,'%f');
		ax1=rgb_sldcrkioio(1);
		ay1=rgb_sldcrkioio(2);
		theta1=rgb_sldcrkioio(3);
		ax2=rgb_sldcrkioio(4);
		ay2=rgb_sldcrkioio(5);
		theta2=rgb_sldcrkioio(6);
		ax3=rgb_sldcrkioio(7);
		ay3=rgb_sldcrkioio(8);
		theta3=rgb_sldcrkioio(9);
	end

% Check for parallel positions.

	ansp='n';
	d1x=cos(theta1*pi/180);
	d1y=sin(theta1*pi/180);
	d2x=cos(theta2*pi/180);
	d2y=sin(theta2*pi/180);
	d3x=cos(theta3*pi/180);
	d3y=sin(theta3*pi/180);
	cross1=abs(d1x*d2y-d1y*d2x);
	cross2=abs(d1x*d3y-d1y*d3x);
	cross3=abs(d2x*d3y-d2y*d3x);
	if abs(cross1)<10^(-8)|abs(cross2)<10^(-8)|abs(cross3)<10^(-8)
		disp('   ')
		disp(' ************* ')
		ansp=input(' Two or more positions approximately parallel, reinput positions? (y/n) [n]: ', 's');
		if isempty(ansp)
			ansp = 'n';
		end
	end
end

% Define the plots

position1=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode', 'xor','color', 'r');
position2=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode', 'xor','color', 'r');
position3=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode', 'xor','color', 'r');
sliderline1=line('xdata', [], 'ydata' ,[], 'linewidth',1,'erasemode',...
            'none','color', 'r');

a1=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
a2=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
a3=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');

pole12=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 8,...
        'erasemode', 'xor','color', 'k');
pole13=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 8,...
        'erasemode', 'xor','color', 'k');
pole23=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 8,...
        'erasemode', 'xor','color', 'k');
ipole23=line('xdata', [], 'ydata',[], 'marker', '+', 'markersize', 8,...
        'erasemode', 'xor','color', 'k');
		
scircle=line('xdata', [], 'ydata',[], 'linewidth',1, 'erasemode', 'xor','color', 'k');

joint1=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 7,...
        'color', 'k');
joint2=line('xdata', [], 'ydata', [], 'color', 'r');
joint3=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 7,...
        'color', 'k');

hinge1=line('xdata', [], 'ydata',[],'erasemode', 'none', 'color', 'r');
crank1=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');

coupler1=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');
coupler2=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
coupler3=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'b');

block1=line('xdata', [], 'ydata',[],'erasemode', 'xor', ...
        'linewidth' ,1, 'color', 'k');

% Identify vectors giving the three components of a and b on the coupler.

fact=pi/180;
theta1r=theta1*fact;
theta2r=theta2*fact;
theta3r=theta3*fact;
ax=[ax1 ax2 ax3];
ay=[ay1 ay2 ay3];
bx=[ax1+cos(theta1r) ax2+cos(theta2r) ax3+cos(theta3r)];
by=[ay1+sin(theta1r) ay2+sin(theta2r) ay3+sin(theta3r)];
drawlim1=max([max(ax); max(ay); max(bx); max(by)]);
drawlim2=min([max(ax); min(ay); min(bx); min(by)]);
drawlim=drawfact*max([abs(drawlim1); abs(drawlim2)]);

% Identify poles using pole.m function.

p12=pole([ax(1);ay(1)],[ax(2);ay(2)],[bx(1);by(1)],[bx(2);by(2)]);	
p13=pole([ax(1);ay(1)],[ax(3);ay(3)],[bx(1);by(1)],[bx(3);by(3)]);
p23=pole([ax(2);ay(2)],[ax(3);ay(3)],[bx(2);by(2)],[bx(3);by(3)]);
pflag12=p12(3);
pflag13=p13(3);
pflag23=p23(3);
pflagtot=pflag12+pflag13+pflag23;

% Identify the image pole p'23

p23prime=ipole(p12,p13,p23);

% Find the circle of sliders relative to the coupler if the poles are finite.

if pflagtot==0;
	center=pole([p12(1);p12(2)],[p13(1);p13(2)],[p13(1);p13(2)],...
           [p23prime(1);p23prime(2)]);
	x0=center(1);
	y0=center(2);
	dflag=center(3);
	rc=sqrt((p12(1)-x0)^2+(p12(2)-y0)^2);

% Draw the circle relative to the frame when the poles are all finite.

	dalpha=2*pi/30;
	for j=1:1:31
		sx(j)=ax1;
		sy(j)=ay1;
		alpha=(j-1)*dalpha;
		if dflag==0
			sx(j)=x0+rc*cos(alpha);
			sy(j)=y0+rc*sin(alpha);
		end
	end
end

% Draw straight line relative to the frame a pole lies at infinity
	
if pflagtot>0
	rc=10^10;
	dflag=1;
	stline=cosline(p12,p13,p23,p23prime);
	for j=1:1:31
		sx(j)=stline(j,1);
		sy(j)=stline(j,2);
	end
	cirangle=pi/2+atan2((sy(31)-sy(1)), (sx(31)-sx(1)));
	x0=sx(15)+rc*cos(cirangle);
	y0=sy(15)+rc*sin(cirangle);
end

% Draw straight line relative to the frame if pole lies at infinity
	
if pflagtot>0
	stline=cosline(p12,p13,p23,p23prime);
	for j=1:1:31
		sx(j)=stline(j,1);
		sy(j)=stline(j,2);
	end
end

% If a pole is far from the positions, do not draw it

p12x=p12(1);
if abs(p12x) > drawlim; p12x=ax; end
p12y=p12(2);
if abs(p12y) > drawlim; p12y=ay; end
p13x=p13(1);
if abs(p13x) > drawlim; p13x=ax; end
p13y=p13(2);
if abs(p13y) > drawlim; p13y=ay; end
p23x=p23(1);
if abs(p23x) > drawlim; p23x=ax; end
p23y=p23(2);
if abs(p23y) > drawlim; p23y=ay; end

% Find axis limits

cx=[ax, bx, p12x, p13x, p23x, sx];
cy=[ay, by, p12y, p13y, p23y, sy];
rangex=max(cx)-min(cx);
rangey=max(cy)-min(cy);
xmin=min(cx)-0.2*rangex;
xmax=max(cx)+0.2*rangex;
ymin=min(cy)-0.2*rangey;
ymax=max(cy)+0.2*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);

% Plot the positions

set(position1, 'xdata', [ax(1) bx(1)], 'ydata', [ay(1) by(1)]);
set(position2, 'xdata', [ax(2) bx(2)], 'ydata', [ay(2) by(2)]);
set(position3, 'xdata', [ax(3) bx(3)], 'ydata', [ay(3) by(3)]);
set(a1, 'xdata',ax(1), 'ydata', ay(1));
set(a2, 'xdata',ax(2), 'ydata', ay(2));
set(a3, 'xdata',ax(3), 'ydata', ay(3));
text(ax(1)+0.1,ay(1),'A1');
text(ax(2)+0.1,ay(2),'A2');
text(ax(3)+0.1,ay(3),'A3');

% Plot the poles

set(pole12, 'xdata',p12(1), 'ydata', p12(2));
set(pole13, 'xdata',p13(1), 'ydata', p13(2));
set(pole23, 'xdata',p23(1), 'ydata', p23(2));
set(ipole23, 'xdata',p23prime(1), 'ydata', p23prime(2));

text(p12(1)+0.1,p12(2),'P12', 'color', 'k');
text(p13(1)+0.1,p13(2),'P13', 'color', 'k');
text(p23(1)+0.1,p23(2),'P23', 'color', 'k');
text(p23prime(1)+0.1,p23prime(2),'P''23', 'color', 'k');

% Plot the slider circle

set(scircle, 'xdata',sx, 'ydata', sy);

% Print input values

disp('   ')
fprintf('         Input Position Information') 
disp('   ')
fprintf('   Position    ax(i)     ay(i)    theta(i)') 
disp('   ')
ii=1;jj=2;kk=3;
fprintf('%7.0f %12.3f %9.3f %10.3f\n', ii, ax1,  ay1,  theta1)
fprintf('%7.0f %12.3f %9.3f %10.3f\n', jj, ax2,  ay2,  theta2)
fprintf('%7.0f %12.3f %9.3f %10.3f\n', kk, ax3,  ay3,  theta3)

scale=sqrt((max(ax)-min(ax))^2+(max(ay)-min(ay))^2);
rpivot=0.02*scale;
delta=2*rpivot;	

%=============================================================

% Part of program where joints are selected.

%=============================================================

disp('   ')
ans='y';
while ans=='y'

% Ask user for mode of input.	
	
	disp('   ')
	disp('Possible modes of inputting data: ')
	disp('  1 - mouse ')
	disp('  2 - keyboard ')
	typei=input('Designate mode of input [1] ');
	if isempty(typei)
		typei = 1;
	end

% **********************************************************************

% Crank point input part.  Crank is designed and then the slider.

% **********************************************************************
		
% while loop to input circle or center points.

	ans1='y';
	while ans1=='y'

% Ask user if circle or center point is desired.	
	
		disp('   ')
		disp('Possible types of points to be input: ')
		disp('  1 - circle point ')
		disp('  2 - center point ')
		typep=input('Designate type of point to be input [1]: ');
		if isempty(typep)
			typep = 1;
		end
			
% Loop for CIRCLE point input.  Start by checking if the input
% is to be through the mouse or keyboard.

		if typep==1
			if typei == 1
				disp('   ')
				disp('  Use mouse to locate circle point')
				[fx1,fy1]=ginput(1);
			end

% Input circle point data using the keyboard	

			if typei == 2
				disp('   ')
				fx1=input(' Input x component of circle point ');
				if isempty(fx1); fx1 = ax(1);end				
				fy1=input(' Input y component of circle point ');
				if isempty(fy1); fy1 = ay(1);end
			end
%
% The point selected is in the frame coordinate system.  Transform
% the point to the coupler coordinate system. 
%
			st1=sin(theta1r);
			ct1=cos(theta1r);
			XC1= (fx1-ax1)*ct1+(fy1-ay1)*st1;
			YC1= -(fx1-ax1)*st1+(fy1-ay1)*ct1;
	
% Compute first center point.

			Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2, ax3,ay3,theta3,XC1,YC1);
			Astar(1) = Values(1);
			Astar(2) = Values(2);
			for ii=1:1:3
				acx(ii)=Values(2*ii+1);
				acy(ii)=Values(2*ii+2);
			end
		end	
		if typep==2

% Loop for inputting CENTER points.  Start by checking if the input is 
% to be through the mouse or keyboard.

			if typei == 1
				disp('   ')
				disp('  Use mouse to locate first center point')						
				[Astarx,Astary]=ginput(1);
				Astar(1)=Astarx;
				Astar(2)=Astary;
				Values = circlepoint (ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
                     theta3, Astarx,Astary);
				acx(1)=Values(1);
				acy(1)=Values(2);	
			end	

% Input center point using the keyboard	

			if typei == 2
				disp('   ')
				Astarx=input(' Input x component of first center point ');
				if isempty (Astarx); 
					XC1=0.5;
					YC1=0;
					Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2,...
					        ax3,ay3,theta3,XC1,YC1);
					Astarx = Values(1);
				end
				Astar(1)=Astarx;
				Astary=input(' Input y component of first center point ');
				if isempty(Astary);
					XC1=0.5;
					YC1=0;
					Values=centerpoint(ax1,ay1,theta1,ax2,ay2,theta2,...
					  	 ax3,ay3,theta3,XC1,YC1);
					Astary = Values(2);
				end
				Astar(2)=Astary;
				Values = circlepoint (ax1,ay1,theta1,ax2,ay2,theta2,ax3,ay3,...
                      theta3, Astarx,Astary);
				acx(1)=Values(1);
				acy(1)=Values(2);	
			end		
		end
		
% Draw circle point and center point for first point.  First find 
% coordinates of first bushing and pin joint.					
				
		ninc=20;  
		npoints=ninc+1;
		centerptx=Astar(1);
		centerpty=Astar(2);					
		coord=circle2(rpivot,centerptx,centerpty,ninc);

% find coordinates of pin

		for i=1:1:npoints
			xcoord1(i)=coord(i,1);
			ycoord1(i)=coord(i,2);
		end

% find coordinates of bushing

		binc=ninc/2;
		coord1=bushing(rpivot,centerptx,centerpty,binc,0);
		bpoints=ninc/2+16;
		for i=1:1:bpoints
			xbush1(i)=coord1(i,1);
			ybush1(i)=coord1(i,2);
		end

% draw the bushing

		set(joint1,'xdata', acx(1), 'ydata',acy(1));
		set(joint2,'xdata', xcoord1, 'ydata',ycoord1);
		set(hinge1,'xdata', xbush1,'ydata', ybush1);
 		set(crank1,'xdata', [acx(1) Astar(1)], 'ydata',[acy(1) Astar(2)]);
%
% Check to see if the user wants to reinput the pivot point
%
		ans1=input('reenter pivot point? y/n [n]: ', 's');
		if isempty(ans1);
			ans1='n';
		end	
	end
	
% **********************************************************************

% Slider point input part

% **********************************************************************

	ans2='y';
	while ans2=='y'

% While loop for SLIDER point input.  Start by checking if the input
% is to be through the mouse or keyboard.

		if typei == 1
			disp('   ')
			disp('  Use mouse to locate slider point')
			[fx1,fy1]=ginput(1);
		end

% Input slider point data using the keyboard	

		if typei == 2
			fx1=input(' Input x component of first slider point ');
			if isempty(fx1); fx1 = 0.97;end				
			fy1=input(' Input y component of first slider point ');
			if isempty(fy1); fy1 = 2.69;end
		end

% Rectify the point to the slider point circle.  First find the line from 
% the point chosen to the center of the circle.  Then find the point on the
% circle that is on the line.  The center of the circle is at x0, y0, and 
% the radius of the circle is rc.  

		angle=atan2(fy1-y0, fx1-x0);
		xn=x0+rc*cos(angle);
		yn=y0+rc*sin(angle);

% The point selected is in the frame coordinate system.  Transform
% the point to the coupler coordinate system. 

		st1=sin(theta1r);
		ct1=cos(theta1r);
				
% Compute the three positions of the slider point and slider line coordinates

		Xn1= (xn-ax1)*ct1+(yn-ay1)*st1;
		Yn1= -(xn-ax1)*st1+(yn-ay1)*ct1;
					
% Determine coordinates of the slider point for three positions
					
		Values=sliderpoint(ax1,ay1,theta1,ax2,ay2,theta2, ax3,ay3,...
				 theta3,Xn1,Yn1);
		for ii=1:1:3
			asx(ii)=Values(2*ii-1);
			asy(ii)=Values(2*ii);
		end
		
% Determine the coordinates of the slider line				
		
		lineangle=Values(7)*fact;
		length1=Values(8);
		if length1 < 2*delta; length1=2*delta; end
		ndash=20;
		csang=cos(lineangle);
		ssang=sin(lineangle);					
		xe = asx(1)-0.5*length1*csang;
		ye = asy(1)-0.5*length1*ssang;
		flag=1;
		lineangled=Values(7);
		coord=frameline(2*length1,xe,ye,ndash,lineangled,flag);					
		npoints1=3*ndash;
					
% Offset slider line by half the height of the slider block.
					
		for i=1:1:npoints1
			xsl1(i)=coord(i,1)+delta*ssang;
			ysl1(i)=coord(i,2)-delta*csang;
		end					
		set(sliderline1, 'xdata', xsl1, 'ydata', ysl1);

% draw the first slider block
					
		xpin=asx(1);
		ypin=asy(1);
		coords = rect(4*delta,2*delta,xpin,ypin,lineangled,0);
		for j=1:1:5
			xblock(j)=coords(j,1);
			yblock(j)=coords(j,2);
		end
		set(joint2,'xdata', xcoord1, 'ydata',ycoord1);
		set(block1,'xdata', xblock, 'ydata',yblock);
		set(joint3,'xdata', xpin, 'ydata',ypin);	
		
% Draw the coupler

		set(coupler1,'xdata', [acx(1) asx(1)], 'ydata',[acy(1) asy(1)]);
				
% Check to see if the user wants to reinput the slider point

		ans2=input('reenter slider point? y/n [n]: ', 's');
		if isempty(ans2);
			ans2='n';
		end	
	end

% Check to see if the user wants to reinput the pivot and slider points

	ans=input('reenter pivot/slider points? y/n [n]: ', 's');
	if isempty(ans);
		ans='n';
	end
end
	
% **********************************************************************

% Slider-crank linkage designed.  Proceed with analysis

% **********************************************************************

% Define angles and distances to the original coupler line.

Q2=atan2((acy(1)-Astar(2)), (acx(1)-Astar(1)))/fact;
temp=atan2((ay(1)-acy(1)), (ax(1)-acx(1)));
phi=atan2((asy(1)-acy(1)), (asx(1)-acx(1)));
beta1=temp-phi;
temp=atan2((by(1)-acy(1)), (bx(1)-acx(1)));
beta2=temp-phi;
rc2=sqrt((by(1)-acy(1))^2+ (bx(1)-acx(1))^2);
rc1=sqrt((ay(1)-acy(1))^2+ (ax(1)-acx(1))^2);

% Define link lengths and offset distance (r4)

r2=sqrt((acx(1)-Astar(1))^2+ (acy(1)-Astar(2))^2);
r3=sqrt((asx(1)-acx(1))^2+ (asy(1)-acy(1))^2);
r4=(asy(1)-Astar(2))*cos(lineangle)-(asx(1)-Astar(1))*sin(lineangle);	

% Print out the link Lengths

disp('   ')
disp('   ')
fprintf('        Link Lengths and Slide Parameters') 
disp('   ')
fprintf('     r2       r3     offset   slidler ang(deg)') 
disp('   ')
fprintf('%8.2f %8.2f %8.2f %12.2f\n',r2,r3,r4, lineangled)

% Print out the center point and circle point and slider line information

disp('   ')
disp('   ')
fprintf(' Center and Circle Point Coordinates in Position 1') 
disp('   ')
fprintf('         Ax      Ay      Astarx    Astary ') 
disp('   ')
fprintf('%12.2f %8.2f %8.2f %8.2f\n',acx(1),acy(1),Astar(1),Astar(2))
disp('   ')
disp('   ')
fprintf(' Slider Point Coordinates in Position 1') 
disp('   ')
fprintf('             Bx       By') 
disp('   ')
fprintf('%16.2f %8.2f %8.2f %8.2f\n',asx(1),asy(1))
disp('   ')
disp('   ')

% Determine the linkage mode by checking the first position.

Bx=asx(1)-Astar(1);
By=asy(1)-Astar(2);
Q1=lineangled;
mode=assemblymode_sc(r2,r3,Bx,By,Q1,Q2);

% Find extreme values for r1.

r1max=0.9999*sqrt((r2+r3)^2-r4^2);
r1min=-r1max;

% Analyze the linkage for r1 between r1min and r1max

rd1=0;
rdd1=0;

% While loop which allows analysis to be conducted with both assembly modes

ansm='y';
while ansm=='y'
	dr1=(r1max-r1min)/100;
	i=0;
	flag=1;

% Start from r1min and go to r1max

	for r1=r1min:dr1:r1max
		points=sldcrks(r1,r2,r3,r4,rd1,rdd1,mode,lineangled,flag); 
		if points(37)==1
			i=i+1;
			Bx(i)=points(25)+Astar(1);
			By(i)=points(26)+Astar(2);
			Cx(i)=points(27)+Astar(1);
			Cy(i)=points(28)+Astar(2);
			theta3=points(7)*pi/180;

% Locate original a and b on coupler

			Dx(i) = Bx(i)+rc1*cos(theta3+beta1);
			Dy(i) = By(i)+rc1*sin(theta3+beta1);
			Ex(i) = Bx(i)+rc2*cos(theta3+beta2);
			Ey(i) = By(i)+rc2*sin(theta3+beta2);
	
% Locate the slider block coordinates

			xpin=Cx(i);
			ypin=Cy(i);
			coords = rect(4*delta,2*delta,xpin,ypin,lineangled,0);
			for j=1:1:5
				xblocka(i,j)=coords(j,1);
				yblocka(i,j)=coords(j,2);
			end
		end
	end

% Now return from r1max and go to r1min

	for r1=r1max:-dr1:r1min
		points=sldcrks(r1,r2,r3,r4,rd1,rdd1,mode,lineangled,flag); 
		if points(37)==1
			i=i+1;
			Bx(i)=points(25)+Astar(1);
			By(i)=points(26)+Astar(2);
			Cx(i)=points(27)+Astar(1);
			Cy(i)=points(28)+Astar(2);
			theta3=points(7)*pi/180;

% Locate original a and b on coupler

			Dx(i) = Bx(i)+rc1*cos(theta3+beta1);
			Dy(i) = By(i)+rc1*sin(theta3+beta1);
			Ex(i) = Bx(i)+rc2*cos(theta3+beta2);
			Ey(i) = By(i)+rc2*sin(theta3+beta2);
		
% Locate the slider block coordinates

			xpin=Cx(i);
			ypin=Cy(i);
			coords = rect(4*delta,2*delta,xpin,ypin,lineangled,0);
			for j=1:1:5
				xblocka(i,j)=coords(j,1);
				yblocka(i,j)=coords(j,2);
			end
		end
	end
	itotal=i;	
	
% Find the max and min values for Cx for the sliderline.

	[maxcx, kx]=max(Cx);
	[mincx, jx]=min(Cx);
	
% Determine the extended coordinates of the sliderline

	lineang=atan2(Cy(kx)-Cy(jx), Cx(kx)-Cx(jx));
	cosang=cos(lineang);
	sisang=sin(lineang);
	lineangd=lineang*180/pi;

	slength=sqrt((Cx(kx)-Cx(jx))^2+(Cy(kx)-Cy(jx))^2)+6*delta;
	ndash=30;
	xe = Cx(jx)-3*delta*cosang;
	ye = Cy(jx)-3*delta*sisang;
	flag=1;
	coord=frameline(slength,xe,ye,ndash,lineangd,flag);					
	npoints1=3*ndash;
					
% Offset slider line by half the height of the slider block.
					
	for i=1:1:npoints1
		xsl1(i)=coord(i,1)+delta*sisang;
		ysl1(i)=coord(i,2)-delta*cosang;
	end					

% Animate the results

	pause;
	clf;
	width=0.8;
	height=0.8;
	h2=axes('position', [.1 .1 width height],'box', 'on', 'xcolor', 'k',...
		'ycolor', 'k');

% Define axes

	temp1= [Bx, Cx, Astar(1), ax,bx,ax1,ax2,ax3,xsl1];
	xmins=min(temp1);
	xmaxs=max(temp1);
	temp2=[By, Cy, Astar(2), ay,by,ay1,ay2,ay3,ysl1];
	ymins=min(temp2);
	ymaxs=max(temp2);
	rangex=xmaxs-xmins;
	rangey=ymaxs-ymins;

% Define the plot limits

	xmin=xmins-0.1*rangex;
	ymin=ymins-0.1*rangey;
	xmax=xmaxs+0.1*rangex;
	ymax=ymaxs+0.1*rangey;
	values=axisadjust(xmin, xmax, ymin, ymax, width, height);
	xmin=values(1);
	xmax=values(2);
	ymin=values(3);
	ymax=values(4);
	axis equal
	axis([xmin xmax ymin ymax]);

% Identify quantities to be plotted in new plot.

	position1=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
	 'xor','color', 'r');
	position2=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
	 'xor','color', 'r');
	position3=line('xdata', [], 'ydata' ,[], 'linewidth',2,'erasemode',...
	 'xor','color', 'r');

	a1=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
	a2=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');
	a3=line('xdata', [], 'ydata', [], 'marker', '+','markersize', ...
       10, 'erasemode', 'xor', 'color', 'r');

	sliderline1=line('xdata', [], 'ydata' ,[], 'linewidth',1,'erasemode',...
            'none','color', 'r');

	joint1=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 7,...
        'erasemode','xor','color', 'k');
	joint2=line('xdata', [], 'ydata', [], 'color', 'r');
	joint3=line('xdata', [], 'ydata',[], 'marker', 'o', 'markersize', 7,...
        'erasemode','xor','color','k');

	hinge1=line('xdata', [], 'ydata',[],'erasemode', 'none', 'color', 'r');
	crank1=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');

	coupler1=line('xdata', [], 'ydata', [],'linewidth',2,'erasemode',...
        'xor', 'color', 'k');
	coupler2=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
	coupler3=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'b');

	block1=line('xdata', [], 'ydata',[],'erasemode', 'xor', ...
        'linewidth' ,1, 'color', 'k');

% Plot fixed quantities
		 
	set(position1, 'xdata', [ax(1) bx(1)], 'ydata', [ay(1) by(1)]);
	set(position2, 'xdata', [ax(2) bx(2)], 'ydata', [ay(2) by(2)]);
	set(position3, 'xdata', [ax(3) bx(3)], 'ydata', [ay(3) by(3)]);

	set(joint2,'xdata', xcoord1, 'ydata',ycoord1);
	set(hinge1,'xdata', xbush1,'ydata', ybush1);
	set(crank1,'xdata', [acx(1) Astar(1)], 'ydata',[acy(1) Astar(2)]);
	set(block1,'xdata', xblock, 'ydata',yblock);
	set(sliderline1, 'xdata', xsl1, 'ydata', ysl1);
	set(coupler1,'xdata', [acx(1) asx(1)], 'ydata',[acy(1) asy(1)]);

	set(a1, 'xdata',ax(1), 'ydata', ay(1));
	set(a2, 'xdata',ax(2), 'ydata', ay(2));
	set(a3, 'xdata',ax(3), 'ydata', ay(3));

	text(ax(1)+0.1,ay(1),'A1');
	text(ax(2)+0.1,ay(2),'A2');
	text(ax(3)+0.1,ay(3),'A3');

	ans='y';
	timesink=10;
	while ans=='y'
	
% Repeat animation 3 times

		for kk=1:1:3
			for i=1:1:itotal
				set(joint1,'xdata', Bx(i), 'ydata',By(i));
				set(joint3,'xdata', Cx(i), 'ydata',Cy(i));
				set(crank1,'xdata',[Astar(1) Bx(i)],'ydata', [Astar(2) By(i)]);
				set(coupler1,'xdata', [Cx(i), Bx(i)],'ydata', [Cy(i), By(i)]);
				set(coupler2,'xdata', [Bx(i), Dx(i)],'ydata', [By(i), Dy(i)]);
				set(coupler3,'xdata', [ Dx(i), Ex(i)],'ydata', [Dy(i), Ey(i)]);		
				for j=1:1:5
					xblock(j)=xblocka(i,j);
					yblock(j)=yblocka(i,j);
				end
				set(block1,'xdata', xblock, 'ydata',yblock);
				drawnow;
		
% Do meaningless calculation to slow down the graphics

				for k=1:1:timesink
					zz=log10(k);
				end
			end
		end
	
% Ask if animation should be repeated

		ans=input('Repeat animation? y/n [y]: ', 's');
		if isempty(ans);
			ans='y';
		end

% Ask if animation should be slowed down or speeded up
	
		if ans=='y'	
			anst=input('Change for animation speed? +/-/0 [-]: ', 's');
			if isempty(anst);
				anst='-';
			end
			if anst=='-'; timesink=timesink*2; end
			if anst=='+'; timesink=timesink/2; end
			if timesink<1; timesink=1; end	
		end
	end
	
% Ask if we should change assembly mode and reanalyze

	ansm=input('Change assembly mode and repeat analysis? y/n [y]: ', 's');
	if isempty(ansm);
		ansm='y';
	end
	temp=mode;
	if ansm=='y'; mode=-temp; end
end


