function [values] = rbody2(r6,beta,rax,ray,vax,vay,aax,aay,theta,omega,alpha,flag)

% This function determines the kinematic properties of a point (C) on a rigid 
% body when the position of one point (A) and the angular position, velocity,
% and acceleration of the link are already known. 
 
% The input values are:

%r6      = length of vector from point A to point C
%beta    = angle from line through AB to the line AC (degrees)
%rax     = x coordinate of point A
%ray     = y coordinate of point A
%vax     = x component of velocity of A
%vay     = y component of velocity of A
%aax     = x component of acceleration of A
%aay     = y component of acceleration of A
%theta   = angle from horizontal to axix from which beta is measured (degrees)
%omega   = angular velocity of rigid body (rad/sec)
%alpha   = angular acceleration of rigid body (rad/sec^2)
%flag    = analysis flag.  If flag = 1, only a position analysis is conducted. 
%          If flag = 2, both a position and velocity analysis is conducted. 
%		   If flag = 3, a position, velocity, and acceleration analysis
%		               is conducted.

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

%values (1:2)   = x,y components of point C
%values (3:4)   = x,y components of velocity of point C
%values (5:6)   = x,y components of acceleration of point C
%values (7)     = angle between fixed x axis and line AC(degrees)

%compute position information

values=zeros(7,1);
fact=pi/180;
theta6=(theta+beta)*fact;
c6=cos(theta6);
s6=sin(theta6);
values(7)=theta6/fact;
rcx=rax+r6*c6;
rcy=ray+r6*s6;
values(1)=rcx;
values(2)=rcy;

% compute the velocity information if fact > 1

if flag>1
	vcx=vax-r6*omega*s6;
	vcy=vay+r6*omega*c6;
	values(3)=vcx;
	values(4)=vcy;
end
	
% compute the acceleration information if fact > 2

if flag>2
	acx=aax-r6*alpha*s6-r6*omega*omega*c6;
	acy=aay+r6*alpha*c6-r6*omega*omega*s6;
	values(5)=acx;
	values(6)=acy;
end
