% rf_cam.m 
%
% This program determines the cam profile for a translating roller
% follower.  

% Variables for calculation 
%   rb=base circle radius
%   r0=radius of cam follower roller or cylindrical face
%   d = follower offset
%   xc=x coordinate of cam point
%   yc=y coordinate of cam point
%   xf=x coordinate of follower center relative to cam
%   yf=y coordinate of follower center relative to cam
%   rise = maximum follower displacement
%   direction = direction of cam rotation [CW(-), CCW(+)]
%   ainc = cam rotation angle increment (deg)

%   Graphics Variables

%   h1=handle on the first axes in the window (cam animation)
%   h2=handle on the second axes in the window (displacement diagram)
%   h3=handle on the fourth axes in the window (pressure angle)
%   Displ=line representing the follower displacement
%   dbead=bead which follows the displacement throughout the animation
%   Pangle=line representing the pressure angle
%   pabead=bead which follows pressure angle throughout the animation
%   stem=line for follower stem
%   roller=line for follower roller
%   cam=line for cam
%   baset=line for top of slider guide
%   baseb=line for bottom of slider guide
%   pin=line for the cam axis
%   bush=line representing the cam bushing
%   xmin, xmax= x limits for the axes
%   ymin, ymax= y limits for the axes

% program 

clear all;
clf;
Lxt=32;
Lyt1=-0.8;
Lyt2=-0.9;
Lyt3=-1.0;
LxL1=15;
LxL2=30;
ansi='y';
ans='y';
disp('   ')
disp('      Cam Synthesis for Axial Roller Follower')
disp('  ')

% Input values for cam variables 

readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (rf_camio.dat): ', 's');
if isempty(inputfile); inputfile = 'rf_camio.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
if readmode==2
	while ansi=='y'
		rb=input('Enter base circle radius [2]:  ');
		if isempty(rb); rb=2; end
		r0=input('Enter radius of cylindrical or roller follower [0.5]: ');
		if isempty(r0);r0 = .5; end  
		d=input('Enter follower offset [1]:  ');
		if isempty(d); d=1; end
		rise=input('Enter follower rise [2]:  ');
		if isempty(rise); rise=2; end
		direction=input('Enter cam rotation direction (CW(-), CCW(+)) [-]: ','s'); 
		if isempty(direction); direction='-';end
		ainc=input('Enter cam angle increment for design (deg) [10]:  '); 
		if isempty(ainc); ainc=10; end
		ncycle=input('Enter number of animation cycles [4]:  ');
		if isempty(ncycle); ncycle=4; end	
		ansi=input('Reinter input variables? y/n [n]: ', 's');
		if isempty(ansi);
			ansi='n';
		end
	end
	
% save the data to the file fileinput.dat
	
	tempio(1)=rb;
	tempio(2)=r0;
	tempio(3)=d;	
	tempio(4)=rise;
	tempio(5)=direction;
	tempio(6)=ainc;
	tempio(7)=ncycle;
	tempio(8)=ansi;
	tempiot=tempio(:);  % store the transpose of tempio for ease of reading
	fprintf(inputfile,'%f \n',tempiot)
end

% Read input data from the file rf_camio.dat

if readmode==1	
	rf_camio = fscanf(fid,'%f');
	rb=rf_camio(1);
	r0=rf_camio(2);
	d=rf_camio(3);
	rise=rf_camio(4);
	direction=rf_camio(5);
	ainc=rf_camio(6);
	ncycle=rf_camio(7);
	ansi=rf_camio(8);	
end
fclose('all');

% Print heading for input information

disp('   ')
fprintf('    Input Variables ') 

disp('   ')
fprintf('  Base Radius   Follower Radius   Follower Offset    rise') 

% Print the input values.

disp('   ')
fprintf('%10.3f %15.3f %17.3f %12.3f\n',rb, r0, d, rise)
disp('   ')

fact=pi/180;

% Compute the displacement of the roller follower.  The variable
% dtheta will change the resolution of the cam.  Make it smaller for
% accurate results.
		
dtheta=ainc;
if direction=='+'
	dtheta=-ainc;
end
temp=-dtheta;
n=abs(360/dtheta);

% Compute the follower displacement and follower center coordinates 
% relative to cam.

for j=1:1:n
	temp=temp+dtheta;
	theta(j)=temp;
	camangle(j)=abs(theta(j));
	[coords]=follower(camangle(j),rise);
	f(j)=coords(1);
	tt=theta(j)*fact;
	if tt<0; tt=tt+2*pi; end
	ct=cos(tt);
	st=sin(tt);
	Rabs(j)=rb+r0+f(j);
	xf(j)=(rb+r0+f(j))*ct-d*st;
	yf(j)=(rb+r0+f(j))*st+d*ct;	
end

% Plot the displacement profile, velocity and acceleration for 
% the follower.  Start by normalizing f, fp, and fpp

temp=-1;
for j=1:1:360
	temp=temp+1;
	thetal(j)=temp;
	[coords]=follower(thetal(j),rise);
	fl(j)=coords(1);	
	flp(j)=coords(2);
	flpp(j)=coords(3);
end

fmax=max(fl);
fmin=min(fl);
fpmax=max(flp);
fpmin=min(flp);
fppmax=max(flpp);
fppmin=min(flpp);
fn=max([abs(fmax), abs(fmin)]);
fpn=max([abs(fpmax), abs(fpmin)]);
fppn=max([abs(fppmax), abs(fppmin)]);
fnorm=fl/fn;
fpnorm=flp/fpn;
fppnorm=flpp/fppn;

% Set up the axes for displacement drawing.

clf;
width=0.9;
height=0.8;
h6=axes('position', [.1 .1 width height],'box', 'on', 'xcolor',...
       'k','ycolor','k');
set(gcf,'color', 'w')
xlabel('Cam Angle','color', 'k')
ylabel('Follower Displacement, Velocity, and Acceleration','color', 'k')
title('\it{Follower Displacement Diagram}','color', 'r','fontsize', 12)

xmin=0;
xmax=365;
ymina=min([min(fnorm), min(fpnorm), min(fppnorm)]);
ymaxa=max([max(fnorm), max(fpnorm), max(fppnorm)]);
yrange=ymaxa-ymina;
ymin=ymina-0.1*yrange;
ymax=ymaxa+0.1*yrange;
axis([xmin xmax ymin ymax]);

% Set up vectors to be drawn in the follower displacement plot

position=line('xdata', [], 'ydata' ,[], 'linewidth',1,...
      'erasemode', 'none','linestyle', '-', 'color', 'r');
velocity=line('xdata', [], 'ydata' ,[], 'linewidth',0.75,...
      'erasemode', 'none','linestyle', '--','color', 'b');
acceleration=line('xdata', [], 'ydata' ,[], 'linewidth' ,0.5, 'erasemode',...
      'none','linestyle', '-.','color', 'k');

% draw the position, velocity, and acceleration curves

set(gca, 'XTick',[0 40 80 120 160 200 240 280 320 360]);
set(position,'xdata',thetal, 'ydata',fnorm);
set(velocity,'xdata',thetal,'ydata', fpnorm);
set(acceleration,'xdata',thetal, 'ydata',fppnorm);
line('xdata', [LxL1;LxL2], 'ydata' ,[Lyt1;Lyt1], 'linewidth',1,...
      'erasemode', 'none','linestyle', '-', 'color', 'r');
line('xdata', [LxL1;LxL2], 'ydata' ,[Lyt2;Lyt2], 'linewidth',0.75,...
      'erasemode', 'none','linestyle', '--', 'color', 'b');
line('xdata', [LxL1;LxL2], 'ydata' ,[Lyt3;Lyt3], 'linewidth',0.5,...
      'erasemode', 'none','linestyle', '-.', 'color', 'k');
text(Lxt,Lyt1,['Pos.,  max value is: ', num2str(fn) ], 'color', 'r');
text(Lxt,Lyt2,['Vel.,  max value is: ', num2str(fpn), ' ( \omega', ' )' ],'color', 'b');
text(Lxt,Lyt3,['Acc.,  max value is: ', num2str(fppn), ' ( \omega^2', ' )' ], 'color', 'k');
pause

% Reorder the prime points in the order of increasing angle
% if cam rotation direction is CCW (+).

if direction =='+'
	for i=2:1:n
		j=n-i+2;
		rfollower=sqrt(xf(j)^2+yf(j)^2);
		angfoll=atan2(yf(j), xf(j));
		xftmp(i)=rfollower*cos(angfoll);
		yftmp(i)=rfollower*sin(angfoll);
	end
	for i=2:1:n
		xf(i)=xftmp(i);
		yf(i)=yftmp(i);
	end	
end
xf(n+1)=xf(1);
yf(n+1)=yf(1);	

% Compute the radius of curvature

for j=1:1:n
	jp1=j+1;
	jm1=j-1;
	if j==1
		a1=[xf(n), yf(n)];
		b1=[xf(n+1), yf(n+1)];
		a2=b1;
		b2=[xf(jp1), yf(jp1)];
		sign=(xf(n+1)-xf(n))*(yf(jp1)-yf(n+1))-(xf(jp1)-xf(n+1))*(yf(n+1)-yf(n));		
	end
	if j>1
		a1=[xf(jm1), yf(jm1)];
		b1=[xf(j), yf(j)];
		a2=b1;
		b2=[xf(jp1), yf(jp1)];
		sign=(xf(j)-xf(jm1))*(yf(jp1)-yf(j))-(xf(jp1)-xf(j))*(yf(j)-yf(jm1));
	end
	coords=pole(a1,a2,b1,b2);
	xcent(j)=coords(1);
	ycent(j)=coords(2);
	rho(j)=10^10;

% If flag =0, compute the true value for rho

	if coords(3)==0
		rho(j)=sqrt((xf(j)-xcent(j))^2+(yf(j)-ycent(j))^2)-r0;
	end	

% Compute the sign for rho

	if sign ~=0;
		sign=sign/abs(sign);
	end
	if sign==0; sign=1; end
	rho(j)=rho(j)*sign;
end

% If rho is large, truncate it for drawing purposes

rhom=2*sqrt((max(xf))^2+(max(yf))^2);
for j=1:1:n
	rhod(j)=rho(j);
	if abs(rho(j))>rhom
		rhod(j)=rhom*rho(j)/abs(rho(j));
	end
end

% Compute the angle gamma

for j=2:1:n
	k=n-j+2;
	if direction =='-'
		k=j;
	end
	tt=camangle(k)*fact;
	gamma(j)=atan2(yf(j)-ycent(j), xf(j)-xcent(j));	
	if rho(j)<0
		gamma(j)=gamma(j)+pi;
	end
end

% Compute the cam coordinates

for j=1:1:n
	xc(j)=xf(j)-r0*cos(gamma(j));
	yc(j)=yf(j)-r0*sin(gamma(j));
end
	xc(n+1)=xf(1)-r0*cos(gamma(1));
	yc(n+1)=yf(1)-r0*sin(gamma(1));

% Compute the pressure angle

for j=1:1:n
	tt=theta(j)*fact;
	xcc(j)=xc(j)*cos(tt)+yc(j)*sin(tt);
	ycc(j)=-xc(j)*sin(tt)+yc(j)*cos(tt);
	pa(j)=abs(atan2(d-ycc(j), Rabs(j)-xcc(j)))/fact;
end
	
% Set up coordinates of bushings and pins.

rpivot=rb/10;
ninc=20;  
npoints=ninc+1;
centerx=0;
centery=0;
coord=circle2(rpivot,centerx,centery,ninc);

% find coordinates of cam pin

for i=1:1:npoints
	xcoordcp(i)=coord(i,1);
	ycoordcp(i)=coord(i,2);
end

% Find coordinates of cam bushing

binc=ninc/2;
coord1=bushing(rpivot,centerx,centery,binc,0);
bpoints=ninc/2+16;
for i=1:1:bpoints
	xbush(i)=coord1(i,1);
	ybush(i)=coord1(i,2);
end

% Set up coordinates of follower circle in position 1.

nincf=20;  
npointf=ninc+1;
centerxf=xf(1);
centeryf=yf(1);
coord=circle2(r0,centerxf,centeryf,nincf);

% find coordinates of follower circle

for i=1:1:npointf
	xcoordf(i)=coord(i,1);
	ycoordf(i)=coord(i,2);
end

% Find coordinates of follower stem

centerxf=xf(1);
centeryf=yf(1);
stemangle=pi/8;
xstem(1)=centerxf+r0*cos(stemangle);
ystem(1)=centeryf+r0*sin(stemangle);
xstem(2)=xstem(1)+2*rise;
ystem(2)=ystem(1);
xstem(4)=centerxf+r0*cos(stemangle);
ystem(4)=centeryf-r0*sin(stemangle);
xstem(3)=xstem(2);
ystem(3)=ystem(4);

% set up coordinates of top sliderline.
		
length=rise;
ndash=8;
xe = xstem(1)+rise;
ye = ystem(1)+0.3*(ystem(1)-ystem(3));
flag=-1;
coordf1=frameline(length,xe,ye,ndash,0,flag);					
npoints1=3*ndash;
for i=1:1:npoints1
	xsl1(i)=coordf1(i,1);
	ysl1(i)=coordf1(i,2);
end

% set up coordinates of bottom sliderline.

ye = ystem(3)-0.3*(ystem(1)-ystem(3));
flag=1;
coordf2=frameline(length,xe,ye,ndash,0,flag);					
for i=1:1:npoints1
	xsl2(i)=coordf2(i,1);
	ysl2(i)=coordf2(i,2);
end

% Set up the axes for first drawing.

clf;
width=0.80;
height=0.80;
h1=axes('position', [.05 .05 width height],'box', 'on', 'xcolor', 'k','ycolor',...
       'k');
set(gcf,'color', 'w')

cx=[ xsl1, xsl2, xstem, xcoordf, xbush, xcoordcp,xc];
cy=[ ysl1, ysl2, ystem, ycoordf, ybush, ycoordcp,yc];

xmina=min(cx);
xmaxa=max(cx);
ymina=min(cy);
if ymina>-.5; ymina=-0.5; end
ymaxa=max(cy);
rangex=xmaxa-xmina;
rangey=ymaxa-ymina;
xmin=xmina-0.1*rangex;
xmax=xmaxa+0.1*rangex;
ymin=ymina-0.1*rangey;
ymax=ymaxa+0.1*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);

% Set up vectors to be drawn in the first plot

cam=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'none','color', 'k');
roller=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'none','color', 'k');
stem=line('xdata', [], 'ydata' ,[], 'linewidth',1,...
      'erasemode', 'none','color', 'k');
		
baset=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'none','color', 'r');
baseb=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'none','color', 'r');

bush=line('xdata',[],'ydata',[],'erasemode','none','color','r');
pin=line('xdata',[],'ydata',[],'erasemode','none','color','r');

		
% draw the pin and bushing.

set(gca, 'XTickmode','auto');
set(pin,'xdata', xcoordcp, 'ydata',ycoordcp);
set(bush,'xdata', xbush,'ydata', ybush);

% Draw the top and bottom frame lines

set (baset,'xdata', xsl1, 'ydata',ysl1)
set (baseb,'xdata', xsl2, 'ydata',ysl2)

% Draw the cam.

set (cam,'xdata', xc, 'ydata', yc);

% Draw follower roller.

set (roller, 'xdata', xcoordf, 'ydata',ycoordf);

% Draw follower stem

set (stem, 'xdata', xstem, 'ydata',ystem);

% Print a file of the cam coordinates and pressure angle

disp('   ')
fprintf('cam.dat','                Cam coordinates') 
fprintf('cam.dat','       cam angle      xc         yc') 
for i=1:1:n
	fprintf('cam.dat','%12.0f %12.3f %10.3f \n',theta(i),xc(i),yc(i))
end
disp(' *** Computing the cam profile ***')

% **********************************************************

% The cam system is designed, now animate the results.

% **********************************************************

pause;
clf;

% Define the axes for the follower displacement

width=0.38;
height=0.35;
h2=axes('position',[0.6 .55 width height],'box','on','xcolor','k','ycolor',...
      'k');   %axis location
set(gca, 'XTick',[0 40 80 120 160 200 240 280 320 360]);
set(gcf,'color', 'w')
xlabel('Cam Angle','color', 'k')
ylabel('Follower Displacement','color', 'k')
Displ=line('xdata', [], 'ydata', [], 'erasemode', 'none',...
      'color', 'r', 'linestyle', '-');
dbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, 'erase',...
      'xor');

% Define the axes for the pressure angle

h3=axes('position', [0.6 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k');
set(gca, 'XTick',[0 40 80 120 160 200 240 280 320 360]);
xlabel('Cam Angle','color', 'k')
ylabel('Pressure Angle, deg','color', 'k')
Pangle=line('xdata', [], 'ydata', [], 'erasemode', 'none', ...
      'color', 'r', 'linestyle', '-');
pabead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, ...
      'erasemode','xor');

% Define the axes for the radius of curvature

h4=axes('position', [0.1 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k');
set(gca, 'XTick',[0 40 80 120 160 200 240 280 320 360]);
xlabel('Cam Angle','color', 'k')
ylabel('Radius of Curvature ','color', 'k')
radius=line('xdata', [], 'ydata', [], 'erasemode', 'none', ...
      'color', 'r', 'linestyle', '-');
rabead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, ...
      'erasemode','xor');
  
% Set up the vectors needed to draw the linkage

h1=axes('position', [0.1 .55 width height],'box', 'on','xcolor', 'k',...
      'ycolor', 'k');

set(gca, 'XTickmode','auto');
cam=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'xor','color', 'k');
roller=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'xor','color', 'k');
stem=line('xdata', [], 'ydata' ,[], 'linewidth',1,...
      'erasemode', 'xor','color', 'k');
		
baset=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'none','color', 'r');
baseb=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode',...
      'none','color', 'r');

bush=line('xdata',[],'ydata',[], 'erasemode','none','color', 'r');
pin=line('xdata',[],'ydata',[], 'erasemode','none','color', 'r');
 
% Loop to analyze the cam machanism for a full cycle

nposn=30;
rotinc=floor(n/nposn);  %rotation increment to get nrot positions per rotation
k=0;
jj=0;
for j=1:rotinc:n
	jj=jj+1;
	Q21=-theta(j);
	Q2 = Q21*fact;
	ct=cos(Q2);
	st=sin(Q2);

% Translate the follower by f(j)

	for i=1:1:4
		k=k+1;
		xstema(jj,i)=xstem(i)+f(j);
		ystema(jj,i)=ystem(i);
		tempx(k)=xstema(jj,i);
		tempy(k)=ystema(jj,i);
	end
	for i=1:1:npointf
		k=k+1;	
		xfollower(jj,i)=xcoordf(i)+f(j);
		yfollower(jj,i)=ycoordf(i);
		tempx(k)=xfollower(jj,i);
		tempy(k)=yfollower(jj,i);
	end

% rotate the cam coordinates by theta(j)

	ii=0;
	for i=1:rotinc:n
		ii=ii+1;
		k=k+1;	
		xca(jj,ii)=xc(i)*ct-yc(i)*st;
		yca(jj,ii)=xc(i)*st+yc(i)*ct;	
		tempx(k)=xca(jj,ii);
		tempy(k)=yca(jj,ii);
	end
	nn=ii;
	xca(jj,ii+1)=xc(1)*ct-yc(1)*st;
	yca(jj,ii+1)=xc(1)*st+yc(1)*ct;	
end

% Set the axis limits for the graphs.  Start with the cam mechanism
% position graph
		 
axes(h1);
xmina=min(tempx)-0.3*abs(min(tempx));
xmaxa=max(tempx)+0.1*abs(max(tempx));
ymina=min(tempy)-0.2*abs(min(tempy));
ymaxa=max(tempy)+0.2*abs(max(tempy));
rangex=xmaxa-xmina;
rangey=ymaxa-ymina;
xmin=xmina-0.05*rangex;
xmax=xmaxa+0.05*rangex;
ymin=ymina-0.05*rangey;
ymax=ymaxa+0.05*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);

axes(h3);
ymin=min(pa)-0.1*abs(max(pa)-min(pa));
ymax=max(pa)+0.1*abs(max(pa)-min(pa));
axis([0 360 ymin ymax]);  %set axis limits for the pressure angle

axes(h2);
ymin=min(f)-0.1*abs(max(f)-min(f));
ymax=max(f)+0.1*abs(max(f)-min(f));
axis([0 360 ymin ymax]);  %set axis limits for the displacement plot

axes(h4);
ymin=min(rhod)-0.1*abs(max(rhod)-min(rhod));
ymax=max(rhod)+0.1*abs(max(rhod)-min(rhod));
axis([0 360 ymin ymax]);  %set axis limits for the radius of curvature plot

% Plot the results.

itotal=n;
ans='y';
	
% Draw the fixed curves first.

set(pin,'xdata', xcoordcp, 'ydata',ycoordcp);
set(bush,'xdata', xbush,'ydata', ybush);
set (baset,'xdata', xsl1, 'ydata',ysl1)
set (baseb,'xdata', xsl2, 'ydata',ysl2)
set(Displ,'xdata', camangle,'ydata', f);
set(Pangle,'xdata', camangle,'ydata', pa);
set(radius,'xdata', camangle,'ydata', rho);

% Animate the results as long as ans = 'y'

timesink=1;
while ans=='y'
	for kk=1:1:ncycle
		ii=0;
		for i=1:rotinc:n
			ii=ii+1;
			for j=1:1:nn+1
				xcam(j)=xca(ii,j);
				ycam(j)=yca(ii,j);
			end
			for j=1:1:4
				xstemn(j)=xstema(ii,j);
				ystemn(j)=ystema(ii,j);
			end
			for j=1:1:npointf
				xroller(j)=xfollower(ii,j);
				yroller(j)=yfollower(ii,j);
			end	

			axes(h1);
			set(cam,'xdata',xcam,'ydata', ycam);
			set(roller,'xdata', xroller,'ydata', yroller);
			set(stem,'xdata', xstemn,'ydata', ystemn);

			set(pabead, 'xdata', camangle(i), 'ydata', pa(i));

			set(rabead, 'xdata', camangle(i), 'ydata', rho(i));

			set(dbead, 'xdata', camangle(i), 'ydata', f(i));
			drawnow;   %flush the draw buffer

% Do meaningless calculation to slow down the graphics

			for k=1:1:timesink
				zz=log10(k);
			end
		end
	end

% Plot figures in final position.  Plot position first.

	i = nposn/2;
		
	for j=1:1:nn+1
		xcam(j)=xca(i,j);
		ycam(j)=yca(i,j);
	end
	for j=1:1:4
		xstemn(j)=xstema(i,j);
		ystemn(j)=ystema(i,j);
	end
	for j=1:1:npointf
		xroller(j)=xfollower(i,j);
		yroller(j)=yfollower(i,j);
	end	
		
	set(cam,'xdata',xcam,'ydata', ycam);
	set(roller,'xdata', xroller,'ydata', yroller);
	set(stem,'xdata', xstemn,'ydata', ystemn);
	set(pabead, 'xdata', camangle(i), 'ydata', pa(i));
	set(dbead, 'xdata', camangle(i), 'ydata', f(i));
	set(rabead, 'xdata', camangle(i), 'ydata', rho(i));

%flush the draw buffer

	drawnow; 

% Ask if the animation should be repeated

	ans=input('Repeat animation? y/n [y]: ', 's');
	if isempty(ans);
		ans='y';
	end

% Ask if animation should be slowed down or speeded up
	
	if ans=='y'	
		anst=input('Change for animation speed? +/-/0 [-]: ', 's');
		if isempty(anst);
			anst='-';
		end
		if anst=='-'; timesink=timesink*4; end
		if anst=='+'; timesink=timesink/4; end
		if timesink<1; timesink=1; end	
	end
end
