function [values] = rpprc(r1,theta1,theta2,td2,tdd2,beta,flag)

% This function analyzes an rppr mechanism when the crank (link 2)
% is the driving link.  The input values are:

%r1      = length of frame
%theta1  = angle from frame x axis to vector r1 (degrees)
%theta2  = angle from frame x axis to vector r2 (degrees)
%td3     = angular velocity of link 2 (rad/sec)
%tdd4    = angular acceleration of link 2 (rad/sec^2)
%beta	 = constant angle from r2 to r4 (degrees) 
%flag    = analysis flag.  If flag = 1, only a position analysis is conducted. 
%          If flag = 2, both a position and velocity analysis are conducted. 
%		   If flag = 3, position, velocity, and acceleration analyses
%		             are conducted.

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

%values (1:4)   = vector lengths
%values (5:8)   = vector angles (degrees)
%values (9:12)  = derivatives of vector lengths
%values (13:16) = derivatives of vector angles (rad/sec)
%values (17:20) = second derivatives of vector lengths
%values (21:24) = second derivatives of vector angles (rad/sec^2)
%values (25:26) = x,y components of position of slider point (point P)
%values (27:28) = x,y components of position of frame point (point Q)
%values (29:30) = x,y components of velocity of point P2 on crank
%values (31:32) = x,y components of velocity of point P3 on slider
%values (33:34) = x,y components of acceleration of point P2 on crank
%values (35:36) = x,y components of acceleration of point P3 on slider
%values (37)    = assembly flag.  If values(37) = 0, mechanism cannot 
%                 assembled.

%convert input data

values=zeros(37,1);
r(1)=r1;
r(3)=0;
theta(3)=0;
theta4=theta2+beta;
theta(1)=theta1;
theta(2)=theta2;
theta(4)=theta4;
fact=pi/180;
t1=theta(1)*fact;
t2=theta(2)*fact;
t4=theta(4)*fact;
td=zeros(4,1);
tdd=zeros(4,1);
rd=zeros(4,1);
rdd=zeros(4,1);
td(2)=td2;
tdd(2)=tdd2;
td(4)=td2;
tdd(4)=tdd2;
s1=sin(t1);
c1=cos(t1);
s2=sin(t2);
c2=cos(t2);
s4=sin(t4);
c4=cos(t4);

% position calculations

AM=[c2 -c4; s2 -s4];
BM=[r(1)*c1; r(1)*s1];
CM=AM\BM;
r(2)=CM(1);
r(4)=CM(2);

% coordinates of P and Q
	
values(25)=r(2)*c2;
values(26)=r(2)*s2;
values(27)=r(1)*c1;
values(28)=r(1)*s1;

%velocity calculation
	
if flag>1
	BM=[r(2)*td(2)*s2-r(4)*td(2)*s4; -r(2)*td(2)*c2+r(4)*td(2)*c4];
	CM=AM\BM;
	rd(2)=CM(1);
	rd(4)=CM(2);
	
	% velocities of P and Q
	
	values(29)=-r(2)*td(2)*s2;
	values(30)=r(2)*td(2)*c2;
	values(31)=values(29)+rd(2)*c2;
	values(32)=values(30)+rd(2)*s2;
end
	
%acceleration calculation
	
if flag>2
	BM=[(r(2)*tdd(2)+2*rd(2)*td(2))*s2-(r(4)*tdd(4)+2*rd(4)*td(4))*s4+...
	r(2)*td(2)^2*c2-r(4)*td(4)^2*c4;...
	 -(r(2)*tdd(2)+2*rd(2)*td(2))*c2+(r(4)*tdd(4)+2*rd(4)*td(4))*c4+...
	r(2)*td(2)^2*s2-r(4)*td(4)^2*s4];
	CM=AM\BM;
	rdd(2)=CM(1);
	rdd(4)=CM(2);
		
	% accelerations of P and Q
	
	values(33)=-r(2)*tdd(2)*s2-r(2)*td(2)^2*c2;
	values(34)=r(2)*tdd(2)*c2-r(2)*td(2)^2*s2;
	values(35)=values(33)+rdd(2)*c2-2*rd(2)*td(2)*s2;
	values(36)=values(34)+rdd(2)*s2+2*rd(2)*td(2)*c2;
end

%store results in array values

for i = 1:4
	values(i)=r(i);
	values(i+4)=theta(i);
	values(i+8)=rd(i);
	values(i+12)=td(i);
	values(i+16)=rdd(i);
	values(i+20)=tdd(i);
end
