function [values] = rpprs(r1,theta1,r2,rd2,rdd2,beta,sigma,flag)

% This function analyzes an rppr mechanism when the crank (link 2)
% is the driving link.  The input values are:

%r1      = length of frame
%theta1  = angle from frame x axis to vector r1 (degrees)
%r2      = length of vector 2
%rd2     = derivative of length of vector 2
%rdd2    = second derivative of length of vector 2
%beta	 = constant angle from r2 to r4 (degrees) 
%sigma   = +1 or -1.  Identifies assembly mode
%flag    = analysis flag.  If flag = 1, only a position analysis is conducted. 
%          If flag = 2, both a position and velocity analysis are conducted. 
%		   If flag = 3, position, velocity, and acceleration analyses
%		             are conducted.

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

%values (1:4)   = vector lengths
%values (5:8)   = vector angles (degrees)
%values (9:12)  = derivatives of vector lengths
%values (13:16) = derivatives of vector angles (rad/sec)
%values (17:20) = second derivatives of vector lengths
%values (21:24) = second derivatives of vector angles (rad/sec^2)
%values (25:26) = x,y components of position of slider point (point P)
%values (27:28) = x,y components of position of frame point (point Q)
%values (29:30) = x,y components of velocity of point P2 on crank
%values (31:32) = x,y components of velocity of point P3 on slider
%values (33:34) = x,y components of acceleration of point P2 on crank
%values (35:36) = x,y components of acceleration of point P3 on slider
%values (37)    = assembly flag.  If values(37) = 0, mechanism cannot 
%                 assembled.

%convert input data

values=zeros(37,1);
r(1)=r1;
r(2)=r2;
betar=beta*pi/180;
theta(1)=theta1;
fact=pi/180;
t1=theta(1)*fact;
td=zeros(4,1);
tdd=zeros(4,1);
rd=zeros(4,1);
rdd=zeros(4,1);
rd(2)=rd2;
rdd(2)=rdd2;
s1=sin(t1);
c1=cos(t1);

% position calculations

arg1=r(2)^2*(cos(betar)^2-1)+r(1)^2;
if arg1>=0
	values(37)=1;
	r(4)=r(2)*cos(betar)+sigma*sqrt(arg1);
	A=r(1)*s1-r(4)*sin(betar);
	B=2*(r(4)*cos(betar)-r(2));
	C=r(1)*s1+r(4)*sin(betar);	
	arg2=B^2-4*A*C;
	if arg2>=0
		t21=2*atan((-B+sqrt(arg2))/(2*A))
		t22=2*atan((-B-sqrt(arg2))/(2*A))
		er1=r(2)*cos(t21)-r(1)*c1-r(4)*cos(t21+betar)
		er2=r(2)*cos(t22)-r(1)*c1-r(4)*cos(t22+betar)

% find sign for square root 

		gama=1;
		t2=t21;
		if(abs(er1)>abs(er2))
			gama=-1;
			t2=t22;
		end
	end	
	theta(2)=t2/fact;
	theta(4)=theta(2)+beta;
	s2=sin(t2);
	c2=cos(t2);
	t4=theta(4)*fact;
	c4=cos(t4);
	s4=sin(t4);
	
	% coordinates of P and Q
	
	values(25)=r(2)*c2;
	values(26)=r(2)*s2;
	values(27)=r(1)*c1;
	values(28)=r(1)*s1;

	% velocity calculation
	
	if flag>1
		AM=[-r(2)*s2+r(4)*s4, -c4; r(2)*c2-r(4)*c4, -s4];
		BM=[-rd(2)*c2;-rd(2)*s2];
		CM=AM\BM;
		td(2)=CM(1)
		td(4)=td(2)
		rd(4)=CM(2);
	
	% velocities of P2 and P3
	
		values(29)=-r(2)*td(2)*s2;
		values(30)=r(2)*td(2)*c2;
		values(31)=values(29)+rd(2)*c2;
		values(32)=values(30)+rd(2)*s2;
	end
	
	%acceleration calculation
	
	if flag>2
		BM=[2*rd(2)*td(2)*s2-2*rd(4)*td(4)*s4+r(2)*td(2)^2*c2-r(4)*td(4)^2*c4-rdd(2)*c2;
		    -2*rd(2)*td(2)*c2+2*rd(4)*td(4)*c4+r(2)*td(2)^2*s2-r(4)*td(4)^2*s4-rdd(2)*s2];
		CM=AM\BM;
		tdd(2)=CM(1);
		rdd(4)=CM(2);
		tdd(4)=tdd(2);
		
	% accelerations of P2 and P3
	
		values(33)=-r(2)*tdd(2)*s2-r(2)*td(2)^2*c2;
		values(34)=r(2)*tdd(2)*c2-r(2)*td(2)^2*s2;
		values(35)=values(33)+rdd(2)*c2-2*rd(2)*td(2)*s2;
		values(36)=values(34)+rdd(2)*s2+2*rd(2)*td(2)*c2;
	end
end

%store results in array values

for i = 1:4
	values(i)=r(i);
	values(i+4)=theta(i);
	values(i+8)=rd(i);
	values(i+12)=td(i);
	values(i+16)=rdd(i);
	values(i+20)=tdd(i);
end
