function [values] = rprpc(r1,r2,theta1,theta2,td2,tdd2,flag)

% This function analyzes an rprp mechanism when the crank 
% is the driving link.  The input values are:

%r1      = length of vector 1 (frame offset)
%r2      = length of vector 4 (crank offset)
%theta1  = angle from frame x axis to vector r1 (degrees).
%theta2  = crank angle (degrees)
%td2     = crank angular velocity (rad/sec)
%tdd2    = crank angular acceleration (rad/sec^2)
%flag    = analysis flag.  If flag = 1, only a position analysis is conducted. 
%          If flag = 2, both a position and velocity analysis are conducted. 
%		   If flag = 3, position, velocity, and acceleration analyses
%		             are conducted.

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

%values (1:4)   = vector lengths
%values (5:8)   = vector angles (degrees)
%values (9:12)  = derivatives of vector lengths
%values (13:16) = derivatives of vector angles (rad/sec)
%values (17:20) = second derivatives of vector lengths
%values (21:24) = second derivatives of vector angles (rad/sec^2)
%values (25:26) = x,y components of position of slider pin (point P)
%values (27:28) = x,y components of position of slider offset point (point Q)
%values (29:30) = x,y components of velocity of slider pin (point P)
%values (31:32) = x,y components of velocity of slider offset point (point Q)
%values (33:34) = x,y components of acceleration of slider pin (point P)
%values (35:36) = x,y components of acceleration of slider offset point (point Q)

%convert input data

values=zeros(37,1);
r(1)=r1;
r(2)=r2;
theta(1)=theta1;
theta(2)=theta2;
theta(3)=theta(2)+90;
theta(4)=theta(1)+90;
fact=pi/180;
t1=theta(1)*fact;
t2=theta(2)*fact;
t3=theta(3)*fact;
t4=theta(4)*fact;
td=zeros(4,1);
tdd=zeros(4,1);
rd=zeros(4,1);
rdd=zeros(4,1);
td(2)=td2;
td(3)=td2;
tdd(2)=tdd2;
tdd(3)=tdd2;
s1=sin(t1);
c1=cos(t1);
s2=sin(t2);
c2=cos(t2);
s3=sin(t3);
c3=cos(t3);
s4=sin(t4);
c4=cos(t4);

% position calculations

A=[c3 -c4;s3 -s4];
B=[r1*c1-r2*c2;r1*s1-r2*s2];
C=A\B;
r(3)=C(1,1);
r(4)=C(2,1);	

% coordinates of P and Q
	
values(25)=r(2)*c2+r(3)*c3;
values(26)=r(2)*s2+r(3)*s3;
values(27)=r(1)*c1+r(4)*c4;
values(28)=r(1)*s1+r(4)*s4;

%velocity calculation
	
if flag>1
	AM=A;
	BM=[r(2)*td(2)*s2+r(3)*td(3)*s3;-r(2)*td(2)*c2-r(3)*td(3)*c3];
	CM=AM\BM;
	rd(3)=CM(1);
	rd(4)=CM(2);
	
	% velocities of P and Q
	
	values(29)=rd(4)*c4;
	values(30)=rd(4)*s4;
	values(31)=-r(2)*td(2)*s2;
	values(32)=r(2)*td(2)*c2;
end
	
%acceleration calculation
	
if flag>2
	BM=[r(2)*tdd(2)*s2+r(2)*td(2)*td(2)*c2+r(3)*tdd(3)*s3+...
		r(3)*td(3)*td(3)*c3+2*rd(3)*td(3)*s3;...
    	-r(2)*tdd(2)*c2+r(2)*td(2)*td(2)*s2-r(3)*tdd(3)*c3+...
		r(3)*td(3)*td(3)*s3-2*rd(3)*td(3)*c3];
	CM=AM\BM;
	rdd(3)=CM(1);
	rdd(4)=CM(2);
		
	% accelerations of P and Q
	
	values(33)=rdd(4)*c4;
	values(34)=rdd(4)*s4;
	values(35)=-r(2)*tdd(2)*s2-r(2)*td(2)*td(2)*c2;
	values(36)=r(2)*tdd(2)*c2-r(2)*td(2)*td(2)*s2;
end

%store results in array values

for i = 1:4
	values(i)=r(i);
	values(i+4)=theta(i);
	values(i+8)=rd(i);
	values(i+12)=td(i);
	values(i+16)=rdd(i);
	values(i+20)=tdd(i);
end
