function [values] = rrppc(r2,theta1,theta2,td2,tdd2,beta,flag)

% This function analyzes an rrpp mechanism when the crank (link 2)
% is the driving link.  The input values are:

%r2      = length of vector 2 (crank)
%theta1  = angle from frame x axis to vector r1 (degrees).
%theta2  = crank angle (degrees)
%td2     = crank angular velocity (rad/sec)
%tdd2    = crank angular acceleration (rad/sec^2)
%beta	 = angle between slider line on coupler and slider line on 
%          frame (degrees)
%flag    = analysis flag.  If flag = 1, only a position analysis is conducted. 
%          If flag = 2, both a position and velocity analysis are conducted. 
%		   If flag = 3, position, velocity, and acceleration analyses
%		             are conducted.

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

%values (1:4)   = vector lengths
%values (5:8)   = vector angles (degrees)
%values (9:12)  = derivatives of vector lengths
%values (13:16) = derivatives of vector angles (rad/sec)
%values (17:20) = second derivatives of vector lengths
%values (21:24) = second derivatives of vector angles (rad/sec^2)
%values (25:26) = x,y components of position of point P
%values (27:28) = x,y components of position of point Q
%values (29:30) = x,y components of velocity of point P
%values (31:32) = x,y components of velocity of point Q
%values (33:34) = x,y components of acceleration of point P
%values (35:36) = x,y components of acceleration of point Q

%convert input data

values=zeros(37,1);
r(2)=r2;
theta(1)=theta1;
theta(2)=theta2;
theta(3)=theta(1)+beta;
fact=pi/180;
t1=theta(1)*fact;
t2=theta(2)*fact;
t3=theta(3)*fact;
td=zeros(4,1);
tdd=zeros(4,1);
rd=zeros(4,1);
rdd=zeros(4,1);
td(2)=td2;
tdd(2)=tdd2;
s1=sin(t1);
c1=cos(t1);
s2=sin(t2);
c2=cos(t2);
s3=sin(t3);
c3=cos(t3);

% position calculations

A=[c1 c3;s1 s3];
B=[r2*c2;r2*s2];
C=A\B;
r(1)=C(1,1);
r(3)=C(2,1);	

% coordinates of P and Q
	
values(25)=r(2)*c2;
values(26)=r(2)*s2;
values(27)=r(1)*c1;
values(28)=r(1)*s1;

%velocity calculation
	
if flag>1
	AM=A;
	BM=[-r(2)*td(2)*s2;r(2)*td(2)*c2];
	CM=AM\BM;
	rd(1)=CM(1);
	rd(3)=CM(2);
	
	% velocities of P and Q
	
	values(29)=-r(2)*td(2)*s2;
	values(30)=r(2)*td(2)*c2;
	values(31)=rd(1)*c2;
	values(32)=rd(1)*s2;
end
	
%acceleration calculation
	
if flag>2
	BM=[-r(2)*tdd(2)*s2-r(2)*td(2)*td(2)*c2;...
    	 r(2)*tdd(2)*c2-r(2)*td(2)*td(2)*s2];
	CM=AM\BM;
	rdd(1)=CM(1);
	rdd(3)=CM(2);
		
	% accelerations of P and Q
	
	values(33)=-r(2)*tdd(2)*s2-r(2)*td(2)*td(2)*c2;
	values(34)=r(2)*tdd(2)*c2-r(2)*td(2)*td(2)*s2;
	values(35)=rdd(1)*c1;
	values(36)=rdd(1)*s1;
end

%store results in array values

for i = 1:3
	values(i)=r(i);
	values(i+4)=theta(i);
	values(i+8)=rd(i);
	values(i+12)=td(i);
	values(i+16)=rdd(i);
	values(i+20)=tdd(i);
end
