function [values] = rrpps(r1,r2,rd1,rdd1,theta1,sigma,beta,flag)

% This function analyzes an rrpp mechanism when the slider on the frame
% is the driving link.  The input values are:

%r1      = length of vector 1 (frame offset)
%r2      = length of vector 2 (crank offset)
%rd1     = derivative of length of vector 1
%rdd1    = second derivative of length of vector 1
%theta1  = angle from frame x axis to vector r1 (degrees).
%sigma   = +1 or -1.  Identifies offset configuration
%beta	 = angle between slider line on coupler and slider line on 
%          frame (degrees)
%flag    = analysis flag.  If flag = 1, only a position analysis is conducted. 
%          If flag = 2, both a position and velocity analysis are conducted. 
%		   If flag = 3, position, velocity, and acceleration analyses
%		             are conducted.

% The results are returned in the vector "values".  The answers are 
% stored in values according to the following:

%values (1:4)   = vector lengths
%values (5:8)   = vector angles (degrees)
%values (9:12)  = derivatives of vector lengths
%values (13:16) = derivatives of vector angles (rad/sec)
%values (17:20) = second derivatives of vector lengths
%values (21:24) = second derivatives of vector angles (rad/sec^2)
%values (25:26) = x,y components of position of slider pin (point P)
%values (27:28) = x,y components of position of slider offset point (point Q)
%values (29:30) = x,y components of velocity of slider pin (point P)
%values (31:32) = x,y components of velocity of slider offset point (point Q)
%values (33:34) = x,y components of acceleration of slider pin (point P)
%values (35:36) = x,y components of acceleration of slider offset point (point Q)
%values (37)    = assembly flag.  If values(37) = 0, mechanism cannot 
%                 assembled.

%convert input data

values=zeros(37,1);
r(1)=r1;
r(2)=r2;
theta(1)=theta1;
theta(3)=theta1+beta;
fact=pi/180;
t1=theta(1)*fact;
t3=theta(3)*fact;
td=zeros(4,1);
tdd=zeros(4,1);
rd=zeros(4,1);
rdd=zeros(4,1);
rd(1)=rd1;
rdd(1)=rdd1;
s1=sin(t1);
c1=cos(t1);
s3=sin(t3);
c3=cos(t3);

% position calculations

B=r(1)*(c1*c3+s1*s3);
C=r(1)^2-r(2)^2;
arg=B^2-C;
if (arg>=0)
	values(37)=1; %assembly flag
	r(3)=-B+sigma*sqrt(arg);
	t2=atan2((r(1)*s1+r(3)*s3),(r(1)*c1+r(3)*c3));
	theta(2)=t2/fact;
	s2=sin(t2);
	c2=cos(t2);

	% coordinates of P and Q
	
	values(25)=r(2)*c2;
	values(26)=r(2)*s2;
	values(27)=r(1)*c1;
	values(28)=r(1)*s1;

%velocity calculation
	
	if flag>1
		AM=[-r(2)*s2 -c3; r(2)*c2 -s3];
		BM=[rd(1)*c1; rd(1)*s1];
		CM=AM\BM;
		td(2)=CM(1);
		rd(3)=CM(2);
	
		% velocities of P and Q
	
		values(29)=-r(2)*td(2)*s2;
		values(30)=r(2)*td(2)*c2;
		values(31)=rd(1)*c2;
		values(32)=rd(1)*s2;
	end
	
%acceleration calculation
	
	if flag>2
		BM=[rdd(1)*c1+r(2)*td(2)*td(2)*c2; rdd(1)*s1+r(2)*td(2)*td(2)*s2];
		CM=AM\BM;
		tdd(2)=CM(1);
		rdd(3)=CM(2);
		
		% accelerations of P and Q
	
		values(33)=-r(2)*tdd(2)*s2-r(2)*td(2)*td(2)*c2;
		values(34)=r(2)*tdd(2)*c2-r(2)*td(2)*td(2)*s2;
		values(35)=rdd(1)*c1;
		values(36)=rdd(1)*s1;
	end
end

%store results in array values

for i = 1:3
	values(i)=r(i);
	values(i+4)=theta(i);
	values(i+8)=rd(i);
	values(i+12)=td(i);
	values(i+16)=rdd(i);
	values(i+20)=tdd(i);
end
