% sixbar.m

% MATLAB program for analyzing sixbar Watt's mechanism.

%Variables
%   r1=frame length (cm)
%   r2=crank length (cm)
%   r3=coupler length (cm)
%   r4=rocker length (cm)
%   cr1=coupler radius (cm) (to point D1 from the crank pin)
%   beta1=coupler angle (deg) (to point D2 from the rocker pin)
%   cpad=coupler angle (rad) (to point D2 from the rocker pin)
%   dmode=assembly mode for dyad (+1 or -1)
%   Q1=frame angle (deg)
%   Q11=frame angle (rad)
%   Q2=crank angle (rad)
%   Q21=crank angle (deg)
%   Q22=crank angle (deg) (vector)
%   Q3,Q33=coupler angle (deg)
%   Q4,Q44=rocker angle (deg)
%   w2=angular velocity of the crank (rad/sec)
%   w22=w2 (vector)
%   w33=angular velocity of the coupler (rad/sec)
%   w44=angular velocity of the rocker (rad/sec)
%   cang1=angle from the line connecting the coupler point one and the crank pin (rad)
%   cang2=angle from the line connecting the coupler point two and the rocker pin (rad)
%   mode,assem=assembly mode
%   tt=number of cycles to animate
%   dt=number of animation positions per revolution
%   itotal=number of animation positions
%   cycle=number of cycles as an integer
%   bush1,bush2=x,y coordinates of the bushings
%   cir1,cir2=x,y coordinates of the circles inside the bushings
%   Bx,By=x,y coordinates of the crank pin
%   Cx,Cy=x,y coordinates of the rocker pin
%   Bx2,By2=x,y coordinates of the crank pointer
%   Cx2,Cy2=x,y coordinates of the rocker pointer
%   d2r,r2d=conversion factors between radians and degrees
%   i=counting variable
%   ans=repitition variable

%Graphics variables
%   xmin,xmax,ymin,ymax=axes limits
%   h1=handle on the first axes in the window (crank angle vs. rocker angle)
%   h2=handle on the second axes in the window (crank angle vs. rocker velocity)
%   h3=handle on the third axes in the window (crank angle vs. input torque)
%   h4=handle on the fourth axes in the window (linkage animation)
%   displ=line representing the crank angle vs. rocker angle
%   vel=line representing the crank angle vs. rocker velocity
%   dbead=bead which follows the crank angle/rocker angle throughout animation
%   vbead=bead which follows the crank angle/rocker velocity throughout animation
%   tbead=bead which follows the crank angle/input torque throughout animation
%   bushy1=line representing the bushing at the crank hinge
%   bushy2=line representing the bushing at the rocker hinge
%   circ1=line representing the circle inside the bushing at the crank hinge 
%   circ2=line representing the circle inside the bushing at the rocker hinge
%   joint1=line representing the joint at the crank pin 
%   joint2=line representing the joint at the rocker pin
%   crank=line representing the crank 
%   coupler=line representing the coupler
%   rocker=line representing the rocker
%   couplerptr=line representing the coupler point
%   couplerpt1=line from the crank pin to the coupler point one
%   couplerpt2=line from the rocker pin to the coupler point two

%Program  
clf;
clear all;
ans='y';
disp('   ')
disp('        Sixbar Dwell Mechanism Analysis Program')

%Input the linkage variables

readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (sixbario.dat): ', 's');
if isempty(inputfile); inputfile = 'sixbario.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
if readmode==2
	tt=input('Enter number of cycles (rev) [3]: ');
	if isempty(tt);tt = 3;end
	r1=input('Enter the frame length (cm) [4]: ');
	if isempty(r1);r1 = 4;end
	r2=input('Enter the crank length (cm) [1]: ');
	if isempty(r2);r2 = 1;end
	r3=input('Enter the coupler length (cm) [3]: ');
	if isempty(r3);r3 = 3;end
	r4=input('Enter the rocker length (cm) [4]: ');
	if isempty(r4);r4 = 4;end
	r5=input('Enter the length or r5 (cm) [1.576]: ');
	if isempty(r5);r5 = 1.576;end
	r6=input('Enter the length or r6 (cm) [2.544]: ');
	if isempty(r6);r6 = 2.544;end
	Gx=input('Enter the x coordinate of pivot G (cm) [1.075] ');
	if isempty(Gx);Gx = 1.075;end
	Gy=input('Enter the y coordinate of pivot G (cm) [3.615]: ');
	if isempty(Gy);Gy = 3.615;end
	cr=input('Enter coupler point radius (cm) [1.579]: ');
	if isempty(cr);cr=1.579;end
	beta=input('Enter angle from coupler line to coupler point (deg) [30]: ');
	if isempty(beta);beta=30;end
	Q1=input('Enter the frame angle (deg) [0]: ');
	if isempty(Q1);Q1 = 0;end
	w2=input('Enter the angular velocity of crank(rad/sec) [1]: ');
	if isempty(w2);w2 = 1;end
	dmode=input('Enter the assembly mode for the dyad (+1 or -1) [1]: ');
	if isempty(dmode);dmode = 1;end
	
% save the data to the file fileinput.dat
	
	tempio(1)=tt;
	tempio(2)=r1;
	tempio(3)=r2;	
	tempio(4)=r3;
	tempio(5)=r4;
	tempio(6)=r5;
	tempio(7)=r6;
	tempio(8)=Gx;
	tempio(9)=Gy;
	tempio(10)=cr;
	tempio(11)=beta;
	tempio(12)=Q1;
	tempio(13)=w2;
	tempio(14)=dmode;	
	tempiot=tempio(:);  % store the transpose of tempio for ease of reading
	fprintf(inputfile,'%f \n',tempiot)
end

% Read input data from the file sixbario.dat

if readmode==1	
	sixbario = fscanf(fid,'%f');
	tt=sixbario(1);
	r1=sixbario(2);
	r2=sixbario(3);
	r3=sixbario(4);
	r4=sixbario(5);
	r5=sixbario(6);
	r6=sixbario(7);
	Gx=sixbario(8);
	Gy=sixbario(9);
	cr=sixbario(10);
	beta=sixbario(11);
	Q1=sixbario(12);
	w2=sixbario(13);
	dmode=sixbario(14);
end
Q11=Q1*pi/180;
disp('I am working ...');

% Define constants

i=0;
d2r=pi/180;
r2d=180/pi;
mode=-1;
dt=10;
 
% Loop to analyze the linkage

for Q21 = 0:dt:360;
	i=i+1;
	Q2 = Q21*d2r;

% Call function to compute the fourbar position

	para = fourbar(r1,r2,r3,r4,Q21,w2,0,mode,Q1,2);
	Q3=para(7);
	Q33(i)=Q3;
	Q4=para(8);
	Q44(i)=Q4;
	w33(i)=para(11);
	w44(i)=para(12);
	assem=para(29);
	cang=(Q3+beta)*d2r;

% Store position information in matrices.

	Bx(i)= para(17);
	By(i)= para(18);
	Cx(i)= para(19);
	Cy(i)= para(20);
	Ex(i)=Bx(i)+cr*cos(cang);
	Ey(i)=By(i)+cr*sin(cang);
	Q22(i) = Q21;
	w22(i) = w2; 

% Analyze the rigid body

	value = rbody1(cr,beta, para(17),para(18),para(19),para(20), ...
		para(21),para(22),para(23),para(24),para(25),para(26),...
		para(27),para(28),2);

% Analyze the dyad

	values = dyad(r5,r6,Ex(i),Ey(i),value(3),value(4),value(5),value(6),Gx,...
		         Gy,0,0,0,0,dmode,2,0);
	Fx(i)= values(1);
	Fy(i)= values(2);
	Q6(i)=values(4);
	w66(i)=values(8);
	if values(13)==0;i=i-1;end
end
itotal=i;

% Check that the output dyad for the linkage can be assembled.

if itotal == 0
	disp('   ')
	disp(' **********************************************  ')	
	disp('        Output dyad cannot be assembled ')
	disp(' **********************************************  ')	
	disp('   ')
end	

if itotal > 0

% Compute the angle change for the output link
	
	Q6min=min(Q6);
	for k=1:itotal
		Q66(k)=Q6(k)-Q6min;
	end		
	Ax= 0;
	Ay= 0;
	Dx= r1*cos(Q1*pi/180);
	Dy= r1*sin(Q1*pi/180);

% Get the bushing points

	bush1=bushing((r1/30),0,0,20,0);
	cir1=circle2((r1/40),0,0,20);
	bush2=bushing((r1/30),Dx,Dy,20,0);
	cir2=circle2((r1/40),Dx,Dy,20);
	bush3=bushing((r1/30),Gx,Gy,20,0);
	cir3=circle2((r1/40),Gx,Gy,20);

% Define the axes for the graph showing the dwell linages

	clf;
	height=0.8;
	width=0.8;
	h0=axes('position',[.05 .05 width height],'box','on','xcolor','k','ycolor','k',...
		        'fontname', 'times', 'fontsize', 10);
	set(gcf,'color', 'w')
	xmina=min([min(Ex),min(Fx),min(Bx),min(Cx),Dx, Gx, ...
	    min(bush1(:,1)), min(bush2(:,1)), min(bush3(:,1))]);
	xmaxa=max([max(Ex),max(Fx),max(Bx),max(Cx),Dx, Gx, ...
	    max(bush1(:,1)), max(bush2(:,1)), max(bush3(:,1))]);
	ymina=min([min(Ey),min(Fy),min(By),min(Cy),Dy, Gy, ...
	    min(bush1(:,2)), min(bush2(:,2)), min(bush3(:,2))]);
	ymaxa=max([max(Ey),max(Fy),max(By),max(Cy),Dy, Gy, ...
	    max(bush1(:,2)), max(bush2(:,2)), max(bush3(:,2))]);
	rangex=xmaxa-xmina;
	rangey=ymaxa-ymina;
	xmin=xmina-0.05*rangex;
	xmax=xmaxa+0.05*rangex;
	ymin=ymina-0.05*rangey;
	ymax=ymaxa+0.05*rangey;
	values=axisadjust(xmin, xmax, ymin, ymax, width, height);
	xmin=values(1);
	xmax=values(2);
	ymin=values(3);
	ymax=values(4);
	axis equal
	axis([xmin xmax ymin ymax]);
   
% Set up the vectors needed to draw the linkage

	joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       6, 'erasemode', 'xor', 'color', 'k');
	joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       6, 'erasemode', 'xor', 'color', 'k');
	joint3=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       6, 'erasemode', 'xor', 'color', 'k');
	joint4=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       6, 'erasemode', 'xor', 'color', 'k');
	crank=line('xdata', [0 0], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
	r5link=line('xdata', [0 0], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
	r6link=line('xdata', [0 0], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
	coupler=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
	couplerptr=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');	   
	couplerpt1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
	rocker=line('xdata', [r1*cos(Q1) 0], 'ydata' ,[r1*sin(Q1) 0], 'linewidth',2,...
       'erasemode', 'xor','color', 'k');
	bushy1=line('xdata', bush1(:,1), 'ydata',bush1(:,2), 'linestyle', '-',...
        'color', 'r');
	bushy2=line('xdata', bush2(:,1), 'ydata',bush2(:,2), 'linestyle', '-', ...
        'color', 'r');
	bushy3=line('xdata', bush3(:,1), 'ydata',bush3(:,2), 'linestyle', '-', ...
        'color', 'r');
	circ1=line('xdata', cir1(:,1), 'ydata',cir1(:,2), 'linestyle', '-',...
        'color', 'r');
	circ2=line('xdata', cir2(:,1), 'ydata',cir2(:,2), 'linestyle', '-', ...
        'color', 'r');
	circ3=line('xdata', cir3(:,1), 'ydata',cir3(:,2), 'linestyle', '-', ...
        'color', 'r');

% Plot the of the linakges alone

	ans='y';

% Draw the coupler curve first.

	set(couplerpt1,'xdata', Ex, 'ydata' ,Ey);

% Animate the linkage as long as ans = 'y'
	
	timesink=1;
	while ans=='y'
		for k=1:1:tt
			for i=1:1:itotal
				set(joint1,'xdata', Bx(i), 'ydata',By(i));
				set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
				set(joint3,'xdata', Ex(i), 'ydata',Ey(i));
				set(joint4,'xdata', Fx(i), 'ydata',Fy(i));
				set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
				set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
				set(rocker,'xdata', [Dx Cx(i)],'ydata', [Dy Cy(i)]);
				set(couplerptr,'xdata', [Bx(i) Ex(i) Cx(i)],...
				'ydata' ,[By(i) Ey(i) Cy(i)]);
				set(r5link,'xdata',[Ex(i) Fx(i)],'ydata', [Ey(i) Fy(i)]);
				set(r6link,'xdata',[Gx Fx(i)],'ydata', [Gy Fy(i)]);
				drawnow;   %flush the draw buffer

% Do meaningless calculation to slow down the linkage

				for k=1:1:timesink
					zzz=log10(k);
				end
			end
		end

% Plot linkage in fixed position.

		i = 3;
		set(joint1,'xdata', Bx(i), 'ydata',By(i));
		set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
		set(joint3,'xdata', Ex(i), 'ydata',Ey(i));
		set(joint4,'xdata', Fx(i), 'ydata',Fy(i));
		set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
		set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
		set(rocker,'xdata', [Dx Cx(i)],'ydata', [Dy Cy(i)]);
		set(couplerptr,'xdata', [Bx(i) Ex(i) Cx(i)],...
			'ydata' ,[By(i) Ey(i) Cy(i)]);
		set(r5link,'xdata',[Ex(i) Fx(i)],'ydata', [Ey(i) Fy(i)]);
		set(r6link,'xdata',[Gx Fx(i)],'ydata', [Gy Fy(i)]);
   
%flush the draw buffer

		drawnow; 
		
% Ask if animation should be repeated

		ans=input('repeat animation? y/n [y]: ','s');
		if isempty(ans);ans='y';end
		if ans=='y'
			rep=input('Change animation speed? (+/-/0) [-]:','s');
			if isempty(rep);rep='-';end
			if rep=='+'; timesink=timesink/10; end
			if rep=='-'; timesink=timesink*10; end
			if timesink<1; timesink=1; end			
		end
	end

% Pause the graphics before showing four plots per graph

	pause;
	clf;

% Draw four graphs per page.  First define the axes for the first graph 
% (4-bar rocker angle graph)

	width=0.23;
	height=0.21;
	h1=axes('position',[0.40 .41 width height],'box','on','xcolor','k','ycolor',...
      'k','fontname', 'times', 'fontsize', 8);   %axis location
	set(gcf,'color', 'w')
	xlabel('Crank Angle (deg)','color', 'k')
	ylabel('4-Bar Rocker Angle (deg)','color', 'k')
	Displ=line('xdata', [], 'ydata', [], 'erasemode', 'none',...
      'color', 'k', 'linestyle', '-');
	abead=line('xdata', [], 'ydata', [],'marker','o','markersize', 4, 'erasemode',...
      'xor');

% Define the axes for the second graph (dwell rocker angle graph)

	h2=axes('position', [0.40 .12 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k','fontname', 'times', 'fontsize', 8);
	set(gcf,'color', 'w')
	xlabel('Crank Angle (deg)','color', 'k')
	ylabel('Dwell Rocker Angle (deg)','color', 'k')
	Dwellp=line('xdata', [], 'ydata', [], 'erasemode', 'none', ...
      'color', 'k', 'linestyle', '-');
	pbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 4, ...
      'erasemode','xor');

% Define the axes for the third graph (dwell rocker angular velocity graph)

	h3=axes('position', [0.1 .12 width height],'box', 'on','xcolor', 'k','ycolor',...
      'k','fontname', 'times', 'fontsize', 8);
	set(gcf,'color', 'w')
	xlabel('Crank Angle (deg)','color', 'k')
	ylabel('Dwell Rocker \omega (rad/s)','color', 'k')
	Dwellv=line('xdata', [], 'ydata', [], 'erasemode','none', ...
      'color', 'k', 'linestyle', '-');
	vbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 4, 'erasemode','xor');

% Define the axes for the fourth graph (linkage position)

	h4=axes('position', [0.1 .41 width height],'box', 'on', 'xcolor', 'k','ycolor',...
       'k','fontname', 'times', 'fontsize', 8);
	set(gcf,'color', 'w')
	axis equal
	axis([xmin xmax ymin ymax]);
     
% Set up the vectors needed to draw the linkage

	joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       4, 'erasemode', 'xor', 'color', 'k');
	joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       4, 'erasemode', 'xor', 'color', 'k');
	joint3=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       4, 'erasemode', 'xor', 'color', 'k');
	joint4=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       4, 'erasemode', 'xor', 'color', 'k');
	crank=line('xdata', [0 0], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');
	r5link=line('xdata', [0 0], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');
	r6link=line('xdata', [0 0], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');
	coupler=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');
	couplerptr=line('xdata', [], 'ydata' ,[], 'linewidth' ,0.75, 'erasemode', ...
       'xor','color', 'k');	   
	couplerpt1=line('xdata', [], 'ydata' ,[], 'linewidth' ,0.5, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
	rocker=line('xdata', [r1*cos(Q1) 0], 'ydata' ,[r1*sin(Q1) 0], 'linewidth',1,...
       'erasemode', 'xor','color', 'k');
	bushy1=line('xdata', bush1(:,1), 'ydata',bush1(:,2), 'linestyle', '-', ...
        'color', 'r');
	bushy2=line('xdata', bush2(:,1), 'ydata',bush2(:,2), 'linestyle', '-', ...
        'color', 'r');
	bushy3=line('xdata', bush3(:,1), 'ydata',bush3(:,2), 'linestyle', '-', ...
        'color', 'r');
	circ1=line('xdata', cir1(:,1), 'ydata',cir1(:,2), 'linestyle', '-',...
        'color', 'r');
	circ2=line('xdata', cir2(:,1), 'ydata',cir2(:,2), 'linestyle', '-', ...
         'color', 'r');
	circ3=line('xdata', cir3(:,1), 'ydata',cir3(:,2), 'linestyle', '-', ...
         'color', 'r');

% Draw four plots on one graph.  Start by setting the axis limits for the graphs

	axes(h1);
	axis([0 360 min(Q44) max(Q44)]);  %set axis limits for the rocker angle plot
	axes(h2);
	axis([0 360 min(Q66) max(Q66)]);  %set axis limits for the velocity plot
	axes(h3);
	axis([0 360 min(w66) max(w66)]);  %set axis limits for the angular velocity plot

% Plot the results.

	ans='y';
	
% Draw the fixed curves first.

	set(couplerpt1,'xdata', Ex, 'ydata' ,Ey);
	set(Dwellp,'xdata', Q22,'ydata', Q66);
	set(Displ,'xdata', Q22,'ydata', Q44);
	set(Dwellv,'xdata', Q22,'ydata', w66);

% Animate the results as long as ans = 'y'
	
	timesink=1;
	while ans=='y'
		for k=1:1:tt
			for i=1:1:itotal
				set(joint1,'xdata', Bx(i), 'ydata',By(i));
				set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
				set(joint3,'xdata', Ex(i), 'ydata',Ey(i));
				set(joint4,'xdata', Fx(i), 'ydata',Fy(i));
				set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
				set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
				set(rocker,'xdata', [Dx Cx(i)],'ydata', [Dy Cy(i)]);
				set(couplerptr,'xdata', [Bx(i) Ex(i) Cx(i)],...
					'ydata' ,[By(i) Ey(i) Cy(i)]);
				set(r5link,'xdata',[Ex(i) Fx(i)],'ydata', [Ey(i) Fy(i)]);
				set(r6link,'xdata',[Gx Fx(i)],'ydata', [Gy Fy(i)]);
				set(abead, 'xdata', Q22(i), 'ydata', Q44(i));
				set(pbead, 'xdata', Q22(i), 'ydata', Q66(i));
				set(vbead, 'xdata', Q22(i), 'ydata', w66(i));
				drawnow;   %flush the draw buffer
				for m=1:1:timesink
					zzz=log10(m);
				end
			end
		end

% Plot figures in final position.

		i = 6;
		if i>itotal; i=itotal; end

% Plot position first

		set(joint1,'xdata', Bx(i), 'ydata',By(i));
		set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
		set(joint3,'xdata', Ex(i), 'ydata',Ey(i));
		set(joint4,'xdata', Fx(i), 'ydata',Fy(i));
		set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
		set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
		set(rocker,'xdata', [Dx Cx(i)],'ydata', [Dy Cy(i)]);
		set(couplerptr,'xdata', [Bx(i) Ex(i) Cx(i)],...
			'ydata' ,[By(i) Ey(i) Cy(i)]);
		set(r5link,'xdata',[Ex(i) Fx(i)],'ydata', [Ey(i) Fy(i)]);
		set(r6link,'xdata',[Gx Fx(i)],'ydata', [Gy Fy(i)]);
   
% Plot Torque information

		set(vbead, 'xdata', Q22(i), 'ydata', w66(i));
 
% Plot angle information

		set(abead, 'xdata', Q22(i), 'ydata', Q44(i));

% Plot velocity information

		set(pbead, 'xdata', Q22(i), 'ydata', Q66(i));

%flush the draw buffer

		drawnow; 

% Ask if the animation should be repeated

		ans=input('Repeat animation? y/n [y]: ', 's');
		if isempty(ans);
			ans='y';
		end
		if ans=='y'
			rep=input('Change animation speed? (+/-/0) [-]:','s');
			if isempty(rep);rep='-';end
			if rep=='+'; timesink=timesink/10; end
			if rep=='-'; timesink=timesink*10; end
			if timesink<1; timesink=1; end			
		end
	end
end



