% slidercrank_anal.m
% MATLAB program for analyzing slider-crank mechanisms

%Variables

%   r1=frame length
%   r2=crank length
%   r3=coupler length
%   r4=rocker length
%   cr1=coupler radius (to point D1 from the crank pin)
%   beta1=coupler angle (deg) (to point D2 from the rocker pin)
%   cpad=coupler angle (rad) (to point D2 from the rocker pin)
%   Q1=frame angle (deg)
%   Q11=frame angle (rad)
%   Q2=crank angle (rad)
%   Q21=crank angle (deg)
%   Q22=crank angle (deg) (vector)
%   Q3,Q33=coupler angle (deg)
%   Q4,Q44=rocker angle (deg)
%   w2=angular velocity of the crank (rad/sec)
%   w22=w2 (vector)
%   w33=angular velocity of the coupler (rad/sec)
%   w44=angular velocity of the rocker (rad/sec)
%   cang1=angle from the line connecting the coupler point one and the crank pin (rad)
%   cang2=angle from the line connecting the coupler point two and the rocker pin (rad)
%   mode,assem=assembly mode
%   tt=number of cycles to animate
%   dt=number of animation positions per revolution
%   itotal=number of animation positions
%   cycle=number of cycles as an integer
%   bush1,bush2=x,y coordinates of the bushings
%   cir1,cir2=x,y coordinates of the circles inside the bushings
%   Bx,By=x,y coordinates of the crank pin
%   Cx,Cy=x,y coordinates of the rocker pin
%   Bx2,By2=x,y coordinates of the crank pointer
%   Cx2,Cy2=x,y coordinates of the rocker pointer
%   d2r,r2d=conversion factors between radians and degrees
%   i=counting variable
%   ans=repitition variable

%Graphics variables
%   xmin,xmax,ymin,ymax=axes limits
%   h1=handle on the first axes in the window (crank angle vs. slider displacement)
%   h2=handle on the second axes in the window (crank angle vs. slider velocity)
%   h3=handle on the third axes in the window (crank angle vs. input torque)
%   h4=handle on the fourth axes in the window (linkage animation)
%   displ=line representing the crank angle vs. rocker angle
%   vel=line representing the crank angle vs. rocker velocity
%   dbead=bead which follows the crank angle/slider displacement throughout animation
%   vbead=bead which follows the crank angle/slider velocity throughout animation
%   tbead=bead which follows the crank angle/input torque throughout animation
%   bushy1=line representing the bushing at the crank hinge
%   circ1=line representing the circle inside the bushing at the crank hinge 
%   joint1=line representing the joint at the crank pin 
%   joint2=line representing the joint at the rocker pin
%   crank=line representing the crank 
%   coupler=line representing the coupler
%   couplerptr=line representing the coupler point
%   couplerpt1=line from the crank pin to the coupler point one

% Program  

clf;
clear all;
ans='y';
disp('   ')
disp('         Slider-Crank Analysis Program')

% Input values for linkage variables

readmode=input('Enter 1 for file input and 2 for interactive input [1]: ');
if isempty(readmode); readmode=1; end
inputfile=input('Enter input file name (slidercrkio.dat): ', 's');
if isempty(inputfile); inputfile = 'slidercrkio.dat'; end
if readmode==1
	fid = fopen(inputfile);
end
if readmode==2
	tt=input('Enter number of cycles (rev) [3]: ');
	if isempty(tt);tt = 3; end
	r4=input('Enter slider-line offset [1]: ');
	if isempty(r4); r4 = 1; end
	Q1=input('Enter the slider line angle (deg) [30]: ');
	if isempty(Q1);Q1 = 30;end
	r2=input('Enter the crank length [3]: ');
	if isempty(r2);r2 = 3;end
	r3=input('Enter the coupler length [8]: ');
	if isempty(r3); r3 = 8.5;end
	cr1=input('Enter coupler point radius [6]: ');
	if isempty(cr1);cr1=6; end	
	beta=input('Enter angle from coupler line to coupler point (deg) [20]: ');
	if isempty(beta);beta=20; end
	driver=input('Enter 1 for crank input, 2 for coupler input, and 3 for slider input [2]: ');
	if isempty(driver);driver=1;end
	mode=input('Enter the assembly mode (+1 or -1) [1]: ');
	if isempty(mode);mode = 1;end
	disp('    Flag to determine if both linkage modes are to be computed ');
	times	=input('Enter 1 for designated mode only and 2 for both modes [2]: ');
	if isempty(times);times=2;end
	if driver ==1;
		w2=input('Enter the angular velocity of crank(rad/sec) [5]: ');
		if isempty(w2); w2 = 5; end
	end
	if driver ==2;
		w3=input('Enter the angular velocity of coupler(rad/sec) [5]: ');
		if isempty(w3); w3 = 5; end
	end
	if driver ==3;
		r1d=input('Enter the linear velocity of slider(length/sec) [5]: ');
		if isempty(r1d); r1d = 5; end
	end	
	
% save the data to the file sliderio.dat
	
	temp1(1)=tt;
	temp1(2)=r2;
	temp1(3)=Q1;
	temp1(4)=r3;
	temp1(5)=r4;
	temp1(6)=cr1;
	temp1(7)=beta;
	temp1(8)=driver;	
	temp1(9)=mode;
	temp1(10)=times;
	if driver==1; temp1(11)=w2; end
	if driver==2; temp1(11)=w3; end
	if driver==3; temp1(11)=r1d; end
	temp1t=temp1(:);  % store the transpose of tempio for ease of reading	
	fprintf(inputfile,'%f \n',tt, r2, Q1, r3, r4, cr1, beta, driver, mode, times, temp1(11))
end

% Read input data from the file fourbario.dat

if readmode==1	
	sliderio = fscanf(fid,'%f');
	tt=sliderio(1);
	r2=sliderio(2);
	Q1=sliderio(3);
	r3=sliderio(4);
	r4=sliderio(5);
	cr1=sliderio(6);
	beta=sliderio(7);
	driver=sliderio(8);
	mode=sliderio(9);
	times=sliderio(10);
	if driver==1; w2=sliderio(11); end
	if driver==2; w3=sliderio(11); end
	if driver==3; r1d=sliderio(11); end
	fclose('all');
end

disp('I am working ...');
fprintf('slidercrank.dat', ' Q2    Q3   r1    w3     rd1     \n\n');

% Define constants

i=0;
d2r=pi/180;
r2d=180/pi;
Q11=Q1*d2r;
cpad=beta*d2r;
qstart=0;
qstop=360;

if driver==1; [qstart, qstop]=sc_angle_limits_cr(r2,r3,r4,Q1); end
if driver==2; [qstart, qstop]=sc_angle_limits_co(r2,r3,r4,Q1); end
if driver==3; [qstart, qstop]=sc_angle_limits_sl(r2,r3,r4,Q1); end

npos=100;
dq=(qstop-qstart)/(npos-1);
Astar=[0;0];
rpivot=0.15*r2;
delta=0.3*r2;
QM1=qstart;	
i2=0;

% Loop to analyze the linkage for tt full cycles

for mmm=1:1:times

% Analyze for two cycles in different modes if driver is 2 or 3

	if mmm ==2
		mode = -mode;
		dq=-dq;
	end
	for motion=1:1:npos
		if motion>1; QM1=QM1+dq; end
		i=i+1;
		QM = QM1*d2r;

% Call function to compute the position

		if driver ==1; para = sldcrkc(r2,r3,r4,QM1,w2,0,mode,Q1,2); end
		if driver ==2; para = sldcrkco(r2,r3,r4,QM1,w3,0,mode,Q1,2); end	
		if driver ==3; para = sldcrks(QM1,r2,r3,r4,r1d,0,mode,Q1,2); end	
	
		Q2=para(6);
		if Q2<0; Q2=Q2+360; end
		Q22(i)=Q2;
		w22(i)=para(14);
		Q3=para(7);
		Q33(i)=Q3;
		w33(i)=para(15);
		r1(i)=para(1);
		rd1(i)=para(9);
		assem=para(37);
		cang1=(Q3+beta)*d2r;

% Store position information in matrices.

		Bx(i)= para(25);
		By(i)= para(26);
		Cx(i)= para(27);
		Cy(i)= para(28);
		Dx1(i)=Bx(i)+cr1*cos(cang1);
		Dy1(i)=By(i)+cr1*sin(cang1);
			
% Determine coordinates for coupler curve for first mode
			
			if mmm==1
				Dx2(i)=Dx1(i);
				Dy2(i)=Dy1(i);
			end
			
% Determine coordinates for coupler curve for second mode
			
			if mmm==2
				i2=i2+1;
				Dx3(i2)=Dx1(i);
				Dy3(i2)=Dy1(i);
			end
		
% Locate the slider block coordinates

		xpin=Cx(i);
		ypin=Cy(i);
		height=delta;
		length=2*height;
		coords = rect(length,height,xpin,ypin,Q1,0);
		for j=1:1:5
			xblocka(i,j)=coords(j,1);
			yblocka(i,j)=coords(j,2);
		end

% Print results in a table

		fprintf('slidercrank.dat','%8.2f %8.2f %8.2f %8.2f %8.2f\n',...
		Q22(i),Q33(i),r1(i),w33(i),rd1(i)) 
	end
end
fclose('all');
itotal=i;

% Determine the coordinates of the slider line				

smax=max(r1);
smin=min(r1);
[cmin, index]=min(Cx);
cymin=Cy(index);
length1=smax-smin+length;
ndash=20;
csang=cos(Q11);
ssang=sin(Q11);	
xe = cmin-(0.5*length)*csang;
ye = cymin-(0.5*length)*ssang;
slflag=1;
coord=frameline(length1,xe,ye,ndash,Q1,slflag);					
npoints1=3*ndash;
					
% Offset slider line by half the height of the slider block.
					
for i=1:1:npoints1
	xsl1(i)=coord(i,1)+(0.6*height)*ssang;
	ysl1(i)=coord(i,2)-(0.6*height)*csang;
end					

% Define the axes for the graph showing the slider-crank mechanism

clf;
width=0.95;
height=0.95;
h0=axes('position',[.05 .05 width height],'box','on','xcolor','k','ycolor',...
       'k');
set(gcf,'color','w');
	   
% Define the pin and bushing coordinates
	
bush=bushing(r2/20,0,0,20,0);
cir1=circle2(r2/20,0,0,20);

% Find the limits for the plots

xmina=min([min(Dx1) min(Bx) min(Cx) min(bush(:,1)) ...
      min(xsl1) min(xblocka) min(coord(:,1))]);
xmaxa=max([max(Dx1) max(Bx) max(Cx) max(bush(:,1)) ...
      max(xsl1) max(xblocka) max(coord(:,1))]);
ymina=min([min(Dy1) min(By) min(Cy) min(bush(:,2)) ...
      min(ysl1) min(yblocka) min(coord(:,2))]);
ymaxa=max([max(Dy1) max(By) max(Cy) max(bush(:,2)) ...
      max(ysl1) max(yblocka) max(coord(:,2))]);
rangex=xmaxa-xmina;
rangey=ymaxa-ymina;
xmin=xmina-0.05*rangex;
xmax=xmaxa+0.05*rangex;
ymin=ymina-0.05*rangey;
ymax=ymaxa+0.05*rangey;
values=axisadjust(xmin, xmax, ymin, ymax, width, height);
xmin=values(1);
xmax=values(2);
ymin=values(3);
ymax=values(4);
axis equal
axis([xmin xmax ymin ymax]);
	    
% Set up the vectors needed to draw the linkage

joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       8, 'erasemode', 'xor', 'color', 'k');
joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       8, 'erasemode', 'xor', 'color', 'k');
gridpt=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','markersize',6,'color', 'k', 'marker', 'o');	
block=line('xdata', [], 'ydata', [], 'linewidth' ,2,'erasemode', ...
       'xor', 'color', 'k');
crank=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
coupler=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
couplerptr=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');	   
couplerpoint1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
couplerpoint2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
bush1=line('xdata', bush(:,1), 'ydata',bush(:,2), 'linestyle', '-', 'markersize', 8,...
        'color', 'r');
circ1=line('xdata', cir1(:,1), 'ydata',cir1(:,2), 'linestyle', '-', 'markersize', 8,...
        'color', 'r');
sliderline1=line('xdata', [], 'ydata',[], 'linestyle', '-', ...
        'color', 'r');

% Plot the of the linkage alone

ans='y';

% Draw the coupler curve first.

set(couplerpoint1,'xdata', Dx2, 'ydata' ,Dy2);
if times==2
	set(couplerpoint2,'xdata', Dx3, 'ydata' ,Dy3);
end

% Draw the frame line.

set(sliderline1, 'xdata', xsl1, 'ydata', ysl1);

% Animate the linkage as long as ans = 'y'
	
timesink=10;
while ans=='y'
	for j=1:1:tt
		for i=1:1:itotal
			set(joint1,'xdata', Bx(i), 'ydata',By(i));
			set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
			set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
			set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
			set(block,'xdata', [xblocka(i,:)],'ydata', [yblocka(i,:)]);
			set(couplerptr,'xdata', [Bx(i) Dx1(i) Cx(i)],...
				'ydata' ,[By(i) Dy1(i) Cy(i)]);
			set(gridpt,'xdata', Dx1(i), 'ydata',Dy1(i));
			drawnow;   %flush the draw buffer

% Do meaningless calculation to slow down the linkage

			for k=1:1:timesink
				zzz=log10(k);
			end
		end
	end

% Plot linkage in fixed position.

	i = 6;
	set(joint1,'xdata', Bx(i), 'ydata',By(i));
	set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
	set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
	set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
	set(gridpt,'xdata',Dx1(i), 'ydata' ,Dy1(i));
	set(block,'xdata', [xblocka(i,:)],'ydata', [yblocka(i,:)]);
	set(couplerptr,'xdata', [Bx(i) Dx1(i) Cx(i)], ...
		'ydata' ,[By(i) Dy1(i) Cy(i)]);
   
%flush the draw buffer

	drawnow; 
		
% Ask if animation should be repeated

	ans=input('Repeat animation? y/n [y]: ','s');
	if isempty(ans);ans='y';end
	if ans=='y'
		rep=input('Change animation speed? (+/-/0) [-]:','s');
		if isempty(rep);rep='-';end
		if rep=='+'; timesink=timesink/10; end
		if rep=='-'; timesink=timesink*10; end
		if timesink<1; timesink=1; end
	end
end

% Pause the graphics before showing four plots per graph

pause;
clf;

% Identify the variable that is the input

if driver==1;inputv=Q22; end
if driver==2;inputv=Q33; end
if driver==3;inputv=r1; end

% Define the axes for the first graph (input/output angle graph)

width=0.38;
height=0.35;
h1=axes('position', [0.6 .55 width height],'box', 'on','xcolor', 'k','ycolor',...
	'k');   %axis location
if driver==1;xlabel('crank angle','color', 'k'); end
if driver==2;xlabel('coupler angle','color', 'k'); end
if driver==3;xlabel('slider position','color', 'k'); end
if driver==1;ylabel('slider position','color', 'k'); end
if driver==2;ylabel('slider position','color', 'k'); end
if driver==3;ylabel('crank angle','color', 'k'); end
Displ4=line('xdata', [], 'ydata', [],'erasemode', 'none',...
	'color', 'r', 'linestyle', '-');
d4bead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, 'erase',...
      'xor');

% Define the axes for the second graph (slide velocity graph)

h2=axes('position', [0.6 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
	'k');
set(gcf,'color', 'w')
if driver==1;xlabel('crank angle','color', 'k'); end
if driver==2;xlabel('coupler angle','color', 'k'); end
if driver==3;xlabel('slider position','color', 'k'); end

if driver==1;ylabel('slider velocity','color', 'k'); end
if driver==2;ylabel('slider velocity','color', 'k'); end
if driver==3;ylabel('crank velocity','color', 'k'); end

Vel=line('xdata', [], 'ydata', [], 'erasemode', 'none', ...
	'color', 'r', 'linestyle', '-');
vbead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, ...
	'erase','xor');

% Define the axes for the third graph (coupler angle graph)

h3=axes('position', [0.1 .1 width height],'box', 'on','xcolor', 'k','ycolor',...
	'k');
set(gcf,'color', 'w')
if driver==1;xlabel('crank angle','color', 'k'); end
if driver==2;xlabel('coupler angle','color', 'k'); end
if driver==3;xlabel('slider position','color', 'k'); end
ylabel('coupler angle','color', 'k')
Displ3=line('xdata', [], 'ydata', [], 'erasemode','none', ...
	'color', 'r', 'linestyle', '-');
d3bead=line('xdata', [], 'ydata', [],'marker','o','markersize', 6, 'erase','xor');

% Define the axes for the fourth graph (linkage position)

h4=axes('position', [0.1 .55 width height],'box', 'on', 'xcolor', 'k','ycolor',...
	'k');
set(gcf,'color', 'w')
axis equal
axis([xmin xmax ymin ymax]);
     
% Set up the vectors needed to draw the linkage

joint1=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       7, 'erasemode', 'xor', 'color', 'k');
joint2=line('xdata', [], 'ydata', [], 'marker', 'o','markersize', ...
       7, 'erasemode', 'xor', 'color', 'k');
gridpt=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','markersize',6,'color', 'k', 'marker', 'o');	
block=line('xdata', [], 'ydata', [], 'linewidth' ,2,'erasemode', ...
       'xor', 'color', 'k');
crank=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
coupler=line('xdata', [], 'ydata' ,[], 'linewidth' ,2, 'erasemode', ...
       'xor','color', 'k');
couplerptr=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'xor','color', 'k');	   
couplerpoint1=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
couplerpoint2=line('xdata', [], 'ydata' ,[], 'linewidth' ,1, 'erasemode', ...
       'none','color', 'r', 'linestyle', '--');	
bush1=line('xdata', bush(:,1), 'ydata',bush(:,2), 'linestyle', '-', 'markersize', 8,...
        'color', 'r');
circ1=line('xdata', cir1(:,1), 'ydata',cir1(:,2), 'linestyle', '-', 'markersize', 8,...
        'color', 'r');
sliderline1=line('xdata', [], 'ydata',[], 'linestyle', '-', ...
        'color', 'r');

% Draw four plots on one graph.  Start by setting the axis limits for the graphs

axes(h3);
axis([min(inputv) max(inputv) min(Q33) max(Q33)]);  %set axis limits for the coupler angle plot

% find the limits when the crank is the driver

if driver == 1|driver == 2
	rmin=min(r1);
	rmax=max(r1);
end

% find the limits when the slider is the driver

if driver == 3
	rmin=min(Q22);
	rmax=max(Q22);
end

if rmax <=rmin; rmax = rmin+0.001; end
axes(h1);
axis([min(inputv) max(inputv) rmin rmax]);  %set axis limits for the slider plot

if driver == 1|driver == 2
	rdmin=min(rd1);
	rdmax=max(rd1);
end
if driver == 3
	rdmin=min(w22);
	rdmax=max(w22);
end

if rdmax <=rdmin; rdmax = rdmin+0.001; end
axes(h2);
axis([min(inputv) max(inputv) rdmin rdmax]);  %set axis limits for the slider velocity plot

% Plot the results.

ans='y';
	
% Draw the fixed curves first.

set(couplerpoint1,'xdata', Dx2, 'ydata' ,Dy2);
if times==2
	set(couplerpoint2,'xdata', Dx3, 'ydata' ,Dy3);
end
set(sliderline1, 'xdata', xsl1, 'ydata', ysl1);
set(Displ3,'xdata', inputv,'ydata', Q33);

if driver==1;set(Displ4,'xdata', inputv,'ydata', r1); end
if driver==2;set(Displ4,'xdata', inputv,'ydata', r1); end
if driver==3;set(Displ4,'xdata', inputv,'ydata', Q22); end

if driver==1;set(Vel, 'xdata', inputv, 'ydata', rd1); end
if driver==2;set(Vel, 'xdata', inputv, 'ydata', rd1); end
if driver==3;set(Vel, 'xdata', inputv, 'ydata',w22); end

% Animate the results as long as ans = 'y'

while ans=='y'
	for k=1:1:tt
		for i=1:1:itotal
			set(joint1,'xdata', Bx(i), 'ydata',By(i));
			set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
			set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
			set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
			set(block,'xdata', [xblocka(i,:)],'ydata', [yblocka(i,:)]);
			set(couplerptr,'xdata', [Bx(i) Dx1(i) Cx(i)],...
				'ydata' ,[By(i) Dy1(i) Cy(i)]);
			set(gridpt,'xdata', Dx1(i), 'ydata',Dy1(i));
			drawnow;   %flush the draw buffer
			set(d3bead, 'xdata', inputv(i), 'ydata', Q33(i));
			if driver==1;set(d4bead, 'xdata', inputv(i), 'ydata', r1(i)); end
			if driver==2;set(d4bead, 'xdata', inputv(i), 'ydata', r1(i)); end
			if driver==3;set(d4bead, 'xdata', inputv(i), 'ydata', Q22(i)); end
			
			if driver==1;set(vbead, 'xdata', inputv(i), 'ydata', rd1(i)); end
			if driver==2;set(vbead, 'xdata', inputv(i), 'ydata', rd1(i)); end
			if driver==3;set(vbead, 'xdata', inputv(i), 'ydata',w22(i)); end
			drawnow;   %flush the draw buffer
		end
	end

% Plot figures in final position.

	i = floor(itotal/3);

% Plot position first

	set(joint1,'xdata', Bx(i), 'ydata',By(i));
	set(joint2,'xdata', Cx(i), 'ydata',Cy(i));
	set(crank,'xdata',[0 Bx(i)],'ydata', [0 By(i)]);
	set(coupler,'xdata', [Bx(i) Cx(i)],'ydata', [By(i) Cy(i)]);
	set(block,'xdata', [xblocka(i,:)],'ydata', [yblocka(i,:)]);
	set(couplerptr,'xdata', [Bx(i) Dx1(i) Cx(i)],...
			'ydata' ,[By(i) Dy1(i) Cy(i)]);
	set(gridpt,'xdata', Dx1(i), 'ydata',Dy1(i));
   
% Plot Torque information

	set(d3bead, 'xdata', inputv(i), 'ydata', Q33(i));
 
% Plot angle information

	if driver==1;set(d4bead, 'xdata', inputv(i), 'ydata', r1(i)); end
	if driver==2;set(d4bead, 'xdata', inputv(i), 'ydata', r1(i)); end
	if driver==3;set(d4bead, 'xdata', inputv(i), 'ydata', Q22(i)); end

% Plot velocity information

	if driver==1;set(vbead, 'xdata', inputv(i), 'ydata', rd1(i)); end
	if driver==2;set(vbead, 'xdata', inputv(i), 'ydata', rd1(i)); end
	if driver==3;set(vbead, 'xdata', inputv(i), 'ydata',w22(i)); end

%flush the draw buffer

	drawnow; 

% Ask if the animation should be repeated

	ans=input('Repeat animation? y/n [y]: ', 's');
	if isempty(ans);
		ans='y';
	end
end



